unit DEngine;

interface

Uses
  DDebug,
  DUtils,
  DCore;

Type
TCoreEngine = class(TEntity)
private
  UpdTimer: TTimer;
  FpsTimer: TTimer;
  Exiting: Boolean;
  FPSCounter: Integer;
  FPS: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure StartMainLoop(Entity: TEntity; UpdInterval: Integer);
  Procedure StopMainLoop;
  Function GetFPS: Integer;
end;

implementation

//
//  TCoreEngine
//
Constructor TCoreEngine.Create;
begin
  Inherited Create(Owner);
  UpdTimer := TTimer.Create(30);
  FpsTimer := TTimer.Create(1000);
end;

Destructor TCoreEngine.Destroy;
begin
  UpdTimer.Free;
  FpsTimer.Free;
  Inherited Destroy;
end;

Procedure TCoreEngine.StartMainLoop(Entity: TEntity; UpdInterval: Integer);
  Var
    Flag: Boolean;
begin
  Exiting := False;
  UpdTimer.SetInterval(UpdInterval);
  UpdTimer.Reset;
  FpsTimer.Reset;
  FPS := 0;
  FPSCounter := 0;
  Entity.Update;
    While Not Exiting do begin
      Flag := False;
        While UpdTimer.Pop do begin
          Entity.Update;
          Flag := True;
        end;
        If Flag then
          Entity.Render;
      Inc(FPSCounter);
        If FPSTimer.Pop then begin
          FPS := FPSCounter;
          FPSCounter := 0;
        end;
    end;
end;

Procedure TCoreEngine.StopMainLoop;
begin
  Exiting := True;
end;

Function TCoreEngine.GetFPS;
begin
  Result := FPS;
end;

end.
