//      ;)
unit DMath;

interface

Uses
  Math, SysUtils, DUtils, DDebug, dglOpenGL;

Const
  INF = 1000000;
  EPS = 10e-6;
  cNo = 0;
  cOnLine = 1;
  cInFront = 2;
  cInBack = 3;
  cInPolygon = 4;
  cOutPolygon = 5;
  cOnVertex = 6;
  DegToRad = pi / 180;

Function Min(A, B: Single): Single;
Function Max(A, B: Single): Single;
// X in [A, B]
Function ValueAt(X, A, B: Double): Boolean;
Function EqualSingle(A, B: Single; EPS: Single = 0.001): Boolean;

Function Sign(A: Double): Integer;

Type
  TVec2D = packed record
    X, Y: Single;
  end;
  TArrayOfVec2D = Array Of TVec2D;

Function Vec2D(X, Y: Double): TVec2D;
Function Add2D(A, B: TVec2D): TVec2D;
Function Sub2D(A, B: TVec2D): TVec2D;
Function Dot2D(A, B: TVec2D): Double;
Function Det2D(A, B: TVec2D): Double;
Function Mul2D(V: TVec2D; D: Double): TVec2D;
Function QLen2D(V: TVec2D): Double;
Function Len2D(V: TVec2D): Double;
Function Dist2D(A, B: TVec2D): Double;
Function Norm2D(V: TVec2D): TVec2D;
Function Equal2D(A, B: TVec2D; EPS: Single): Boolean;

Type
  TRect2D = packed record
    A, B: TVec2D;
  end;

Function Rect2D(A, B: TVec2D): TRect2D;
Function VecInRect(R: TRect2D; V: TVec2D): Boolean;

Type
  TLine2D = packed record
    N: TVec2D;
    D: Double;
  end;

Function Line2D(A, B: TVec2D): TLine2D;
Function LineD2D(S, D: TVec2D): TLine2D;
Function MirrorLine(L: TLine2D): TLine2D;
Function DistToLine(L: TLine2D; V: TVec2D): Double;
Function ClassifyLine(L: TLine2D; V: TVec2D): Integer;
Function CollideLineWithLine(A, B: TLine2D; Var V: TVec2D): Boolean;
Function CollideRestWithRest(A, B, C, D: TVec2D; Var V: TVec2D): Boolean;
Function CollideLineWithRest(L: TLine2D; A, B: TVec2D; Var V: TVec2D): Boolean;

Type
TPolygon2D = class
  P: TArrayOfVec2D;
  Constructor Create; overload;
  Constructor Create(A, B, C: TVec2D); overload;
  Destructor Destroy; override;
  Function Copy: TPolygon2D;
  Function CollideWithHalfPlane(H: TLine2D): TPolygon2D;
  Function Collide(A: TPolygon2D): TPolygon2D;
  Function ClassifyVec(V: TVec2D): Integer;
  Function CollideRest(A, B: TVec2D; Var V: Tvec2D): Boolean;
  Function GetSqr: Double;
  Procedure SetLength(Len: Integer);
  Function Length: Integer;
  Function High: Integer;
  Procedure AddVec(V: TVec2D);
  Function GetItem(ID: Integer): TVec2D;
  Procedure SetItem(ID: Integer; V: TVec2D);
  Property Items[Index: Integer]: TVec2D read GetItem write SetItem; default;
end;

PVec3D = ^TVec3D;
TVec3D = packed record
  X, Y, Z: Single;
end;

Function Vec3D(X, Y, Z: Single): TVec3D;
Function Add3D(A, B: TVec3D): TVec3D;
Function Sub3D(A, B: TVec3D): TVec3D;
Function Inv3D(A: TVec3D): TVec3D;
Function Mul3D(V: TVec3D; S: Single): TVec3D;
Function Cross3D(A, B: TVec3D): TVec3D;
Function Dot3D(A, B: TVec3D): Single;
Function QLen3D(V: TVec3D): Single;
Function Len3D(V: TVec3D): Single;
Function Dist3D(A, B: TVec3D): Single;
Function Norm3D(V: TVec3D): TVec3D;
Function Rand3D(size: single): TVec3D;
Function SignedRand3D(size: single): TVec3D;
Function Equal3D(A, B: TVec3D; EPS: Single): Boolean;
Function Lerp3D(A, B: TVec3D; T: Single): TVec3D;

Type
TVec4D = packed record
  X, Y, Z, W: Single;
end;

Function Vec4D(X, Y, Z, W: Single): TVec4D;
Function Add4D(A, B: TVec4D): TVec4D;
Function sub4D(a,b:TVec4D):TVec4D;
Function Mul4D(V: TVec4D; D: Double): TVec4D;
Function Len4D(V: TVec4D): Single;
Function Norm4D(V: TVec4D): TVec4D;
Function Lerp4D(V1, V2: TVec4D; T: Single): TVec4D;

//
//  Quaternion
//
Type
TQuaternion = TVec4D;

Function CreateQuaternion(X, Y, Z, W: Single): TQuaternion;
Function CreateQuaternionFromYawPitchRoll(Yaw, Pitch, Roll: Single): TQuaternion;
Function CreateQuaternionFromVectorAngle(V: TVec3D; Angle: Single): TQuaternion;
Function MultiplyQuaternion(Q1, Q2: TQuaternion): TQuaternion;
Function NormalizeQuaternion(Q: TQuaternion): TQuaternion;
Function LerpQuaternion(Q1, Q2: TQuaternion; T: Single): TQuaternion;
Function SlerpQuaternion(Q1, Q2: TQuaternion; T: Single): TQuaternion;
Function InvertQuaternion(Q: TQuaternion): TQuaternion;
Function ApplyQuaternion(Q: TQuaternion; V: TVec3D): TVec3D;

//
//  Frame
//
Type
TFrame = packed record
  Pos: TVec3D;
  Quat: TQuaternion;
end;

Function CreateFrame(Pos: TVec3D; Quat: TQuaternion): TFrame;
Function InterpolateFrame(Frame1, Frame2: TFrame; T: Single): TFrame;
Function MultiplyFrame(Frame1, Frame2: TFrame): TFrame;

Type
TFrameList = class
  Frames: Array Of TFrame;
  Constructor Create(Data: TData);
  Function GetFrameAt(Start, Finish: Integer; T: Single): TFrame;
  Procedure SetItem(Index: Integer; Item: TFrame);
  Function GetItem(Index: Integer): TFrame;
  Property Items[Index: Integer]: TFrame read GetItem write SetItem; default;
end;

//
//  Matrix 4x4
//
Procedure Matrix_LoadIdentity(Var M: TArray16OfSingle);
Procedure Matrix_Rotate(Var M: TArray16OfSingle; Angle: Single; V: TVec3D);
Procedure Matrix_Translate(Var M: TArray16OfSingle; V: TVec3D);
Procedure Matrix_Scale(Var M: TArray16OfSingle; Scaler: TVec3D);
Procedure Matrix_Multiply(Var Dest: TArray16OfSingle; Src1, Src2: TArray16OfSingle);
Function  Matrix_GetDet(Const M: TArray16OfSingle): Single;
Procedure Matrix_Inverse(Var Dest: TArray16OfSingle; Src: TArray16OfSingle);
Procedure Matrix_SetSystem(Var M: TArray16OfSingle; Pos, Up, Right, Frwrd: TVec3D);
Procedure Matrix_Frame(Var M: TArray16OfSingle; Frame: TFrame);

Type
TMatrix = class
private
  M: TArray16OfSingle;
  //State: TMatrixState;
public
  Constructor Create(V00, V10, V20, V30,
                     V01, V11, V21, V31,
                     V02, V12, V22, V32,
                     V03, V13, V23, V33: Single); overload;
  //
  Constructor Create; overload;
  //
  Procedure LoadIdentity;
  // Self := Matrix
  Procedure Copy(Matrix: TMatrix);
  //
  Procedure Multiply(Matrix: TArray16OfSingle); overload;
  // Self := Self * Matrix
  Procedure Multiply(Matrix: TMatrix); overload;
  // Self := Self + V
  Procedure Translate(V: TVec3D);
  //
  Procedure RotateX(Angle: Single);
  //
  Procedure RotateY(Angle: Single);
  //
  Procedure RotateZ(Angle: Single);
  // Self := Self * (Rotation Angle V)
  Procedure Rotate(Angle: Single; V: TVec3D);
  // Self := Self * Scale
  Procedure Scale(Scaler: TVec3D);
  //
  Procedure LookAt(Pos, Target: TVec3D; Roll: Single = 0.0);
  //
  Procedure base(ord, right, up, target: TVec3D);
  //
  Procedure SetFrame(Frame: TFrame);
  //
  Procedure MulFrame(Frame: TFrame);
  //    OpenGL.
  Procedure LoadToOpenGL;
  //     OpenGL. 
  Procedure LoadFromOpenGL(MatrixMode: Cardinal = GL_MODELVIEW_MATRIX);
  //   Self      OpenGL.
  Procedure MulToOpenGL;
  //
  Procedure MulFromOpenGL;
  // 
  Procedure Inverse;
  //   
  Function CreateInverse: TMatrix;
  //    
  Function Mul(V: TVec3D): TVec3D;
  //      .
  Function GetCellValue(X, Y: Byte): Single;
  //      .
  Procedure SetCellValue(X, Y: Byte; Value: Single);
  //  
  Procedure PrintToLog(Name: String);
  Property Cells[Index1, Index2: Byte]: Single read GetCellValue write SetCellValue; default;
end;

// Bits
Function IncToPOT(Value: Cardinal): Cardinal;

implementation

Function Min(A, B: Single): Single;
begin
    If A < B then
      Result := A
    else
      Result := B;
end;

Function Max(A, B: Single): Single;
begin
    If A < B then
      Result := B
    else
      Result := A;
end;

Function Sign(A: Double): Integer;
begin
    If A < 0 then
      Result := -1
    else
      Result := 1; 
end;

Function ValueAt(X, A, B: Double): Boolean;
begin
  Result := (A <= X)And(X <= B);
end;

Function EqualSingle;
begin
  Result := Abs(A - B) < EPS;
end;

Function Vec2D(X, Y: Double): TVec2D;
begin
  Result.X := X;
  Result.Y := Y;
end;

Function Add2D(A, B: TVec2D): TVec2D;
begin
  Result.X := A.X + B.X;
  Result.Y := A.Y + B.Y;
end;

Function Sub2D(A, B: TVec2D): TVec2D;
begin
  Result.X := A.X - B.X;
  Result.Y := A.Y - B.Y;
end;

Function Dot2D(A, B: TVec2D): Double;
begin
  Result := A.X*B.X + A.Y*B.Y;
end;

Function Det2D(A, B: TVec2D): Double;
begin
  Result := A.X*B.Y - B.X*A.Y;
end;

Function Mul2D(V: TVec2D; D: Double): TVec2D;
begin
  Result.X := V.X * D;
  Result.Y := V.Y * D;
end;

Function QLen2D(V: TVec2D): Double;
begin
  Result := Sqr(V.X) + Sqr(V.Y);
end;

Function Len2D(V: TVec2D): Double;
begin
  Result := Sqrt(QLen2D(V));
end;

Function Dist2D(A, B: TVec2D): Double;
begin
  Result := Len2D(Sub2D(B, A));
end;

Function Norm2D(V: TVec2D): TVec2D;
  Var
    L: Double;
begin
  L := Len2D(V);
    If L < EPS then
      Result := Vec2D(0, 0)
    else
      Result := Mul2D(V, 1/L);
end;

Function Equal2D;
begin
  Result := EqualSingle(A.X, B.X, EPS) And
            EqualSingle(A.Y, B.Y, EPS);
end;

//
//  Line2D
//
Function Line2D(A, B: TVec2D): TLine2D;
begin
  Result.N.X := A.Y - B.Y;
  Result.N.Y := B.X - A.X;
  Result.N := Norm2D(Result.N);
  Result.D := - A.X * Result.N.X - A.Y * Result.N.Y;
end;

Function LineD2D(S, D: TVec2D): TLine2D;
begin
  Result.N := Vec2D(-D.Y, D.X);
  //Result.N := Norm2D(Result.N);
  Result.D := - S.X * Result.N.X - S.Y * Result.N.Y;
end;

Function MirrorLine(L: TLine2D): TLine2D;
begin
  Result.N.X := - L.N.X;
  Result.N.Y := - L.N.Y;
  Result.D := - L.D;
end;

Function DistToLine(L: TLine2D; V: TVec2D): Double;
begin
  Result := L.N.X*V.X + L.N.Y*V.Y + L.D;
end;

Function ClassifyLine(L: TLine2D; V: TVec2D): Integer;
  Var
    D: Double;
begin
  D := DistToLine(L, V);
    If D > EPS then
      Result := cInFront
    else If D < -EPS then
      Result := cInBack
    else
      Result := cOnLine;
end;

Function Rect2D(A, B: TVec2D): TRect2D;
begin
  Result.A.X := Min(A.X, B.X);
  Result.A.Y := Min(A.Y, B.Y);
  Result.B.X := Max(A.X, B.X);
  Result.B.Y := Max(A.Y, B.Y);
end;

Function VecInRect(R: TRect2D; V: TVec2D): Boolean;
begin
  Result := ValueAt(V.X, R.A.X, R.B.X) And ValueAt(V.Y, R.A.Y, R.B.Y);
end;

Function CollideLineWithLine(A, B: TLine2D; Var V: TVec2D): Boolean;
  Var
    Z: Double;
begin
  Result := False;
  Z := A.N.X*B.N.Y - A.N.Y*B.N.X;
    If Abs(Z) < EPS then
      Exit;
  V.X := (A.N.Y*B.D - A.D*B.N.Y)/Z;
  V.Y := (A.D*B.N.X - A.N.X*B.D)/Z;
  Result := True;
end;

Function CollideRestWithRest(A, B, C, D: TVec2D; Var V: TVec2D): Boolean;
begin
  Result := CollideLineWithLine(Line2D(A, B), Line2D(C, D), V);
  Result := Result And VecInRect(Rect2D(A, B), V) And VecInRect(Rect2D(C, D), V);
end;

Function CollideLineWithRest(L: TLine2D; A, B: TVec2D; Var V: TVec2D): Boolean;
begin
  Result := CollideLineWithLine(L, Line2D(A, B), V);
  Result := Result And VecInRect(Rect2D(A, B), V);
end;

Constructor TPolygon2D.Create;
begin
  SetLength(0);
end;

Constructor TPolygon2D.Create(A, B, C: TVec2D);
begin
  SetLength(3);
  P[0] := A;
  P[1] := B;
  P[2] := C;
    If GetSqr < 0 then begin
      P[0] := C;
      P[2] := A;
    end;
end;

Destructor TPolygon2D.Destroy;
begin
  SetLength(0);
end;

Function TPolygon2D.Copy: TPolygon2D;
  Var
    I: Integer;
begin
  Result := TPolygon2D.Create;
  Result.SetLength(Length);
    For I := 0 to High do
      Result[I] := P[I];
end;


Function TPolygon2D.CollideWithHalfPlane(H: TLine2D): TPolygon2D;
  Var
    I: Integer;
    S, F, V: TVec2D;
begin
  Result := TPolygon2D.Create;
    For I := 0 to High do begin
      S := P[I];
      F := P[(I+1) mod Length];
        If CollideLineWithRest(H, S, F, V) then
          Result.AddVec(V);
        If ClassifyLine(H, F) = cInFront then
          Result.AddVec(F);
    end;
end;

Function TPolygon2D.Collide(A: TPolygon2D): TPolygon2D;
  Var
    P: TPolygon2D;
    I: Integer;
    S, F: TVec2D;
begin
  P := Copy;
    For I := 0 to A.High do begin
      S := A[I];
      F := A[(I+1) mod Length];
      Result := P.CollideWithHalfPlane(Line2D(S, F));
      P.Free;
      P := Result;
    end;
end;
      
Function TPolygon2D.ClassifyVec(V: TVec2D): Integer;
  Var
    I: Integer;
begin
  Result := cInPolygon;
    For I := 0 to High do
      Case ClassifyLine(Line2D(P[I], P[(I+1) mod Length]), V) of
        cInBack:
          begin
            Result := cOutPolygon;
            Exit;
          end;
        cOnLine:
          begin
            Result := cOnLine;
            Exit;
          end;
      end;
end;

Function TPolygon2D.CollideRest(A, B: TVec2D; Var V: Tvec2D): Boolean;
  Var
    I: Integer;
begin
  Result := False;
    For I := 0 to High do
      If CollideRestWithRest(A, B, P[I], P[(I+1) mod Length], V) then begin
        Result := True;
        Exit;
      end;
end;

Function TPolygon2D.GetSqr: Double;
  Var
    I: Integer;
begin
  Result := 0;
    For I := 0 to High do
      Result := Result + (P[(I+1) mod Length].Y + P[I].Y)*(P[I].X - P[(I+1) mod Length].X) / 2;
end;

Procedure TPolygon2D.SetLength(Len: Integer);
begin
  System.SetLength(P, Len);
end;

Function TPolygon2D.Length: Integer;
begin
  Result := System.Length(P);
end;

Function TPolygon2D.High: Integer;
begin
  Result := System.High(P);
end;

Procedure TPolygon2D.AddVec(V: TVec2D);
begin
  SetLength(Length + 1);
  P[High] := V;
end;

Function TPolygon2D.GetItem(ID: Integer): TVec2D;
begin
  Result := P[ID];
end;

Procedure TPolygon2D.SetItem(ID: Integer; V: TVec2D);
begin
  P[ID] := V;
end;

Function Vec3D(X, Y, Z: Single): TVec3D;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
end;

Function Add3D(A, B: Tvec3D): TVec3D;
begin
  Result.X := A.X + B.X;
  Result.Y := A.Y + B.Y;
  Result.Z := A.Z + B.Z;
end;

Function Sub3D(A, B: TVec3D): TVec3D;
begin
  Result.X := A.X - B.X;
  Result.Y := A.Y - B.Y;
  Result.Z := A.Z - B.Z;
end;

Function Inv3D(A: TVec3D): TVec3D;
begin
  Result.X := -A.X;
  Result.Y := -A.Y;
  Result.Z := -A.Z;
end;

Function Mul3D(V: TVec3D; S: Single): TVec3D;
begin
  Result.X := V.X * S;
  Result.Y := V.Y * S;
  Result.Z := V.Z * S;
end;

Function Cross3D(A, B: TVec3D): TVec3D;
begin
  Result.X := A.Y * B.Z - A.Z * B.Y;
  Result.Y := A.Z * B.X - A.X * B.Z;
  Result.Z := A.X * B.Y - A.Y * B.X;
end;

Function Dot3D(A, B: TVec3D): Single;
begin
  Result := A.X*B.X + A.Y*B.Y + A.Z*B.Z;
end;

Function QLen3D(V: TVec3D): Single;
begin
  Result := Sqr(V.X) + Sqr(V.Y) + Sqr(V.Z);
end;

Function Len3D(V: TVec3D): Single;
begin
  Result := Sqrt(QLen3D(V));
end;

Function Dist3D(A, B: TVec3D): Single;
begin
  Result := Len3D(Sub3D(A, B));
end;

Function Norm3D(V: TVec3D): TVec3D;
  Var
    L: Single;
begin
  L := Len3D(V);
    If Abs(L) < EPS then
      Result := Vec3D(0, 0, 0)
    else
      Result := Mul3D(V, 1/L);
end;

Function rand3D(size: single): TVec3D;
begin
  result := mul3D(norm3D(vec3D(random, random, random)), size);
end;

Function signedRand3D(size: single): TVec3D;
begin
  result := mul3D(norm3D(vec3D(random - 0.5, random - 0.5, random - 0.5)), size * 2);
end;

Function Equal3D;
begin
  Result := EqualSingle(A.X, B.X, EPS) And
            EqualSingle(A.Y, B.Y, EPS) And
            EqualSingle(A.Z, B.Z, EPS);
end;

Function Lerp3D;
begin
  Result := Add3D(Mul3D(A, (1 - T)), Mul3D(B, T));
end;

//
//  TVec4D
//
Function Vec4D(X, Y, Z, W: Single): TVec4D;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
  Result.W := W;
end;

Function Add4D(A, B: TVec4D): TVec4D;
begin
  Result.X := A.X + B.X;
  Result.Y := A.Y + B.Y;
  Result.Z := A.Z + B.Z;
  Result.W := A.W + B.W;
end;

Function sub4D(a,b:TVec4D):TVec4D;
begin
  result.x := a.x - b.x;
  result.y := a.y - b.y;
  result.z := a.z - b.z;
  result.w := a.w - b.w;
end;

Function Mul4D(V: TVec4D; D: Double): TVec4D;
begin
  Result.X := V.X * D;
  Result.Y := V.Y * D;
  Result.Z := V.Z * D;
  Result.W := V.W * D;
end;

Function Len4D;
begin
  Result := Sqrt(V.X * V.X + V.Y * V.Y + V.Z * V.Z + V.W + V.W);
end;

Function Norm4D;
begin
  Result := Mul4D(V, 1 / Len4D(V));
end;

Function Lerp4D;
begin
  Result := Add4D(Mul4D(V1, 1 - T), Mul4D(V2, T));
end;

//
//  Quaternion
//
Function CreateQuaternion;
begin
  Result := Vec4D(X, Y, Z, W);
end;

Function CreateQuaternionFromYawPitchRoll;
  Var
    QRoll, QPitch, QYaw: TQuaternion;
begin
  QRoll  := CreateQuaternion(Sin(Roll  / 2), 0, 0, Cos(Roll / 2));
  QPitch := CreateQuaternion(0, Sin(Pitch / 2), 0, Cos(Pitch / 2));
  QYaw   := CreateQuaternion(0, 0, Sin(Yaw / 2), Cos(Yaw   / 2));
  Result := MultiplyQuaternion(QYaw, MultiplyQuaternion(QPitch, QRoll));
end;

Function CreateQuaternionFromVectorAngle;
  Var
    Vec: TVec3D;
begin
  Vec := Mul3D(V, Sin(Angle * DegToRad / 2));
  Result.X := Vec.X;
  Result.Y := Vec.Y;
  Result.Z := Vec.Z;
  Result.W := Cos(Angle * DegToRad / 2);
  Result := NormalizeQuaternion(Result);
end;

Function MultiplyQuaternion;
  Var
    V, V1, V2: TVec3D;
begin
  V1 := Vec3D(Q1.X, Q1.Y, Q1.Z);
  V2 := Vec3D(Q2.X, Q2.Y, Q2.Z);
  V := Add3D(Cross3D(V1, V2), Add3D(Mul3D(V1, Q2.W), Mul3D(V2, Q1.W)));
  Result.X := V.X;
  Result.Y := V.Y;
  Result.Z := V.Z;
  Result.W := Q1.W * Q2.W - Dot3D(V1, V2);
  //Result := NormalizeQuaternion(Result);
end;

Function NormalizeQuaternion;
begin
  Result := Norm4D(Q);
end;

Function LerpQuaternion;
begin
  Result := Lerp4D(Q1, Q2, T);
end;

Function SlerpQuaternion;
  Var
    CosOm, Omega, SinOm, Scale1, Scale2: Single;
begin
  CosOm := Q1.X * Q2.X + Q1.Y * Q2.Y + Q1.Z * Q2.Z + Q1.W * Q2.W;
    If CosOm < 0 then begin
      CosOm := - CosOm;
      Q2 := Mul4D(Q2, -1);
    end;
    If CosOm < 0.9999 then begin
      Omega := ArcCos(CosOm);
      SinOm := Sin(Omega);
      Scale1 := Sin((1 - T) * Omega) / SinOm;
      Scale2 := Sin(T * Omega) / SinOm;
      Result := Add4D(Mul4D(Q1, Scale1), Mul4D(Q2, Scale2));
    end else
      Result := Lerp4D(Q1, Q2, T);
end;

Function InvertQuaternion;
begin
  Result := NormalizeQuaternion(Q);
  Result.W := - Result.W;
end;

Function ApplyQuaternion;
  Var
    Temp: TQuaternion;
begin
  Temp := CreateQuaternion(V.X, V.Y, V.Z, 0);
  Temp := MultiplyQuaternion(Temp, Q);
  Temp := MultiplyQuaternion(InvertQuaternion(Q), Temp);
  Result := Vec3D(Temp.X, Temp.Y, Temp.Z);
end;

//
//  Frames
//
Function CreateFrame(Pos: TVec3D; Quat: TQuaternion): TFrame;
begin
  Result.Pos := Pos;
  Result.Quat := Quat;
end;

Function InterpolateFrame;
begin
  Result.Pos := Lerp3D(Frame1.Pos, Frame2.Pos, T);
  Result.Quat := SLerpQuaternion(Frame1.Quat, Frame2.Quat, T);
end;

Function MultiplyFrame;
begin
  //Frame2.Pos := ApplyQuaternion(Frame1.Quat, Frame2.Pos);
  //Result.Pos := Add3D(Frame1.Pos, Frame2.Pos);
  Result.Pos := Vec3D(0, 0, 0);
  Result.Quat := MultiplyQuaternion(Frame1.Quat, Frame2.Quat);
end;

//
//  TFrameList
//
Constructor TFrameList.Create;
  Var
    I: Integer;
begin
  If Data.GetByte > 0 then begin
    I := Data.GetInteger;
    SetLength(Frames, I);
    I := Data.GetInteger;
    Data.GetVar(@Frames[0], Length(Frames) * SizeOf(TFrame));
  end else
    SetLength(Frames, 0);
  {
  For I := 0 to High(Frames) do
    With Frames[I] do
      Log(IntToStr(Round(Pos.X * 100)) + ' ' +
          IntToStr(Round(Pos.Y * 100)) + ' ' +
          IntToStr(Round(Pos.Z * 100)) + ' ' +
          IntToStr(Round(Quat.X * 100)) + ' ' +
          IntToStr(Round(Quat.Y * 100)) + ' ' +
          IntToStr(Round(Quat.Z * 100)) + ' ' +
          IntToStr(Round(Quat.W * 100)) + ' ');
  }
end;

Function TFrameList.GetFrameAt;
begin
  If Length(Frames) > 0 then
    Result := InterpolateFrame(Items[Start], Items[Finish], T)
  else begin
    Result.Pos := Vec3D(0, 0, 0);
    Result.Quat := Vec4D(0, 0, 0, 0);
  end;
end;

Procedure TFrameList.SetItem;
begin
  If Length(Frames) > 0 then
    Frames[Index Mod Length(Frames)] := Item;
end;

Function TFrameList.GetItem;
begin
  If Length(Frames) > 0 then
    Result := Frames[Index Mod Length(Frames)]
  else begin
    Result.Pos := Vec3D(0, 0, 0);
    Result.Quat := Vec4D(0, 0, 0, 0);
  end;
end;

//
//  Matrices math
//
Procedure Matrix_LoadIdentity;
  Var
    I: Integer;
begin
    For I := 0 to 15 do
      M[I] := Ord((I mod 5) = 0);
end;

Procedure Matrix_Rotate;
  Var
 s, c, inv_c : single;
begin
  s := sin(Angle);
  c := cos(Angle);
  inv_c := 1 - c;

  M[ 0] := (inv_c * v.X * v.X) + c;
  M[ 1] := (inv_c * v.X * v.Y) - (v.Z * s);
  M[ 2] := (inv_c * v.Z * v.X) + (v.Y * s);

  M[ 4] := (inv_c * v.X * v.Y) + (v.Z * s);
  M[ 5] := (inv_c * v.Y * v.Y) + c;
  M[ 6] := (inv_c * v.Y * v.Z) - (v.X * s);

  M[ 8] := (inv_c * v.Z * v.X) - (v.Y * s);
  M[ 9] := (inv_c * v.Y * v.Z) + (v.X * s);
  M[10] := (inv_c * v.Z * v.Z) + c;

  M[12] := 0;
  M[13] := 0;
  M[14] := 0;
  M[ 3] := 0;
  M[ 7] := 0;
  M[11] := 0;
  M[15] := 1;
end;

Procedure Matrix_Translate;
begin
  Matrix_LoadIdentity(M);
  
  M[12] := V.X;
  M[13] := V.Y;
  M[14] := V.Z;
  {
  M[ 3] := V.X;
  M[ 7] := V.Y;
  M[11] := V.Z;
  }
end;

Procedure Matrix_Scale;
begin
  Matrix_LoadIdentity(M);
  M[ 0] := Scaler.X;
  M[ 5] := Scaler.Y;
  M[10] := Scaler.Z;
  //M[15] := 1;
end;

Procedure Matrix_Multiply;
begin
  DDebug.Assert(@Dest[0] <> @Src1[0], 'Matrix_Multiply: Dest = Src1');
  DDebug.Assert(@Dest[0] <> @Src2[0], 'Matrix_Multiply: Dest = Src2');
  Dest[ 0] := Src1[ 0]*Src2[0] + Src1[ 1]*Src2[4] + Src1[  2 ]*Src2[  8 ] + Src1[ 3]*Src2[12];
  Dest[ 1] := Src1[ 0]*Src2[1] + Src1[ 1]*Src2[5] + Src1[  2 ]*Src2[  9 ] + Src1[ 3]*Src2[13];
  Dest[ 2] := Src1[ 0]*Src2[2] + Src1[ 1]*Src2[6] + Src1[  2 ]*Src2[ 10 ] + Src1[ 3]*Src2[14];
  Dest[ 3] := Src1[ 0]*Src2[3] + Src1[ 1]*Src2[7] + Src1[  2 ]*Src2[ 11 ] + Src1[ 3]*Src2[15];
  Dest[ 4] := Src1[ 4]*Src2[0] + Src1[ 5]*Src2[4] + Src1[  6 ]*Src2[  8 ] + Src1[ 7]*Src2[12];
  Dest[ 5] := Src1[ 4]*Src2[1] + Src1[ 5]*Src2[5] + Src1[  6 ]*Src2[  9 ] + Src1[ 7]*Src2[13];
  Dest[ 6] := Src1[ 4]*Src2[2] + Src1[ 5]*Src2[6] + Src1[  6 ]*Src2[ 10 ] + Src1[ 7]*Src2[14];
  Dest[ 7] := Src1[ 4]*Src2[3] + Src1[ 5]*Src2[7] + Src1[  6 ]*Src2[ 11 ] + Src1[ 7]*Src2[15];
  Dest[ 8] := Src1[ 8]*Src2[0] + Src1[ 9]*Src2[4] + Src1[ 10 ]*Src2[  8 ] + Src1[11]*Src2[12];
  Dest[ 9] := Src1[ 8]*Src2[1] + Src1[ 9]*Src2[5] + Src1[ 10 ]*Src2[  9 ] + Src1[11]*Src2[13];
  Dest[10] := Src1[ 8]*Src2[2] + Src1[ 9]*Src2[6] + Src1[ 10 ]*Src2[ 10 ] + Src1[11]*Src2[14];
  Dest[11] := Src1[ 8]*Src2[3] + Src1[ 9]*Src2[7] + Src1[ 10 ]*Src2[ 11 ] + Src1[11]*Src2[15];
  Dest[12] := Src1[12]*Src2[0] + Src1[13]*Src2[4] + Src1[ 14 ]*Src2[  8 ] + Src1[15]*Src2[12];
  Dest[13] := Src1[12]*Src2[1] + Src1[13]*Src2[5] + Src1[ 14 ]*Src2[  9 ] + Src1[15]*Src2[13];
  Dest[14] := Src1[12]*Src2[2] + Src1[13]*Src2[6] + Src1[ 14 ]*Src2[ 10 ] + Src1[15]*Src2[14];
  Dest[15] := Src1[12]*Src2[3] + Src1[13]*Src2[7] + Src1[ 14 ]*Src2[ 11 ] + Src1[15]*Src2[15];
end;

Function  Matrix_GetDet;
begin
  Result := M[ 0] * M[ 5] * M[10] + M[ 4] * M[ 9] * M[ 2] + M[ 8] * M[ 1] * M[ 6]
          - M[ 8] * M[ 5] * M[ 2] - M[ 4] * M[ 1] * M[10] - M[ 0] * M[ 9] * M[ 6];
end;

Procedure Matrix_Inverse;
  Var
    _Det: Single;
begin
  _Det := 1 / Matrix_GetDet(Src);
  Dest[ 0] :=  (Src[5] * Src[10] - Src[9] * Src[6]) * _Det;
  Dest[ 1] := -(Src[1] * Src[10] - Src[9] * Src[2]) * _Det;
  Dest[ 2] :=  (Src[1] * Src[ 6] - Src[5] * Src[2]) * _Det;
  Dest[ 3] := 0;
  Dest[ 4] := -(Src[4] * Src[10] - Src[8] * Src[6]) * _Det;
  Dest[ 5] :=  (Src[0] * Src[10] - Src[8] * Src[2]) * _Det;
  Dest[ 6] := -(Src[0] * Src[ 6] - Src[4] * Src[2]) * _Det;
  Dest[ 7] := 0;
  Dest[ 8] :=  (Src[4] * Src[9] - Src[8] * Src[5]) * _Det;
  Dest[ 9] := -(Src[0] * Src[9] - Src[8] * Src[1]) * _Det;
  Dest[10] :=  (Src[0] * Src[5] - Src[4] * Src[1]) * _Det;
  Dest[11] := 0;
  Dest[12] := -(Src[12] * Dest[0] + Src[13] * Dest[4] + Src[14] * Dest[ 8]);
  Dest[13] := -(Src[12] * Dest[1] + Src[13] * Dest[5] + Src[14] * Dest[ 9]);
  Dest[14] := -(Src[12] * Dest[2] + Src[13] * Dest[6] + Src[14] * Dest[10]);
  Dest[15] := 1;
end;

Procedure Matrix_SetSystem;
begin
  Matrix_LoadIdentity(M);
  {
  M[ 0] := Right.X;
  M[ 4] := Right.Y;
  M[ 8] := Right.Z;
  M[ 1] := Up.X;
  M[ 5] := Up.Y;
  M[ 9] := Up.Z;
  M[ 2] := Frwrd.X;
  M[ 6] := Frwrd.Y;
  M[10] := Frwrd.Z;
  M[12] := Pos.X;
  M[13] := Pos.Y;
  M[14] := Pos.Z;
  }
  M[ 0] := Right.X;
  M[ 1] := Right.Y;
  M[ 2] := Right.Z;
  M[ 4] := Up.X;
  M[ 5] := Up.Y;
  M[ 6] := Up.Z;
  M[ 8] := -Frwrd.X;
  M[ 9] := -Frwrd.Y;
  M[10] := -Frwrd.Z;
  M[12] := Pos.X;
  M[13] := Pos.Y;
  M[14] := Pos.Z;
end;

Procedure Matrix_Frame;
  Var
    wx, wy, wz, xx, yy, yz, xy, xz, zz, x2, y2, z2 : single;
begin
    With Frame.Quat do begin
      x2 := x + x;   y2 := y + y;   z2 := z + z;
      xx := x * x2;  xy := x * y2;  xz := x * z2;
      yy := y * y2;  yz := y * z2;  zz := z * z2;
      wx := w * x2;  wy := w * y2;  wz := w * z2;
    end;

  M[0] := 1 - (yy + zz);  M[1] := xy + wz;        M[ 2] := xz - wy;
  M[4] := xy - wz;        M[5] := 1 - (xx + zz);  M[ 6] := yz + wx;
  M[8] := xz + wy;        M[9] := yz - wx;        M[10] := 1 - (xx + yy);

  m[ 3] := 0;
  m[ 7] := 0;
  m[11] := 0;
  m[12] := Frame.Pos.X;
  m[13] := Frame.Pos.Y;
  m[14] := Frame.Pos.Z;
  m[15] := 1;
end;

//
//  TMatrix
//
Constructor TMatrix.Create(V00, V10, V20, V30,
                           V01, V11, V21, V31,
                           V02, V12, V22, V32,
                           V03, V13, V23, V33: Single);
begin
  Cells[0, 0] := V00;
  Cells[1, 0] := V10;
  Cells[2, 0] := V20;
  Cells[3, 0] := V30;
  Cells[0, 1] := V01;
  Cells[1, 1] := V11;
  Cells[2, 1] := V21;
  Cells[3, 1] := V31;
  Cells[0, 2] := V02;
  Cells[1, 2] := V12;
  Cells[2, 2] := V22;
  Cells[3, 2] := V32;
  Cells[0, 3] := V03;
  Cells[1, 3] := V13;
  Cells[2, 3] := V23;
  Cells[3, 3] := V33;
  //State := msAny;
end;

Constructor TMatrix.Create;
begin
  LoadIdentity;
end;

Procedure TMatrix.LoadIdentity;
begin
  M[0 ] := 1;
  M[1 ] := 0;
  M[2 ] := 0;
  M[3 ] := 0;

  M[4 ] := 0;
  M[5 ] := 1;
  M[6 ] := 0;
  M[7 ] := 0;

  M[8 ] := 0;
  M[9 ] := 0;
  M[10] := 1;
  M[11] := 0;

  M[12] := 0;
  M[13] := 0;
  M[14] := 0;
  M[15] := 1;

  //State := msIdentity;
end;

Procedure TMatrix.Copy;
begin
  Move(Matrix.M[0], M[0], SizeOf(Single) * 16);
end;

Procedure TMatrix.Multiply(Matrix: TArray16OfSingle);
begin
  Matrix_Multiply(M, Matrix, M);
  //Matrix_Multiply(M, M, Matrix);
end;

Procedure TMatrix.Multiply(Matrix: TMatrix);
begin
    {If State = msIdentity then
      Clone(Matrix)
    else begin
    }
      {
      M[ 0] := M[ 0]*Matrix.M[0] + M[ 1]*Matrix.M[4] + M[  2 ]*Matrix.M[  8 ] + M[ 3]*Matrix.M[12];
      M[ 1] := M[ 0]*Matrix.M[1] + M[ 1]*Matrix.M[5] + M[  2 ]*Matrix.M[  9 ] + M[ 3]*Matrix.M[13];
      M[ 2] := M[ 0]*Matrix.M[2] + M[ 1]*Matrix.M[6] + M[  2 ]*Matrix.M[ 10 ] + M[ 3]*Matrix.M[14];
      M[ 3] := M[ 0]*Matrix.M[3] + M[ 1]*Matrix.M[7] + M[  2 ]*Matrix.M[ 11 ] + M[ 3]*Matrix.M[15];
      M[ 4] := M[ 4]*Matrix.M[0] + M[ 5]*Matrix.M[4] + M[  6 ]*Matrix.M[  8 ] + M[ 7]*Matrix.M[12];
      M[ 5] := M[ 4]*Matrix.M[1] + M[ 5]*Matrix.M[5] + M[  6 ]*Matrix.M[  9 ] + M[ 7]*Matrix.M[13];
      M[ 6] := M[ 4]*Matrix.M[2] + M[ 5]*Matrix.M[6] + M[  6 ]*Matrix.M[ 10 ] + M[ 7]*Matrix.M[14];
      M[ 7] := M[ 4]*Matrix.M[3] + M[ 5]*Matrix.M[7] + M[  6 ]*Matrix.M[ 11 ] + M[ 7]*Matrix.M[15];
      M[ 8] := M[ 8]*Matrix.M[0] + M[ 9]*Matrix.M[4] + M[ 10 ]*Matrix.M[  8 ] + M[11]*Matrix.M[12];
      M[ 9] := M[ 8]*Matrix.M[1] + M[ 9]*Matrix.M[5] + M[ 10 ]*Matrix.M[  9 ] + M[11]*Matrix.M[13];
      M[10] := M[ 8]*Matrix.M[2] + M[ 9]*Matrix.M[6] + M[ 10 ]*Matrix.M[ 10 ] + M[11]*Matrix.M[14];
      M[11] := M[ 8]*Matrix.M[3] + M[ 9]*Matrix.M[7] + M[ 10 ]*Matrix.M[ 11 ] + M[11]*Matrix.M[15];
      M[12] := M[12]*Matrix.M[0] + M[13]*Matrix.M[4] + M[ 14 ]*Matrix.M[  8 ] + M[15]*Matrix.M[12];
      M[13] := M[12]*Matrix.M[1] + M[13]*Matrix.M[5] + M[ 14 ]*Matrix.M[  9 ] + M[15]*Matrix.M[13];
      M[14] := M[12]*Matrix.M[2] + M[13]*Matrix.M[6] + M[ 14 ]*Matrix.M[ 10 ] + M[15]*Matrix.M[14];
      M[15] := M[12]*Matrix.M[3] + M[13]*Matrix.M[7] + M[ 14 ]*Matrix.M[ 11 ] + M[15]*Matrix.M[15];
      }
      //Matrix_Multiply(M, Matrix.M, M);
   {
        Case State Of
          msRotation:
            If (Matrix.State = msTranslation)Or(Matrix.State = msScaler) then
              State := msAny;
          msTranslation:
            If (Matrix.State = msRotation)Or(Matrix.State = msScaler) then
              State := msAny;
          msScaler:
            If (Matrix.State = msTranslation)Or(Matrix.State = msRotation) then
              State := msAny;
        end;
    end;
   }
   Multiply(Matrix.M);
end;

Procedure TMatrix.Translate(V: TVec3D);
  Var
    _M: TArray16OfSingle;
begin
   {
    If State = msIdentity then begin
      M[12] := V.X;
      M[13] := V.Y;
      M[14] := V.Z;
      State := msTranslation;
      Exit;
    end;
    If State = msTranslation then begin
      M[12] := M[12] + V.X;
      M[13] := M[13] + V.Y;
      M[14] := M[14] + V.Z;
      State := msTranslation;
      Exit;
    end;
   }
  {
    For I := 0 to 15 do
      N[I] := Ord(I mod 5 = 0);
  N[12] := V.X;
  N[13] := V.Y;
  N[14] := V.Z;
  }
  {
  M[12] := M[ 0] * V.X + M[ 4] * V.Y + M[ 8] * V.Z + M[12];
  M[13] := M[ 1] * V.X + M[ 5] * V.Y + M[ 9] * V.Z + M[13];
  M[14] := M[ 2] * V.X + M[ 6] * V.Y + M[10] * V.Z + M[14];
  M[15] := M[ 3] * V.X + M[ 7] * V.Y + M[11] * V.Z + M[15];
  }
  Matrix_Translate(_M, V);
  Multiply(_M);
end;

Procedure TMatrix.RotateX;
begin
end;

Procedure TMatrix.RotateY;
begin
end;

Procedure TMatrix.RotateZ;
begin
end;

Procedure TMatrix.Rotate;
  Var
    _M: TArray16OfSingle;
begin
  Matrix_Rotate(_M, Angle, V);
  Multiply(_M);
end;

Procedure TMatrix.Scale;
  Var
    _M: TArray16OfSingle;
begin
  Matrix_Scale(_M, Scaler);
  Multiply(_M);
end;

Procedure TMatrix.LookAt;
  Var
    _M: TArray16OfSingle;
    Right, Up: TVec3D;
begin
  Right := Cross3D(Target, Vec3D(0, 1, 0));
  Up := Cross3D(Right, Target);
  Matrix_SetSystem(_M, Pos, Up, Right, Target);
  Matrix_Inverse(_M, _M);
  Multiply(_M);
end;

Procedure TMatrix.base(ord, right, up, target: TVec3D);
  Var
    temp: TArray16OfSingle;
begin
  temp[0] := right.x;
  temp[1] := right.y;
  temp[2] := right.z;
  temp[3] := 0;

  temp[4] := up.x;
  temp[5] := up.y;
  temp[6] := up.z;
  temp[7] := 0;

  temp[8] := target.x;
  temp[9] := target.y;
  temp[10] := target.z;
  temp[11] := 0;

  temp[12] := ord.x;
  temp[13] := ord.y;
  temp[14] := ord.z;
  temp[15] := 1;

  Matrix_Multiply(m, temp, m);
end;

Procedure TMatrix.SetFrame;
begin
  Matrix_Frame(M, Frame);
end;

Procedure TMatrix.MulFrame;
  Var
    _M: TArray16OfSingle;
begin
  Matrix_Frame(_M, Frame);
  Multiply(_M);
end;

Procedure TMatrix.LoadToOpenGL;
begin
  glLoadMatrixf(@M[0]);
end;

Procedure TMatrix.LoadFromOpenGL;
begin
  glGetFloatv(MatrixMode, @M[0]);
end;

Procedure TMatrix.MulToOpenGL;
begin
  glMultMatrixf(@M[0]);
end;

Procedure TMatrix.MulFromOpenGL;
begin
end;

Procedure TMatrix.Inverse;
begin
  Matrix_Inverse(M, M);
end;

Function TMatrix.CreateInverse;
begin
  Result := TMatrix.Create;
  Result.Copy(Self);
  Result.Inverse;
end;

Function TMatrix.Mul;
begin
  Result.X := V.X * M[0] + V.Y * M[4] + V.Z * M[ 8] + M[12];
  Result.Y := V.X * M[1] + V.Y * M[5] + V.Z * M[ 9] + M[13];
  Result.Z := V.X * M[2] + V.Y * M[6] + V.Z * M[10] + M[14];
  {
  Result.X := V.X * M[0] + V.Y * M[1] + V.Z * M[ 2] + M[ 3];
  Result.Y := V.X * M[4] + V.Y * M[5] + V.Z * M[ 6] + M[ 7];
  Result.Z := V.X * M[8] + V.Y * M[9] + V.Z * M[10] + M[11];
  }
end;

Function TMatrix.GetCellValue(X, Y: Byte): Single;
begin
  Result := M[X + (Y shl 2)];
end;

Procedure TMatrix.SetCellValue(X, Y: Byte; Value: Single);
begin
  M[X + (Y shl 2)] := Value;
end;

Procedure TMatrix.PrintToLog;
  Var
    I: Integer;
begin
  Log.PrintLn('');
  Log.Print('Matrix ' + '"' + Name + '"');
    For I := 0 to 3 do
      Log.PrintLn('   ' +
          FloatToStr(M[I     ]) + ' ' +
          FloatToStr(M[I +  4]) + ' ' +
          FloatToStr(M[I +  8]) + ' ' +
          FloatToStr(M[I + 12]));
  Log.PrintLn('');
end;

//
//  Bits
//
Function IncToPOT(Value: Cardinal): Cardinal;
begin
  Result := 1;
    While Result < Value do
      Result := Result shl 1;
end;

end.
