//17.10.2008 [*] fixed uninherited TData.Destroy
//           [*] fixed signed and unsifgned addition in TData.seekCur and TData.getVar
//25.06.2008 [+] TTimer.SetInterval
unit DUtils;

interface

Uses
  Windows;

Type
  PArrayOfBytes = ^TArrayOfBytes;
  TArrayOfBytes = array[0..1] of Byte;
  TArray4OfByte = array[0..3] of Byte;
  TArray16OfSingle = array[0..15] of Single;

Function IntToStr(Value: integer): string;
Function StrToInt(Value: string): integer;
Function StrToIntDef(Value: String; Def: integer): integer;
Function StrToFloat(S: String): Single;
Function FloatToStr(F: Single): String;
function CharArrToStr(const C : Array of Char) : String;
Function FileExist(Filename: string): boolean;
Procedure CreateFile(FileName: string);
Procedure DeleteFile(Filename: string);
Function GetTimer: integer;
//     
Function AnsiLowerCase(const S: string): string;

Type
TField = record
  Name, Value: String;
end;

TSection = record
  Name: String;
  Fields: array of TField;
end;

TIniFile = class
private
  FileName: String;
  Sections: array of TSection;
  Class Function FindFieldInSection(Var Section: TSection; Field: string): integer;
  Class Procedure SetFieldInSection(Var Section: TSection; Field: string; Value: string);
  Class Function GetFieldInSection(Var Section: TSection; Field: string): string;
  Function FindSection(Section: string): integer;
  Procedure AddSection(Section: string);
  Procedure Clear;
public
  Constructor Create(FileName: String);
  Destructor Destroy; override;
  Procedure Update;
  Procedure Flush;
  Function GetField(Section, Field: String): String;
  Procedure SetField(Section, Field, Value: String);
  Property Fields[Index1, Index2: String]: String read GetField write SetField; default;
end;

//@class  -       .
TTimer = class
private
  Old: Integer;
  Int: Integer;
public
  Constructor Create(Interval: Integer);
  Procedure SetInterval(Interval: Integer);
  Function Pop: Boolean;
  Procedure Reset;
end;

//@class       (  TStream)
TData = class
  Bytes: PArrayOfBytes;
  Len: cardinal;
  Pos: cardinal;
  Constructor Create(Filename: string);
  Destructor Destroy; override;
  Function IsEmpty: boolean;
  Procedure GetVar(p: pointer; size: integer);
  Function GetByte: byte;
  Function GetWord: word;
  Function GetInteger: integer;
  Function GetCardinal: cardinal;
  Function GetFloat: single;
  Function GetString: string;
  Function GetPChar: string;
  Function GetPtr: pointer;
  Function SeekCur(delta: integer): integer;
  Function SeekAbs(offs: integer): integer;
  Function Ready: Boolean;
end;


implementation

Function InttoStr;
begin
  Str(Value, Result);
end;

Function StrtoInt;
  Var
    Code: integer;
begin
  Val(Value, Result, Code);
end;

Function StrToIntDef(Value: String; Def: integer): integer;
  Var
    Code: integer;
begin
  Val(Value, Result, Code);
    If Code <> 0 then
      Result := Def;
end;

Function StrToFloat(S: String): Single;
  Var
    Code: Integer;
begin
    If Pos('.', S) <> 0 then begin
      //Insert(',', S, Pos('.', S));
      //Delete(S, Pos('.', S), 1);
    end;
  Val(S, Result, Code);
end;

Function FloatToStr(F: Single): String;
begin
  Str(F, Result);
    If Pos(',', Result) <> 0 then begin
      Insert('.', Result, Pos(',', Result));
      Delete(Result, Pos(',', Result), 1);
    end;
end;

function CharArrToStr(const C : Array of Char) : String;
var I : Integer;
begin
  // convert the array of characters to a String
  I :=0;
  result :='';
  while C[i] <> #0 do
  begin
    result := result + C[I];
    Inc(I);
  end;
end;

Function FileExist;
  Var
    F: File;
begin
  FileMode := 64;
  AssignFile(F, Filename);
  {$I-}
  Reset(F);
  {$I+}
  Result := IOResult = 0;
    If Result then
      CloseFile(F);
end;

Procedure CreateFile;
  Var
    F: TextFile;
begin
    If FileExist(FileName) then
      Exit;
  Assign(F, FileName);
  {$I-}
  Rewrite(F);
  {$I+}
    If IOResult = 0 then
      Close(F);
end;

Procedure DeleteFile;
begin
    If not FileExist(Filename) then
      Exit;
  Windows.DeleteFile(PAnsiChar(Filename));
end;

Function GetTimer;
var
  T, F: LARGE_INTEGER;
begin
  QueryPerformanceFrequency(int64(F));
  QueryPerformanceCounter(int64(T));
  Result:=Trunc(1000*T.QuadPart/F.QuadPart);
end;

Function AnsiLowerCase(Const S: String): String;
  Var
    Len: Integer;
begin
  Len := Length(S);
  SetString(Result, PChar(S), Len);
    If Len > 0 then
      CharLowerBuff(Pointer(Result), Len);
end;

//
// TIniFile
//
Class Function TIniFile.FindFieldInSection;
  Var
    I: integer;
begin
    With Section do begin
        For I := 0 to High(Fields) do
          With Fields[I] do
            If Name = Field then begin
              Result := I;
              Exit;
            end;
    end;
  Result := -1;
end;

Class Procedure TIniFile.SetFieldInSection;
  Var
    Index: integer;
begin
  Index := FindFieldInSection(Section, Field);
    If Index = -1 then
        With Section do begin
          SetLength(Fields, Length(Fields) + 1);
          Fields[High(Fields)].Name := Field;
          Index := High(Fields);
        end;
    With Section do
      Fields[Index].Value := Value;
end;

Class Function TIniFile.GetFieldInSection;
  Var
    Index: integer;
begin
  Index := FindFieldInSection(Section, Field);
    If Index = -1 then
      Result := '???'
    else
      With Section do
        Result := Fields[Index].Value;
end;

Function TIniFile.FindSection;
  Var
    I: integer;
begin
    For I := 0 to High(Sections) do
      If Sections[I].Name = Section then begin
        Result := I;
        Exit;
      end;
  Result := -1;
end;

Procedure TIniFile.AddSection;
begin
  SetLength(Sections, Length(Sections) + 1);
  Sections[High(Sections)].Name := Section;
  SetLength(Sections[High(Sections)].Fields, 0);
end;

Procedure TIniFile.Clear;
  Var
    I: integer;
begin
    For I := 0 to High(Sections) do
      With Sections[I] do
        SetLength(Fields, 0);
  SetLength(Sections, 0);
end;

Constructor TIniFile.Create;
begin
  Self.FileName := FileName;
  Update;
end;

Destructor TIniFile.Destroy;
begin
  Flush;
  inherited;
end;

Procedure TIniFile.Update;
  Var
    F: TextFile;
    S: String;
    Field, Value: String;
begin
    If not FileExist(FileName) then
      CreateFile(FileName);
  Clear;
  Assign(F, FileName);
  {$I-}
  Reset(F);
  {$I+}
    If IOResult <> 0 then begin
      Exit;
    end;
    While not Eof(F) do begin
      Readln(F, S);
        If Pos('//', S) <> 1 then
          If (Pos('[', S) > 0)and(Pos('[', S) < Pos(']', S)) then begin
            Delete(S, 1, Pos('[', S));
            Delete(S, Pos(']', S), Length(S) - Pos(']', S) + 1);
            AddSection(S);
          end else
          If Pos('=', S) <> 0 then begin
            Field := S;
            Delete(Field, Pos('=', Field), Length(Field) - Pos('=', Field) + 1);
            Value := S;
            Delete(Value, 1, Pos('=', Value));
              If Length(Sections) = 0 then
                AddSection('');
            SetFieldInSection(Sections[High(Sections)], Field, Value);
          end;
    end;
  Close(F);
end;

Procedure TIniFile.Flush;
  Var
    F: TextFile;
    I, J: integer;
begin
  Assign(F, Filename);
  {$I-}
  Rewrite(F);
  {$I+}
    For I := 0 to High(Sections) do begin
      With Sections[I] do begin
        Writeln(F, '[' + Name + ']');
          For J := 0 to High(Fields) do
            With Fields[J] do
              Writeln(F, Name + '=' + Value);
      end;
    end;
  Close(F);
end;

Function TIniFile.GetField;
  Var
    Index: integer;
begin
  Index := FindSection(Section);
    If Index = -1 then
      Result := '???'
    else
      Result := GetFieldInSection(Sections[Index], Field);
end;

Procedure TIniFile.SetField;
  Var
    Index: integer;
begin
  Index := FindSection(Section);
    If Index = -1 then begin
      AddSection(Section);
    end;
  SetFieldInSection(Sections[High(Sections)], Field, Value);
end;

//
//  TTimer
//
Constructor TTimer.Create;
begin
  Int := Interval;
  Old := GetTimer;
end;

Procedure TTimer.SetInterval(Interval: Integer);
begin
  Int := Interval;
end;

Function TTimer.Pop;
begin
  Result := GetTimer > (Old + Int);
    If Result then
      Inc(Old, Int);
end;

Procedure TTimer.Reset;
begin
  Old := GetTimer;
end;

//
//  TData
//
Constructor TData.create(filename:string);
  Var f : file;
begin
  Bytes := nil;
  assign(f,filename);
  {$I-}
  reset(f,1);
  {$I+}
    If IOResult <> 0 then
      exit;
  len := filesize(f);
    If len = 0 then
      exit;
  GetMem(bytes, len);
  BlockRead( f , bytes^ , len );
  Close    ( f );
  Pos := 0;
end;

//-----------------------------------------------------------------------------
Destructor TData.Destroy;
begin
  FreeMem(bytes, len);
end;

//-----------------------------------------------------------------------------
Function TData.isEmpty : boolean;
begin
  result := pos >= len;
end;

//-----------------------------------------------------------------------------
Procedure TData.getVar(p: pointer; size: integer);
  Var i: integer;
begin
    for i := 0 to size - 1 do
      PArrayOfBytes(p)^[i] := bytes^[Abs(pos) + i];
  inc(pos, size);
end;

//-----------------------------------------------------------------------------
Function TData.GetByte : byte;
begin
  Result := 0;
    If pos < len then begin
      Result := Bytes^[pos];
      Inc(Pos);
    end else
      Result := 0;
end;

//-----------------------------------------------------------------------------
Function TData.GetWord : word;
begin
  Result := 0;
    If pos < len - 1 then begin
      Result := word(getPtr^);
      Inc(Pos, SizeOf(word));
    end;
end;

//-----------------------------------------------------------------------------
Function TData.GetInteger: integer;
begin
  Result := 0;
    If pos < len - 3 then begin
      Result := integer(GetPtr^);
      Inc(Pos, SizeOf(integer))
    end
end;

//-----------------------------------------------------------------------------
Function TData.GetCardinal : cardinal;
begin
  Result := 0;
    If pos < len - 3 then begin
      result := cardinal( getPtr^ );
      inc( pos, sizeOf( cardinal ) );
    end;
end;

//-----------------------------------------------------------------------------
Function TData.GetFloat : single;
begin
  Result := 0;
    If Pos < Len - SizeOf(single) + 1 then begin
      Result := single(GetPtr^);
      Inc(Pos, SizeOf(single));
    end;
end;

//-----------------------------------------------------------------------------
Function TData.getString : string;
  Var len: byte;
begin
  Len := GetByte;
  SetLength(Result, Len);
  GetVar(@result[1], Len);
end;

//-----------------------------------------------------------------------------
Function TData.getPChar: string;
  Var
  c: char;
begin
  Result := '';
  c := chr(GetByte);
    while c <> #0 do begin
      Result := Result + c;
      c := chr(GetByte);
    end;
end;

//-----------------------------------------------------------------------------
Function TData.getPtr : pointer;
begin
    If Pos < Len then
      Result := @Bytes^[pos]
    else
      Result := nil;
end;

//-----------------------------------------------------------------------------
Function TData.seekCur(delta: integer): integer;
begin
  Pos := Abs(Pos) + delta;
    If Pos > Len then
      Pos := Len;                          
  Result := Pos;
end;

//-----------------------------------------------------------------------------
Function TData.seekAbs(offs: integer): integer;
begin
  Pos := Offs;
    If Pos > Len then
      Pos := Len;
  Result := Pos;
end;

Function TData.Ready: Boolean;
begin
  Result := Bytes <> nil;
end;

end.
