unit GCicloids;

interface

Uses
  {$I Units.inc};

implementation

//
//  TCicloidForm
//
Type
TCicloidForm = class(TObjectForm)
protected
  Procedure CreateBody; override;
public
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Procedure TCicloidForm.CreateBody;
begin
  FBody := TRect.Create(Self);
  TRect(FBody).A := Sub2D(Vec2D(160, 120), Vec2D(5, 5));
  TRect(FBody).B := Add2D(Vec2D(160, 120), Vec2D(5, 5));
end;

Procedure TCicloidForm.AssignVar(Name, Value: String);
begin
  If Name = 'w' then begin
    TRect(FBody).A := Vec2D(0, TRect(FBody).A.Y); //:= Sub2D(Vec2D(160, 120), Vec2D(5, 5));
    TRect(FBody).B := Vec2D(StrToInt(Value), TRect(FBody).B.Y);
  end else If Name = 'h' then begin
    TRect(FBody).A := Sub2D(Vec2D(160, 120), Vec2D(5, 5));
    TRect(FBody).B := Add2D(Vec2D(160, 120), Vec2D(5, 5));
  end;
end;

Procedure TCicloidForm.Render;
begin
  Inherited Render;
  {glBegin(GL_LINE_LOOP);
    glVertex2f(TRect(FBody).LT.X, TRect(FBody).LT.Y);
    glVertex2f(TRect(FBody).RT.X, TRect(FBody).RT.Y);
    glVertex2f(TRect(FBody).RB.X, TRect(FBody).RB.Y);
    glVertex2f(TRect(FBody).LB.X, TRect(FBody).LB.Y);
  glEnd;}
end;

//
//  TCicloidRender
//
Type
TCicloidRender = class(TGameDecorator)
private
  R, G, B: Single;
  FType: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Render; override;
end;

Constructor TCicloidRender.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  R := 0.75;
  G := 0.75;
  B := 0.0;
end;

Procedure TCicloidRender.Render;
  Var
    ReqPos: TEvReqPos;
begin
  Inherited Render;
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);
  glColor3f(Sin(GetTimer/1000), 1-Sin(GetTimer/1000), Cos(GetTimer/1000));
    With ReqPos.Pos do begin  
    //  glRectf(X-5, Y-5, X+5, Y+5);
      DrawEllipse(X, Y, 5, 5, 8,
                  0.75, 0.00, 0.0, 1.0,
                  0.25,  0.75, 0.25, 1.0);
    end;
  glColor3f(1, 1, 1);
end;

//
//  TCicloidRenderEx
//
Type
TCicloidRenderEx = class(TGameDecorator)
private
  R, G, B: Single;
  FType: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Constructor TCicloidRenderEx.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  R := 0.75;
  G := 0.75;
  B := 0.0;
end;

Procedure TCicloidRenderEx.AssignVar(Name, Value: String);
begin
    If Name = 'r' then
      R := StrToFloat(Value)
    else If Name = 'g' then
      G := StrToFloat(Value)
    else If Name = 'b' then
      B := StrToFloat(Value)
    else If Name = 'type' then
      FType := StrToInt(Value);
end;

Procedure TCicloidRenderEx.Render;
  Var
    ReqPos: TEvReqPos;
    ReqVel: TEvReqVel;
begin
  Inherited Render;
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);
  ReqVel := TEvReqVel.Create(Self);
  Decoratable.SendEvent(ReqVel);
    With ReqPos.Pos do begin
    //  glRectf(X-5, Y-5, X+5, Y+5);
        Case FType of
          1: DrawEllipse(X, Y, 5, 5, 12,
                         R, G, B, 1.0,
                         R-0.1, G-0.1, B-0.1, 1.0);
          2: begin
               glBegin(GL_TRIANGLES);
                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X-5, Y+5);
                 glVertex2f(X+5, Y+5);

                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X+5, Y+5);
                 glVertex2f(X,   Y-5);

                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X,   Y-5);
                 glVertex2f(X-5, Y+5);
               glEnd;
             end;
          3: begin
               glColor3f(R, G, B);
               glBegin(GL_TRIANGLES);
                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X-5, Y-5);
                 glVertex2f(X+5, Y-5);

                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X+5, Y-5);
                 glVertex2f(X+5, Y+5);


                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X+5, Y+5);
                 glVertex2f(X-5, Y+5);

                 glColor3f(R, G, B);
                 glVertex2f(X, Y);
                 glColor3f(R*0.5, G*0.5, B*0.5);
                 glVertex2f(X-5, Y+5);
                 glVertex2f(X-5, Y-5);
               glEnd;
             end;
        end;
    end;
  glColor3f(1, 1, 1);
  ReqPos.Free;
  ReqVel.Free;
end;

//
//  TSmoge
//
Type
  TParticle = packed record
    X, Y, VX, VY, Alpha: Single;
    Start: Integer;
  end;
  
TSmogeRender = class(TGameDecorator)
private
  FLast: Integer;
  FParticles: Array[0..1000] Of TParticle;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Render; override;
  Procedure Update; override;
end;

Constructor TSmogeRender.Create(Owner: TEntity);
  Var
    I: Integer;
begin
  Inherited Create(Owner);
    For I := Low(FParticles) to High(FParticles) do
      FParticles[I].Alpha := -1;
  FLast := GetTimer;
end;

Procedure TSmogeRender.Render;
  Var
    I: Integer;
begin
  glEnable(GL_BLEND);
  glBlendFunc(GL_ONE, GL_ONE);
  SpriteSetSize(10, 10);
  SpriteSetFullTexture;
  //SpriteSetTexture(Texture);
  SpriteSetColor(1, 0, 0);
    For I := Low(FParticles) to High(FParticles) do
      With FParticles[I] do begin
        Alpha := 1 - (GetTimer - Start)/1000;
          If Alpha < 0 then
            Continue;
          If Start > GetTimer then
            Continue;
        SpriteSetPos(X + VX*(GetTimer-Start),
                     Y + VY*(GetTimer-Start));
        SpriteSetAngle((GetTimer-Start)/1000);
        SpriteSetColor(Alpha, Alpha, Alpha);
        //SpriteDraw;
              DrawEllipse(
                  X + VX*(GetTimer-Start),
                  Y + VY*(GetTimer-Start),
                  10, 10, 8,
                  Alpha,
                  Alpha,
                  Alpha,
                  0,
                  0, 0, 0, 0);
      end;
  glDisable(GL_BLEND);
end;

Procedure TSmogeRender.Update;
  Var
    NeedNew: Integer;
    I: Integer;
    R: Single;
    ReqPos: TEvReqPos;
begin
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);
  NeedNew := (GetTimer - FLast) div 1;
    If NeedNew > 0 then
      For I := Low(FParticles) to High(FParticles) do begin
          If FParticles[I].Alpha < 0 then
            With FParticles[I] do begin
              Start := GetTimer + Random(1000);
              X := ReqPos.Pos.X;
              Y := ReqPos.Pos.Y;
              R := Random;
              VX := 0.005*Cos(R*2*pi);
              VY := 0.005*Sin(R*2*pi);
              Dec(NeedNew);
              //Inc(Last, 11);
            end;
          If NeedNew <= 0 then begin
            FLast := GetTimer;
            Break;
          end;
      end;
  ReqPos.Free;
end;

//
//  Utility
//
Procedure CreateCicloid(Owner: TGameDecorator; X, Y, VX, VY: Single);
  Var
    S: String;
begin
  S := '@Cicloid = ';
  S := S + 'TCicloidForm';
  S := S + ', TPositer!x=' + IntToStr(Round(X)) + '/y=' + IntToStr(Round(Y));
  S := S + ', TBody';
  S := S + ', TVelocity!x=' + FloatToStr(VX) + '/y=' + FloatToStr(VY);
  S := S + ', TLineMirrored';
  S := S + ', TLineMirrored!aim=TBackground';
  S := S + ', TCicloidRender';
  S := S + ', TOutKiller';
  S := S + ', TDamagable!aim=TSubm/damage=1';
  S := S + ', TDamagable!aim=TBoss/damage=1';
  //Log.PrintLn(S);
  Owner.Game.DoString(S);
end;

Procedure CreateLineBreaker(Game: TGame; X, Y, VX, VY: Single; BGMirror: Boolean = False);
  Var
    S: String;
begin
  S := '@LineBreaker = ';
  S := S + 'TCicloidForm';
  S := S + ', TPositer!x=' + IntToStr(Round(X)) + '/y=' + IntToStr(Round(Y));
  S := S + ', TBody';
  S := S + ', TVelocity!x=' + FloatToStr(VX) + '/y=' + FloatToStr(VY);
  S := S + ', TLineBreaker';
  S := S + ', TCicloidRenderEx!type=2/r=1/g=0/b=1';
  S := S + ', TOutKiller';
  S := S + ', TDamagable!aim=TSubm/damage=0';
    If BGMirror then
      S := S + ', TLineMirrored!aim=TBackground';
  Game.DoString(S);
end;

Procedure CreateGun(Game: TGame; X, Y, VX, VY: Single);
  Var
    S: String;
begin
  S := '@LineBreaker = ';
  S := S + 'TCicloidRespawnFront!count=10/vel=0.5/interval=3000';
  S := S + ', TPositer!x=' + IntToStr(Round(X)) + '/y=' + IntToStr(Round(Y));
  S := S + ', TCicloidForm';
  S := S + ', TBody';
  S := S + ', TVelocity!x=' + FloatToStr(VX) + '/y=' + FloatToStr(VY);
  S := S + ', TLineMirrored';
  S := S + ', TLineMirrored!aim=TBackground';
  S := S + ', TCicloidRenderEx!type=3/r=1/g=1/b=0';
  S := S + ', TOutKiller';
  Game.DoString(S);
end;

Procedure CreateSmoge(Game: TGame; X, Y, VX, VY: Single);
  Var
    S: String;
begin
  S := '@Cicloid = ';
  S := S + 'TCicloidForm';
  S := S + ', TPositer!x=' + IntToStr(Round(X)) + '/y=' + IntToStr(Round(Y));
  S := S + ', TBody';
  S := S + ', TVelocity!x=' + FloatToStr(VX) + '/y=' + FloatToStr(VY);
  S := S + ', TLineMirrored';
  S := S + ', TLineMirrored!aim=TBackground';
  S := S + ', TSmogeRender';
  S := S + ', TOutKiller!offset=100';
  S := S + ', TDamagable!aim=TSubm/damage=1';
  //Log.PrintLn(S);
  Game.DoString(S);
end;

//
//  TCicloidRespawnFront
//
Const
  clSimple = 1;
  clLineBreaker = 2;
  clGun = 3;
  clSmoge = 4;

Type
TCicloidRespawnFront = class(TGameDecorator)
private
  FCount: Integer;
  FOldCreated: Integer;
  FInterval: Integer;
  FVel: Single;
  FType: Integer;
  FAim: TVec2D;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Update; override;
end;

Constructor TCicloidRespawnFront.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FCount := 1;
  FOldCreated := 0;
  FInterval := 100;
  FVel := 1;
  FType := clSimple;
  FAim := Vec2D(160, 120);
end;

Procedure TCicloidRespawnFront.AssignVar(Name, Value: String);
begin
  //Log.PrintLN(Name + ' = ' + Value);
    If Name = 'count' then
      FCount := StrToInt(Value)
    else If Name = 'interval' then
      FInterval := StrToInt(Value)
    else If Name = 'vel' then
      FVel := StrToFloat(Value)
    else If Name = 'type' then begin
      FType := StrToInt(Value);
    end else If Name = 'aimx' then
      FAim.X := StrToFloat(Value)
    else If Name = 'aimy' then
      FAim.Y := StrToFloat(Value);
end;

Procedure TCicloidRespawnFront.Update;
  Var
    ReqPos: TEvReqPos;
    Vel: TVec2D;
begin
  Inherited Update;
    If (FCount > 0)And(GetTimer > FOldCreated + FInterval) then begin
      FOldCreated := FOldCreated + FInterval;
        If (GetTimer > FOldCreated + FInterval) then
          FOldCreated := GetTimer;
      Dec(FCount);
      ReqPos := TEvReqPos.Create(Self);
      Decoratable.SendEvent(ReqPos);
        If FType = clSimple then begin
          Vel := Sub2D(FAim, ReqPos.Pos);
          Vel := Mul2D(Norm2D(Vel), FVel);
          CreateCicloid(Self,
                        ReqPos.Pos.X, ReqPos.Pos.Y,
                        Vel.X, Vel.Y);
        end else If FType = clLineBreaker then begin
          Vel := Sub2D(FAim, ReqPos.Pos);
          Vel := Mul2D(Norm2D(Vel), FVel);
          CreateLineBreaker(Game,
                        ReqPos.Pos.X, ReqPos.Pos.Y,
                        Vel.X, Vel.Y);
        end else If FType = clSmoge then begin
          Vel := Sub2D(FAim, ReqPos.Pos);
          Vel := Mul2D(Norm2D(Vel), FVel);
          CreateSmoge(Game,
                        ReqPos.Pos.X, ReqPos.Pos.Y,
                        Vel.X, Vel.Y);
        end else begin
          Vel := Sub2D(FAim, ReqPos.Pos);
          Vel := Mul2D(Norm2D(Vel), FVel);
          CreateGun(Game,
                        ReqPos.Pos.X, ReqPos.Pos.Y,
                        Vel.X, Vel.Y);
        end;
      ReqPos.Free;
    end;
end;

//
//  TCicloidGun
//
Type
TCicloidGun = class(TGameDecorator)
private
  FStart: Integer;
  FTime: Integer;
  FCount: Integer;
public
  Procedure AssignVar(Name, Value: String); override;
end;

Procedure TCicloidGun.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then begin
      FTime := StrToInt(Value);
      FStart := GetTimer;
    end else If Name = 'count' then
      FCount := StrToInt(Value);
end;

//
//  Boss
//
Type
TBoss = class(TGameDecorator)
private
  FState: Integer;
  FStart: Integer;
  FAim: TVec2D;
  FCount: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Update; override;
end;

Constructor TBoss.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FState := 3;
  FAim := Vec2D(400, 120);
  Decoratable.SendEventAndFree(TEvSetPos.Create(Self, Vec2D(400, 120)));
end;

Procedure TBoss.Update;
  Var
    ReqPos: TEvReqPos;
begin
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);

    Case FState of
      1:
         begin
           Decoratable.SendEventAndFree(TEvSetPos.Create(Self,
                Add2D(ReqPos.Pos, Mul2D(Sub2D(FAim, ReqPos.Pos), 0.1))
                ));
             If Dist2D(FAim, ReqPos.Pos) < 10 then begin
               FState := 2;
               FStart := GetTimer;
               FCount := 0;
             end;
         end;
      2:
         begin
             While GetTimer > FStart + FCount*625 do begin
               Inc(FCount);
                 If FCount mod 2 = 0 then
                     If FCount mod 3 = 0 then
                       CreateLineBreaker(Game,
                                         ReqPos.Pos.X-10,
                                         ReqPos.Pos.Y-10,
                                         -0.25, -0.5, True)
                     else
                       CreateCicloid(Self,
                                     ReqPos.Pos.X-10,
                                     ReqPos.Pos.Y-10,
                                     -0.25, -0.5)
                 else
                     If FCount mod 3 = 0 then
                       CreateLineBreaker(Game,
                                         ReqPos.Pos.X-10,
                                         ReqPos.Pos.Y+10,
                                         -0.25, 0.5, True)
                     else
                       CreateCicloid(Self,
                                     ReqPos.Pos.X-10,
                                     ReqPos.Pos.Y+10,
                                     -0.25, 0.5);
             end;
             If GetTimer > FStart + 5000 then begin
               FState := 3;
               FStart := GetTimer;
               FAim := Vec2D(440, 120);
             end;
         end;
      3:
         begin
           Decoratable.SendEventAndFree(TEvSetPos.Create(Self,
                Add2D(ReqPos.Pos, Mul2D(Sub2D(FAim, ReqPos.Pos), 0.01))
                ));
             If Dist2D(FAim, ReqPos.Pos) < 10 then begin
               FState := 1;
               FStart := GetTimer;
               FAim := Vec2D(240, 120);
             end;
         end;
    end;
    
  ReqPos.Free;
end;

initialization
  Factory.RegistClass(TCicloidForm);
  Factory.RegistClass(TCicloidRender);
  Factory.RegistClass(TCicloidRenderEx);
  Factory.RegistClass(TSmogeRender);
  Factory.RegistClass(TCicloidRespawnFront);
  Factory.RegistClass(TBoss);
end.
