unit GEditors;

interface

Uses
  {$I Units.inc};

implementation

//
//    (   DATA\Editors\Lines.ded)
//   ,    .
//
Type
TEditorLines = class(TDecorator)
private
  FAll: TList;
  FCurve: TCurve2D;
  FMaxLen: Double;
  FCreating: Boolean;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure MouseDown;
  Procedure MouseUp;
  Procedure MouseMoved(X, Y: Integer);
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TEditorLines.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FCurve := TCurve2D.Create(Self);
  FAll := TList.Create(Self);
end;

Destructor TEditorLines.Destroy;
  Var
    F: TextFile;
    I, J: Integer;
    Count: Integer;
begin
  Log.PrintLn('TEditorLines: saving...');
  Assign(F, 'DATA\Editors\Lines.ded');
  Rewrite(F);
  Count := 0;
    For I := 0 to FAll.High do begin
      Inc(Count, TCurve2D(FAll[I]).PointCount-1);
    end;
  Writeln(F, '    Lines: Array[0..', Count-1, '] Of Array[0..1] Of TVec2D = (');
    For I := 0 to FAll.High do
      For J := 0 to TCurve2D(FAll[I]).PointCount-2 do begin
        Writeln(F, '        ((x:', TCurve2D(FAll[I])[J  ].X, '; y:', TCurve2D(FAll[I])[J  ].Y, '),');
        Write  (F, '         (x:', TCurve2D(FAll[I])[J+1].X, '; y:', TCurve2D(FAll[I])[J+1].Y, '))');
          If (I <> FAll.High)Or(J <> TCurve2D(FAll[I]).PointCount-2) then
            Writeln(F, ', ')
          else
            Writeln(F);
      end;
  Writeln(F, '    );');
  Close(F);

  FAll.FreeAll;
  FAll.Free;
  FCurve.Free;
  Inherited Destroy;
end;

Procedure TEditorLines.Render;
  Var
    I: Integer;
begin
  glColor3f(1, 0, 0);
  FCurve.Render;
  glColor3f(1, 1, 1);
    For I := 0 to FAll.High do
      FAll[I].Render;
end;

Procedure TEditorLines.MouseDown;
begin
  FCurve.ClearPoints;
  FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
  FCreating := True;
end;

Procedure TEditorLines.MouseUp;
begin
  FCreating := False;
  FAll.Add(FCurve);
  FCurve := TCurve2D.Create(Self);
end;

Procedure TEditorLines.MouseMoved(X, Y: Integer);
begin
    If FCreating then begin
      FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
    end;
end;

Procedure TEditorLines.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEventMouseButton') then
      With TEventMouseButton(Event) do begin
        If (Key = 0) And Down then
          MouseDown;
        If (Key = 0) And (Not Down) then
          MouseUp;
      end;
    If Event.IsType('TEvMouseMoved') then
      With TEvMouseMoved(Event) do
        MouseMoved(X, Y);
end;

//
//  TEditorEllipses -  
//
Const
  S_NO = 0;
  S_F1 = 1;
  S_F2 = 2;
  S_RAD = 3;

Type
TEditorEllipseEntry = packed record
  F1, F2: TVec2D;
  Rad: Single;
end;

TEditorEllipses = class(TDecorator)
private
  FAll: Array Of TEditorEllipseEntry;
  FMaxLen: Double;
  FState: Integer;
  Function Curr: TEditorEllipseEntry;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure MouseDown;
  Procedure MouseUp;
  Procedure MouseMoved(X, Y: Integer);
  Procedure OnEvent(Event: TEvent); override;
end;

Function TEditorEllipses.Curr;
begin
  Result := FAll[High(FAll)];
end;

Constructor TEditorEllipses.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
end;

Destructor TEditorEllipses.Destroy;
  Var
    F: TextFile;
    I, J: Integer;
    Count: Integer;
begin
  Log.PrintLn('TEditorEllipses: saving...');
  Assign(F, 'DATA\Editors\Lines.ded');
  Rewrite(F);
  Count := 0;
  {
    For I := 0 to High(FAll) do begin
      Inc(Count, TCurve2D(FAll[I]).PointCount-1);
    end;
  Writeln(F, 'Lines: Array[0..', Count-1, '] Of Array[0..1] Of TVec2D = (');
    For I := 0 to FAll.High do
      For J := 0 to TCurve2D(FAll[I]).PointCount-2 do begin
        Writeln(F, '(', TCurve2D(FAll[I])[J].X, ', ', TCurve2D(FAll[I])[J].Y, '),');
        Write(F, '(', TCurve2D(FAll[I])[J].X, ', ', TCurve2D(FAll[I])[J].Y, ')');
          If (I <> FAll.High)Or(J <> TCurve2D(FAll[I]).PointCount-2) then
            Writeln(F, ', ')
          else
            Writeln(F);
      end;
  Writeln(F, ');');
  }
  Close(F);

  //FAll.FreeAll;
  //FAll.Free;
  //FCurve.Free;
  Inherited Destroy;
end;

Procedure TEditorEllipses.Render;
  Var
    I: Integer;
begin
  glColor3f(1, 0, 0);
  //FCurve.Render;
  glColor3f(1, 1, 1);
    //For I := 0 to FAll.High do
    //  FAll[I].Render;
end;

Procedure TEditorEllipses.MouseDown;
begin
  //FCurve.ClearPoints;
  //FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
  //FCreating := True;
end;

Procedure TEditorEllipses.MouseUp;
begin
  //FCreating := False;
  //FAll.Add(FCurve);
  //FCurve := TCurve2D.Create(Self);
end;

Procedure TEditorEllipses.MouseMoved(X, Y: Integer);
begin
    //If FCreating then begin
    //  FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
    //end;
end;

Procedure TEditorEllipses.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEventMouseButton') then
      With TEventMouseButton(Event) do begin
        If (Key = 0) And Down then
          MouseDown;
        If (Key = 0) And (Not Down) then
          MouseUp;
      end;
    If Event.IsType('TEvMouseMoved') then
      With TEvMouseMoved(Event) do
        MouseMoved(X, Y);
end;


initialization
  Factory.RegistClass(TEditorLines);
end.
