unit GEnemy;

interface

Uses
  {$I Units.inc};

implementation

Type
TAsteroid = class(TDecorator)
private
  FPos: TVec2D;
  FVel: TVec2D;
  //FCir: TCircle;
  FRect: TRect;  
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TAsteroid.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FPos := Vec2D(320, Random(240));
  FVel := Mul2D(Norm2D(Sub2D(Vec2D(160, 120), FPos)), 0.5);
  //FCir := TCircle.Create(Self);
  //FCir.Rad := 5;
  FRect := TRect.Create(Self);
  FRect.A := Vec2D(0, 0);
  FRect.B := Vec2D(10, 10);
end;

Destructor TAsteroid.Destroy;
begin
  //FCir.Free;
  FRect.Free;
  Inherited Destroy;
end;

Procedure TAsteroid.Render; 
begin
  glColor3f(1, 0, 0);
  glRectf(FPos.X-5, FPos.Y-5, FPos.X+5, FPos.Y+5);
end;

Procedure TAsteroid.Update; 
begin
  FPos := Add2D(FPos, FVel);
  //FCir.Cen := FPos;
  FRect.SetPos(FPos);
    If (FPos.X <  -10)Or(FPos.Y < -10)Or
       (FPos.X >  330)Or(FPos.Y > 250) then
      Decoratable.Kill;
end;

Procedure TAsteroid.OnEvent(Event: TEvent);
  Var
    Collision: TCollision;
    Tar, Nor, Proj, Dop: TVec2D;
    D: Double;
    Line: TLine2D;
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvReqMathBody') then 
      TEvReqMathBody(Event).Body := FRect;
    If Event.IsType('TEvCollided') then begin
      Collision := TEvCollided(Event).Col;
        If Collision.PointsCount < 0 then
          Exit;
      Tar := Norm2D(Sub2D(Collision.GetPoint(0), Collision.GetPoint(1)));
      Line := TLine2D.Create(Self);
      Line.ComputePoints(Collision.GetPoint(0), Collision.GetPoint(1));
        If Line.GetDist(FPos) * Dot2D(Line.N, FVel) > 0 then
	  Exit;
      Line.Free;
      Proj := Mul2D(Tar, Dot2D(FVel, Tar));
      Dop := Sub2D(Proj, FVel);
      FVel := Add2D(FVel, Mul2D(Dop, 2));
    end;
    If Event.IsType('TEvSetPos') then begin
      FPos := TEvSetPos(Event).Pos;
    end;
end;

Type
TRespawn = class(TGameDecorator)
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TRespawn.OnEvent(Event: TEvent);
begin
    If Event.IsType('TEvDestroying') then 
      With TEvDestroying(Event) do
        If Entity is TAsteroid then
	  Game.DoScript('DATA\Scripts\CreateAsteroid.dsc');
end;

initialization
  Factory.RegistClass(TAsteroid);
  Factory.RegistClass(TRespawn);
end.
