{$DEFINE OS_WIN32}
{$APPTYPE CONSOLE}

Uses
  //  
  DDebug in 'Core\DDebug.pas',
  DUtils in 'Core\DUtils.pas',
  DCore in 'Core\DCore.pas',
  DEngine in 'Core\DEngine.pas',
  DMath in 'Core\DMath.pas',
  DParser in 'Core\DParser.pas',
  DBinary in 'Core\DBinary.pas',

  //  
  MCore in 'Math\MCore.pas',
  MCurve in 'Math\MCurve.pas',
  MCollider in 'Math\MCollider.pas',

  //  
  Windows,

  {$IFDEF CC_FPC}
  uFMOD in 'uFMOD_FPC\uFMOD.pas',
  {$ELSE}
  uFMOD in 'uFMOD_BD\uFMOD.pas',
  {$ENDIF}

  {$IFDEF OS_WIN32}
  DWinApi in 'OS_Win32\DWinApi.pas',
  dglOpenGL in 'OS_Win32\dglOpenGL.pas',
  {$ENDIF}
  
  // 
  RPrim in 'Renderer\RPrim.pas',
  RText in 'Renderer\RText.pas',
  RSprite in 'Renderer\RSprite.pas',

  //  
  GEvents in 'GEvents.pas',
  GCore in 'GCore.pas',
  GObjects in 'GObjects.pas',
  GPhys in 'GPhys.pas',
  GObjectForm in 'GObjectForm.pas',
  GPlayer in 'GPlayer.pas',
  GCicloids in 'GCicloids.pas',
  GAudio in 'GAudio.pas',
  GRender in 'GRender.pas',
  GIntro in 'GIntro.pas',

  // 
  GEditors in 'GEditors.pas',

  //  
  GEnemy in 'GEnemy.pas',
  GOven in 'GOven.pas',
  GBlock in 'GBlock.pas',
  GShape in 'GShape.pas';

Var
  CoreEngine: TCoreEngine;

Type
TApp = class(TEntity)                  
private
  FGame: TGame;
  FMapName: String;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TApp.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  CreateWindow(320, 240, 32, 'PowerLine', False);
  
    If ParamCount > 0 then
      FMapName := ParamStr(1)
    else
      FMapName := 'Default';

  FGame := TGame.Create(Self);
  FGame.Load('DATA\Scripts\' + FMapName + '.dgm');

  //  windows   TApp
  WinApi.Subscriber := Self;
end;

Destructor TApp.Destroy; 
begin
  FGame.Free;
  Inherited Destroy;
end;

Procedure TApp.Render; 
begin
  glClearColor(0, 0, 0, 1);
  glClear(GL_COLOR_BUFFER_BIT);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  glOrtho(0, 320, 240, 0, -1, 1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  
  FGame.Render;
  //FGame.Update;
  //FGame.SendEventAndFree(TEvPhysStep.Create(Self));

  glPointSize(4);
    glBegin(GL_POINTS);
      glColor4f(1, 1, 1, 1);
      glVertex2f(Mouse.X, Mouse.Y);
    glEnd;
  glPointSize(1);

  SwapBuffers(DC);
end;

Procedure TApp.Update;
begin
    If Keys[27] then
      CoreEngine.StopMainLoop;
  FGame.Update;
    If FGame.Exiting then
      CoreEngine.StopMainLoop;
end;

Procedure TApp.OnEvent(Event: TEvent);
begin
  FGame.SendEvent(Event);
end;

Var
  Main: TList;
  App: TApp;

begin
  Randomize;
  //Log.UseLogFile := False;
  CoreEngine := TCoreEngine.Create(Nil);
  Main := TList.Create(Nil);
  Main.Add(WinApi);
  App := TApp.Create(Nil);
  Main.Add(App);
    try
      CoreEngine.StartMainLoop(Main, 25);
    except
      Writeln('Exeption in CoreEngine.StartMainLoop');
    end;
  Main.Free;
  App.Free;
end.
