unit GObjects;

interface

Uses
  {$I Units.inc};

implementation

//
//  ,      
//
Type
TGameTimer = class(TGameDecorator)
private
  FTime: Integer;
  FScr: String;
public
  Procedure AssignVar(Name, Value: String); override;
  Procedure Update; override;
end;

Procedure TGameTimer.AssignVar;
  Var
    I: Integer;
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then begin
      FTime := StrToInt(Value);
    end;
    If Name = 'scr' then begin
      FScr := '';
        For I := 1 to Length(Value) do
          Case Value[I] of
            ';': FScr := FScr + ',';
            ':': FScr := FScr + '=';
            '_': FScr := FScr + '/';
          else
            FScr := FScr + Value[I];
          end;
    end;
end;

Procedure TGameTimer.Update;
begin
  Inherited Update;
    If Game.GetTime > FTime then begin
      Game.DoString(FScr);
      Decoratable.Kill;
      //Log.PrintLn(FScr);
    end;
end;

//
//  ,      
//
Type
TLineMirrored = class(TGameDecorator)
private
  FCollidedPoint: TVec2D;
  FAim: String;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TLineMirrored.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FCollidedPoint := Vec2D(-10, -10);
  FAim := 'TMouseCatcher';
end;

Procedure TLineMirrored.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'aim' then
      FAim := Value;
end;

Procedure TLineMirrored.OnEvent(Event: TEvent);
  Var
    ReqPos: TEvReqPos;
    ReqVel: TEvReqVel;
    Pos: TVec2D;
    Vel: TVec2D;
    Collision: TCollision;
    Tar, Nor, Proj, Dop: TVec2D;
    D: Double;
    Line: TLine2D;
    I, J: Integer;
    MI, MJ: Integer;
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvCollided') then begin
        With TEvCollided(Event) do
          With TDecorator(Ent).Decoratable do
            If Not HasDecorator(FAim) then
              Exit;
      Collision := TEvCollided(Event).Col;
        If Collision.PointsCount < 2 then
          Exit;

      MI := 0;
      MJ := 1;
        For I := 0 to Collision.PointsCount - 1 do
          For J := I+1 to Collision.PointsCount - 1 do
            If Dist2D(Collision.GetPoint(I), Collision.GetPoint(J)) >
               Dist2D(Collision.GetPoint(MI), Collision.GetPoint(MJ)) then begin
              MI := I;
              MJ := J;
            end;

        //If Dist2D(Collision.GetPoint(MI), Collision.GetPoint(MJ)) < 1 then
        //  Exit;
  
      ReqPos := TEvReqPos.Create(Self);
      Decoratable.SendEvent(ReqPos);
      Pos := ReqPos.Pos;
      ReqPos.Free;

        If Dist2D(FCollidedPoint, Pos) < 20 then
          Exit;

      //Log.PrintLn(IntToStr(GetTimer) + ' Collided!');
      
      ReqVel := TEvReqVel.Create(Self);
      Decoratable.SendEvent(ReqVel);
      Vel := ReqVel.Vel;
      ReqVel.Free;
      
      Tar := Norm2D(Sub2D(Collision.GetPoint(MI), Collision.GetPoint(MJ)));
      Line := TLine2D.Create(Self);
      Line.ComputePoints(Collision.GetPoint(MI), Collision.GetPoint(MJ));
        {
        If Line.GetDist(Pos) * Dot2D(Line.N, Vel) > 0 then
          Exit;
        If Abs(Line.GetDist(Pos) * Dot2D(Line.N, Vel)) < 10e-6 then
          Log.PrintLn('0!!!!!!');
        }
        With Line do
          Vel := Sub2D(Vel, Mul2D(N, 2*Dot2D(Vel, N)));
      Line.Free;
      {
      Proj := Mul2D(Tar, Dot2D(Vel, Tar));
      Dop := Sub2D(Proj, Vel);
      Vel := Add2D(Vel, Mul2D(Dop, 2));
      }
      Decoratable.SendEventAndFree(TEvSetVel.Create(Self, Vel));

      FCollidedPoint := Pos;
    end;
end;

//
//  ,        
//
Type
TLineBreaker = class(TGameDecorator)
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TLineBreaker.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvCollided') then begin
      With TEvCollided(Event) do
        If TDecorator(Ent).Decoratable.HasDecorator('TMouseCatcher') then
          TDecorator(Ent).Decoratable.
             SendEventAndFree(TEvLineBreaked.Create(Self));
    end;
end;

//
// THp - ,   TEvDamage      
//
Type
THp = class(TGameDecorator)
private
  FHp, FMax: Double;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor THp.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FMax := 100;
  FHp := FMax;
end;

Procedure THp.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'hp' then
      FHp := StrToInt(Value)
    else If Name = 'max' then
      FMax := StrToInt(Value);
end;

Procedure THp.OnEvent(Event: TEvent);
  Var
    SavedHp: Single; 
begin
  Inherited OnEvent(Event);
  SavedHp := FHp;
    If Event.IsType('TEvDamage') then begin
      FHp := Max(0, FHp - TEvDamage(Event).Damage);
    end;
    If (FHp < SavedHp) And (FHp <= 10e-6) then
      Decoratable.SendEventAndFree(TEvKilled.Create(Self));

    If Event.IsType('TEvReqHp') then begin
      TEvReqHp(Event).Hp := FHp;
      TEvReqHp(Event).Max := FMax;
    end;
end;

//
//  Bar -   
//
Type
TBar = class(TGameDecorator)
protected
  FColor: TVec3D;
  FPos: TVec2D;
  FSize: TVec2D;
  FVal: Single;
  FMax: Single;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Constructor TBar.Create;
begin
  Inherited Create(Owner);
  FPos := Vec2D(20, 40);
  FSize := Vec2D(100, 10);
  FColor := Vec3D(1, 0, 0);
end;

Procedure TBar.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'x' then
      FPos.X := StrToFloat(Value)
    else If Name = 'y' then
      FPos.Y := StrToFloat(Value)
    else If Name = 'w' then
      FSize.X := StrToFloat(Value)
    else If Name = 'h' then
      FSize.Y := StrToFloat(Value)
    else If Name = 'r' then
      FColor.X := StrToFloat(Value)
    else If Name = 'g' then
      FColor.Y := StrToFloat(Value)
    else If Name = 'b' then
      FColor.Z := StrToFloat(Value);
end;

Procedure TBar.Render;
  Var
    ReqHp: TEvReqHp;
begin
  Inherited Render;
  DrawProgressBar(FPos, FSize, FColor, FVal/FMax);
end;

//
// Bar for Hp
//
Type
THpBar = class(TBar)
public
  Constructor Create(Owner: TEntity); override;
  Procedure Update; override;
end;

Constructor THpBar.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FVal := 1;
  FMax := 1;
end;

Procedure THpBar.Update;
  Var
    ReqHp: TEvReqHp;
begin
  Inherited Update;
  ReqHp := TEvReqHp.Create(Self);
  Decoratable.SendEvent(ReqHp);
  FVal := ReqHp.Hp;
  FMax := ReqHp.Max;
  ReqHp.Free;
end;

//
// Bar for line length
//
Type
TLineBar = class(TBar)
public
  Procedure Update; override;
end;

Procedure TLineBar.Update;
  Var
    ReqLen: TEvReqLineLen;
begin
  Inherited Update;
  ReqLen := TEvReqLineLen.Create(Self);
  Decoratable.SendEvent(ReqLen);
  FVal := ReqLen.Len;
  FMax := ReqLen.Max;
  ReqLen.Free;
end;

//
//   ,    
//
Type
TOutKiller = class(TGameDecorator)
private
  FXOffset: Single;
  FYOffset: Single;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Update; override;
end;

Constructor TOutKiller.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FXOffset := 20;
  FYOffset := 20;
end;

Procedure TOutKiller.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'offset' then begin
      FXOffset := StrToInt(Value);
      FYOffset := StrToInt(Value);
    end;
end;

Procedure TOutKiller.Update;
  Var
    ReqPos: TEvReqPos;
begin
  Inherited Update;
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);
    With ReqPos.Pos do  
      If ((X < -FXOffset)Or(X > 340 + FXOffset))Or
         ((Y < -FYOffset)Or(Y > 240 + FYOffset)) then begin
        Decoratable.Kill;
      end;
  ReqPos.Free;
end;

//
//       
//
Type
TTimeKiller = class(TGameDecorator)
private
  FTime: Integer;
  FScr: String;
public
  Procedure AssignVar(Name, Value: String); override;
  Procedure Update; override;
end;

Procedure TTimeKiller.AssignVar;
  Var
    I: Integer;
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then begin
      FTime := StrToInt(Value);
    end;
end;

Procedure TTimeKiller.Update;
begin
  Inherited Update;
    If Game.GetTime > FTime then
      Decoratable.Kill;
end;

//
//        
//
Type
TDamagable = class(TGameDecorator)
private
  FDamage: Single;
  FAim: String;
public
  Procedure DoString(S: String); override;
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TDamagable.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
begin
  Inherited DoString(S);
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
        If Pos('damage=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FDamage := StrToInt(List[I]);
        end;
        If Pos('aim=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FAim := List[I];
        end;
    end;
end;

Procedure TDamagable.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvCollided') then begin
      With TEvCollided(Event) do
        If TDecorator(Ent).Decoratable.HasDecorator(FAim) then begin
          TDecorator(Ent).Decoratable.
             SendEventAndFree(TEvDamage.Create(Self, FDamage));
          Decoratable.Kill;
        end;
    end;
end;

//
//   -    
//
Type
TPositer = class(TGameDecorator)
private
  FPos: TVec2D;
public
  Procedure DoString(S: String); override;
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TPositer.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
begin
  Inherited DoString(S);
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
        If Pos('x=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FPos.X := StrToInt(List[I]);
        end;
        If Pos('y=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FPos.Y := StrToInt(List[I]);
        end;
    end;
end;

Procedure TPositer.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvSetPos') then
      FPos := TEvSetPos(Event).Pos;
    If Event.IsType('TEvReqPos') then
      TEvReqPos(Event).Pos := FPos;
end;

//
//   
//
Type
TBlackScreen = class(TGameDecorator)
private
  FStart: Integer;
  FTime: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Constructor TBlackScreen.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FStart := GetTimer;
  FTime := 10;
end;

Procedure TBlackScreen.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then
      FTime := StrToInt(Value);
end;

Procedure TBlackScreen.Render;
begin
  Inherited Render;
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    If GetTimer > FStart + FTime then
      glColor4f(0, 0, 0, 1)
    else
      glColor4f(0, 0, 0, (GetTimer-FStart)/FTime); 
  glRectf(0, 0, 320, 240);
  glColor4f(1, 1, 1, 1);
  glDisable(GL_BLEND);
end;

//
//   -     ;)
//
Type
TVelocity = class(TGameDecorator)
private
  FVel: TVec2D;
public
  Procedure Update; override;
  Procedure DoString(S: String); override;
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TVelocity.Update;
  Var
    ReqPos: TEvReqPos;
begin
  Inherited Update;
  ReqPos := TEvReqPos.Create(Self);
  Decoratable.SendEvent(ReqPos);
  Decoratable.SendEventAndFree(TEvSetPos.Create(Self, Add2D(ReqPos.Pos, FVel)));
  ReqPos.Free;
end;

Procedure TVelocity.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
begin
  Inherited DoString(S);
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
        If Pos('x=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FVel.X := StrToFloat(List[I])*1.5;
        end;
        If Pos('y=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FVel.Y := StrToFloat(List[I])*1.5;
        end;
    end;
end;

Procedure TVelocity.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvSetVel') then
      FVel := TEvSetVel(Event).Vel;
    If Event.IsType('TEvReqVel') then
      TEvReqVel(Event).Vel := FVel;
end;

{
Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      Fill: Boolean = True); overload;
  Var
    I: Integer;
begin
    If Fill then      
      glBegin(GL_POLYGON)
    else
      glBegin(GL_LINE_STRIP);
    For I := 0 to Sides-1 do
      glVertex2f(X + RadX*Cos(2*pi*I/Sides),
                 Y + RadY*Sin(2*pi*I/Sides));
    If Not Fill then
      glVertex2f(X + RadX, Y);
  glEnd;
end;

Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      R1, G1, B1, A1: Single;
                      R2, G2, B2, A2: Single); overload;
  Var
    I: Integer;
begin
  glBegin(GL_TRIANGLE_FAN);
    glColor4f(R1, G1, B1, A1);
    glVertex2f(X, Y);
    glColor4f(R2, G2, B2, A2);
      For I := 0 to Sides do
        glVertex2f(X + RadX*Cos(2*pi*I/Sides),
                   Y + RadY*Sin(2*pi*I/Sides));
  glEnd;
end;

Procedure DrawPixel(X, Y, R, G, B: Single);
begin
  glPointSize(1);
  glBegin(GL_POINTS);
    glColor3f(R, G, B);
    glVertex2f(X, Y);
  glEnd;
end;

Procedure DrawBubble(X, Y, Size: Single; R, G, B: Single);
begin
  DrawEllipse(X, Y, Size, Size, 15,
              R, G, B, 1,
              R+0.1, G+0.1, B+0.1, 1);  
end;
}
//
//  TSubmRender
//
Type
TSubmRender = class(TGameDecorator)
private
  R, G, B: Single;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Render; override;
end;

Constructor TSubmRender.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  R := 0.75;
  G := 0.75;
  B := 0.0;
end;

Procedure TSubmRender.Render;
  Procedure DrawIll(X, Y: Single);
  begin
    DrawEllipse(X, Y, 10, 10, 20,
                   0,    0, 0.75, 1,
                   0,    0,    1, 1);
    glColor3f(0, 0, 0);
    glLineWidth(1);
    DrawEllipse(X, Y, 10, 10, 20,
                False);
  end;
  Procedure DrawT(X, Y: Single);
    Const
      h = 20;
  begin
    glBegin(GL_QUADS);
      glColor3f(1, 1, 0);
      glVertex2f(X-5, Y-h);
      glColor3f(0.75, 0.75, 0);
      glVertex2f(X+5, Y-h);
      glColor3f(0.75, 0.75, 0);
      glVertex2f(X+5, Y);
      glColor3f(1, 1, 0);
      glVertex2f(X-5, Y);
    glEnd;
  end;
begin
  DrawT(160, 110);

  DrawEllipse(160, 120, 40, 20, 60,
                 1,    1, 0, 1,
              0.75, 0.75, 0, 1);
  glColor3f(0, 0, 0);
  glLineWidth(1);
  DrawEllipse(160, 120, 40, 20, 60,
              False);

  DrawIll(145, 120);
  DrawIll(175, 120);

  glColor3f(1, 1, 1);
end;

Type
TGar = packed record
  Start: Integer;
  X, Y: Single;
  VX, VY: Single;
  Typ: Integer;
end;

TBackground = class(TGameDecorator)
private
  FGars: Array[0..100] Of TGar;
  FGround: Array Of TVec2D;
  FGroundBody: TCurve2D;
  FUpBody: TCurve2D;
  FLastUpdate: Integer;
  FLast: Integer;
  FSpeed: Integer;
  FMode: Integer;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure DoString(S: String); override;
  Procedure Render; override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TBackground.Create(Owner: TEntity);
  Var
    I: Integer;
begin
  Inherited Create(Owner);
    For I := Low(FGars) to High(FGars) do
      FGars[I].Y := -1;
  FSpeed := 100;
  FLastUpdate := GetTimer;
  FLast := GetTimer;
  FMode := 1;
  FGroundBody := TCurve2D.Create(Self);
  FUpBody := TCurve2D.Create(Self);
end;

Destructor TBackground.Destroy;
begin
  FUpBody.Free;
  FGroundBody.Free;
  Inherited Destroy;
end;

Procedure TBackground.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
begin
  Inherited DoString(S);
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
        If Pos('mode=', List[I]) = 1 then begin
          List[I] := StringDeletePre(List[I], Pos('=', List[I]));
          FMode := StrToInt(List[I]);
          //Log.PrintLn(List[I]);
          //Log.PrintLn('mode = ' + IntToStr(FMode));
        end;
    end;
end;

Procedure TBackground.Render;
  Procedure DrawVodr(X, Y: Single; Typ: Integer);
  begin
      If Typ = 2 then begin
        DrawBubble(X,    Y, 3, 0, 0, 0.25);
        DrawBubble(X+3,  Y-5, 3, 0, 0, 0.25);
        DrawBubble(X-2, Y-10, 3, 0, 0, 0.25);
        Exit;
      end;
    glColor3f(0, 0.5, 0);
    glBegin(GL_LINES);
      glVertex2f(    X, Y);
      glVertex2f(    X, Y - 10);
        Case Typ of
          0: begin
               glVertex2f(    X, Y - 10);
               glVertex2f(X - 5, Y - 17);
             end;
          1: begin
               glVertex2f(    X, Y - 10);
               glVertex2f(X + 5, Y - 17);
             end;
        end;
    glEnd;
    glColor3f(0, 0, 0);
  end;
  Var
    I: Integer;
begin
  Inherited Render;
  glColor3f(0, 0, 0.15);
  //glRectf(0, 0, 320, 240);
  glBegin(GL_QUADS);
    glColor3f(0, 0, 0.30);
    glVertex2f(0, 0);
    glVertex2f(320, 0);
    glColor3f(0, 0, 0.15);
    glVertex2f(320, 240);
    glVertex2f(0, 240);
  glEnd;
    For I := Low(FGars) to High(FGars) do
      With FGars[I] do begin
          If Y < 0 then 
            Continue;
        DrawVodr(X, Y, Typ);
      end;
    If (FMode = 2) then begin
      glColor3f(0.75, 0.5, 0.0);
      glBegin(GL_QUADS);
        For I := 0 to High(FGround)-1 do begin
          glColor3f(0.55, 0.3, 0.0);
          glVertex2fv(@FGround[I]);
          glVertex2fv(@FGround[I+1]);
          glColor3f(0.75, 0.5, 0.0);
          glVertex2f(FGround[I+1].X, 240);
          glVertex2f(FGround[I].X, 240);
        end;
      glEnd;
    end;
    If (FMode = 3) then begin
      glBegin(GL_QUADS);
          glColor3f(0.55, 0.3, 0.0);
          glVertex2f(0, 200);
          glVertex2f(320, 200);
          glColor3f(0.75, 0.5, 0.0);
          glVertex2f(320, 240);
          glVertex2f(0, 240);

          glColor3f(0.55, 0.3, 0.0);
          glVertex2f(0, 40);
          glVertex2f(320, 40);
          glColor3f(0.75, 0.5, 0.0);
          glVertex2f(320, 0);
          glVertex2f(0, 0);
      glEnd;
    end;
end;

Procedure TBackground.Update;
  Var
    I, J, NeedNew: Integer;
begin
  Inherited Update;
    For I := Low(FGars) to High(FGars) do
      With FGars[I] do begin
        X := X + (GetTimer - Start)/1000*VX;
        Y := Y + (GetTimer - Start)/1000*VY;
        Start := GetTimer;
      end;
  NeedNew := (GetTimer - FLast) div 800;
    For I := Low(FGars) to High(FGars) do
      With FGars[I] do begin
          If NeedNew <= 0 then
            Break;
          If Y >= 0 then
            Continue;
          Case FMode of  
            1: begin
                 VX := 0;
                 VY := -FSpeed;
                 X := Random(320);
                 Y := 260;
               end;
            2, 3: begin
                 X := 340;
                 Y := Random(240);
                 VX := -FSpeed;
                 VY := 0;
               end;
          end;
        Typ := Random(3);
        Start := GetTimer;
        Dec(NeedNew);
        FLast := GetTimer;
      end;

    If FMode = 2 then begin
        If Length(FGround) < 2 then begin
          SetLength(FGround, 2);
          FGround[0] := Vec2D(320, 240);
          FGround[1] := Vec2D(380, 220);  
        end;
        For I := 0 to High(FGround) do
          FGround[I].X := FGround[I].X -
              (GetTimer - FLastUpdate)/1000*FSpeed;
      I := 0;
        While (I < High(FGround)-1)And
              (FGround[I].X < 0)And
              (FGround[I+1].X < 0) do
          Inc(I);
      Dec(I);
        If I > 0 then begin
            For J := 0 to High(FGRound)-I-1 do
              FGRound[J] := FGRound[J+I+1];
          SetLength(FGround, Length(FGround)-(I+1));
        end;
        If FGround[High(FGround)].X < 400 then begin
          SetLength(FGround, Length(FGround) + 1);
          FGround[High(FGround)].Y := 200 + Random(40);
            //If FGround[High(FGround)].Y > FGround[High(FGround)-1].Y then
            //  FGround[High(FGround)].X := FGround[High(FGround)-1].X + 20 + Random(50)
            //else
            //  FGround[High(FGround)].X := FGround[High(FGround)-1].X + Random(10);
          FGround[High(FGround)].X := FGround[High(FGround)-1].X + Random(50);
        end;
        //For J := High(FGround) downto 0 do
        //  FGround[J] := FGround[J-I];
      FGroundBody.ClearPoints;
        For I := 0 to High(FGround) do
          FGroundBody.AddPoint(FGround[I]);
    end;
    If FMode = 3 then begin
      FGroundBody.ClearPoints;
      FGroundBody.AddPoint(Vec2D(0, 200));
      FGroundBody.AddPoint(Vec2D(10e+6, 200));
      FGroundBody.AddPoint(Vec2D(10e+6, 40));
      FGroundBody.AddPoint(Vec2D(0, 40));
    end;

  FLastUpdate := GetTimer;
end;

Procedure TBackground.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvReqMathBody') then begin
      TEvReqMathBody(Event).Body := FGroundBody;
    end;
end;

//
//  TGameOver
//
Type
THpGameOver = class(TGameDecorator)
private
  FHappend: Boolean;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Update; override;
end;

Constructor THpGameOver.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FHappend := False;
end;

Procedure THpGameOver.Update;
  Var
    ReqHp: TEvReqHp;
begin
  Inherited Update;
    If FHappend then
      Exit;
  ReqHp := TEvReqHp.Create(Self);
  Decoratable.SendEvent(ReqHp);
    If ReqHp.Hp < 10e-6 then begin
      Game.SendEvent(TEvGameOver.Create(Self));
      FHappend := True;
    end;
end;

//
//  THpGameWin
//
Type
THpGameWin = class(TGameDecorator)
private
  FHappend: Boolean;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Update; override;
end;

Constructor THpGameWin.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FHappend := False;
end;

Procedure THpGameWin.Update;
  Var
    ReqHp: TEvReqHp;
begin
  Inherited Update;
    If FHappend then
      Exit;
  ReqHp := TEvReqHp.Create(Self);
  Decoratable.SendEvent(ReqHp);
    If ReqHp.Hp < 10e-6 then begin
      Game.SendEvent(TEvGameWin.Create(Self));
      FHappend := True;
    end;
end;

Type
TOnGameOver = class(TGameDecorator)
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TOnGameOver.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvGameOver') then begin
      Game.DoString('@BlackScreen = TBlackscreen!time=4000');
      Game.DoString('@Restart = TGameTimer!time=' + IntToStr(Game.GetTime+4000) +
                    '/scr=^' + Game.MapName);
    end;
end;

Type
TOnGameWin = class(TGameDecorator)
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TOnGameWin.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvGameWin') then begin
      Game.DoString('@BlackScreen = TBlackscreen!time=4000');
      Game.DoString('@Win = TGameTimer!time=' + IntToStr(Game.GetTime+4000) +
                    '/scr=^Win');
    end;
end;

Type
TExitCondition = class(TGameDecorator)
  Procedure Update; override;
end;

Procedure TExitCondition.Update;
  Var
    ReqLen: TEvReqLineLen;
begin
  Inherited Update;
  ReqLen := TEvReqLineLen.Create(Self);
  Decoratable.SendEvent(ReqLen);
    If ReqLen.Len/ReqLen.Max > 0.7 then begin
      Game.DoString('@BlackScreen = TBlackScreen!time=3000');
      Game.DoString('@Exit = TGameTimer!time=' + IntToStr(Game.GetTime+3000) +
                    '/scr=Exit');
      Decoratable.Kill;
    end;
  ReqLen.Free;
end;

initialization
  Factory.RegistClass(TGameTimer);
  Factory.RegistClass(TLineMirrored);
  Factory.RegistClass(TLineBreaker);
  Factory.RegistClass(TOutKiller);
  Factory.RegistClass(TTimeKiller);
  Factory.RegistClass(TDamagable);
  Factory.RegistClass(THp);
  Factory.RegistClass(THpBar);
  Factory.RegistClass(TLineBar);
  Factory.RegistClass(TPositer);
  Factory.RegistClass(TBlackScreen);
  Factory.RegistClass(TVelocity);
  Factory.RegistClass(TSubmRender);
  Factory.RegistClass(TBackground);
  Factory.RegistClass(THpGameOver);
  Factory.RegistClass(THpGameWin);
  Factory.RegistClass(TOnGameOver);
  Factory.RegistClass(TOnGameWin);
  Factory.RegistClass(TExitCondition);
end.
