unit GPhys;

interface

Uses
  {$I Units.inc};

implementation

Type
TBody = class(TGameDecorator)
private
  FPos: TVec2D;
  FBody: TMathBody;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
  Property Body: TMathBody read FBody;
end;

Constructor TBody.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FBody := Nil;
end;

Procedure TBody.Update;
  Var
    ReqBody: TEvReqMathBody;
begin
  ReqBody := TEvReqMathBody.Create(Self);
  Decoratable.SendEvent(ReqBody);
    If ReqBody.Body <> Nil then
      FBody := ReqBody.Body;
  ReqBody.Free;
end;

Procedure TBody.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvAddMathBody') then
      FBody := TEvAddMathBody(Event).Body
    else If Event.IsType('TEvDestroying') then begin
      With TEvDestroying(Event) do
        If Entity = FBody then
	  FBody := Nil;    
    end else If Event.IsType('TEvSetPos') then begin
      //FPos := TEvSetPos(Event).Pos;
    end else If Event.IsType('TEvReqPos') then begin
      //TEvReqPos(Event).Pos := FPos;
    end;
end;

//
//  TPhys -    
//
//      
//        1)     TBody  
//        2)  update  
//               ,  -
//           ,   
//            TEvCollided (. GEvents.pas)
//           ,   
//            (     TDecoratable')
Type
TPhys = class(TDecorator)
private
  FBodies: TList;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TPhys.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FBodies := TList.Create(Self);
end;

Destructor TPhys.Destroy;
begin
  FBodies.Free;
  Inherited Destroy;
end;

Procedure TPhys.Update;
  Var
    I, J: Integer;
    Collision: TCollision;
begin
  Inherited Update;
  //Writeln(FBodies.Length);
    For I := 0 to FBodies.High do
      For J := I + 1 to FBodies.High do begin
          If (TBody(FBodies[I]).Body = Nil)Or
	     (TBody(FBodies[J]).Body = Nil) then
	    Continue;
	  try
            Collision := Collider.Collide(TBody(FBodies[I]).Body, 
	                                  TBody(FBodies[J]).Body);
	  except
	    Log.PrintLn('Error in TPhys.Update');
	  end;
	  If (Collision <> Nil)And(Collision.Test) then begin
            //Writeln('Collision in');
	    FBodies[I].Owner.SendEventAndFree(
		       TEvCollided.Create(Self, Collision, FBodies[J]));
            FBodies[J].Owner.SendEventAndFree(
		       TEvCollided.Create(Self, Collision, FBodies[I]));
	    //Writeln('Sent EvCollided');
            Collision.Free;
	  end;
      end;
end;

Procedure TPhys.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);  
     If Event.IsType('TEvCreated') then 
        With TEvCreated(Event) do begin
            If Entity is TBody then
	      FBodies.Add(Entity);  
        end
      else If Event.IsType('TEvDestroying') then 
        With TEvDestroying(Event) do begin
            If Entity is TBody then
	      FBodies.Del(Entity);
	  //Writeln('Deleted!');
        end
      else If Event.IsType('TEvPhysStep') then
        Update;
end;

initialization
  Factory.RegistClass(TBody);
  Factory.RegistClass(TPhys);
end.
