unit GPlayer;

interface

Uses
  {$I Units.inc};

implementation

//
//  TSpaceShip
//
Type
TSpaceShip = class(TDecorator)
private
  FWidth: Single;
  FHeight: Single;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Render; override;
end;

Constructor TSpaceShip.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FWidth := 320/6;
  FHeight := 240/5
end;

Procedure TSpaceShip.Render;
begin
  glColor3f(0, 0, 0.75);
  glRectf(160 - FWidth/2,
	  120 - FHeight/2,
          160 + FWidth/2,
	  120 + FHeight/2);
end;

//
//  TMouseCatcher
//  
Type
TMouseCatcher = class(TDecorator)
private
  FCurve: TCurve2D;
  FLod: TCurve2D;
  FMaxLen: Double;
  FCreating: Boolean;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure MouseDown;
  Procedure MouseUp;
  Procedure MouseMoved(X, Y: Integer);
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TMouseCatcher.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FCurve := TCurve2D.Create(Self);
  FLod := TCurve2D.Create(Self);
  FMaxLen := 80;
end;

Destructor TMouseCatcher.Destroy;
begin
  FLod.Free;
  FCurve.Free;
  Inherited Destroy;
end;

Procedure TMouseCatcher.Render;
begin 
  glColor3f(1, 1, 1);
  FCurve.Render;
  //FLod.Render;
end;

Procedure TMouseCatcher.MouseDown;
begin
  FCurve.ClearPoints;
  FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
  FLod.CopyLod(FCurve);
  FCreating := True;
end;

Procedure TMouseCatcher.MouseUp;
begin
  FCreating := False;
end;

Procedure TMouseCatcher.MouseMoved(X, Y: Integer);
begin
    If FCreating then begin
      FCurve.AddPoint(Vec2D(Mouse.X, Mouse.Y));
	      If FCurve.Len > FMaxLen then
	        FCreating := False;
      //FLod.CopyLod(FCurve);
      FCurve.CutToLen(FMaxLen);
    end;
end;

Procedure TMouseCatcher.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEventMouseButton') then 
      With TEventMouseButton(Event) do begin
        If (Key = 0) And Down then
          MouseDown;
        If (Key = 0) And (Not Down) then
          MouseUp;
      end;
    If Event.IsType('TEvMouseMoved') then
      With TEvMouseMoved(Event) do
        MouseMoved(X, Y);
    If Event.IsType('TEvReqMathBody') then
      TEvReqMathBody(Event).Body := FCurve;
    If Event.IsType('TEvLineBreaked') then begin
      FCreating := False;
      FCurve.ClearPoints;
    end;
    If Event.IsType('TEvReqLineLen') then begin
      TEvReqLineLen(Event).Len := FCurve.Len;
      TEvReqLineLen(Event).Max := FMaxLen;
    end;
end;

//
//  TMousePositer
//
Type
TMousePositer = class(TGameDecorator)
  Procedure Update; override;
end;

Procedure TMousePositer.Update;
begin
  Inherited Update;
  Owner.SendEventAndFree(TEvSetPos.Create(Self, Vec2D(Mouse.X, Mouse.Y)));
end;

//
//  TSubmForm
//
Type
TSubmForm = class(TObjectForm)
protected
  Procedure CreateBody; override; 
end;

Procedure TSubmForm.CreateBody;
begin
  FBody := TRect.Create(Self);
  TRect(FBody).A := Sub2D(Vec2D(160, 120), Vec2D(35, 15));
  TRect(FBody).B := Add2D(Vec2D(160, 120), Vec2D(35, 15));
end;

//
//  TSubm
//
Type
TSubm = class(TGameDecorator)
  Procedure OnEvent(Event: TEvent); override;
end;

Procedure TSubm.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    {
    If Event.IsType('TEvCollided') then begin
      //Log.PrintLn(TEvCollided(Event).Ent.ClassName);
      With TEvCollided(Event) do
        If TDecorator(Ent).Decoratable.HasDecorator(TMouseCatcher) then
          TDecorator(Ent).Decoratable.
             SendEventAndFree(TEvLineBreaked.Create(Self));
    end;
    }
end;

initialization
  Factory.RegistClass(TSpaceShip);
  Factory.RegistClass(TMouseCatcher);
  Factory.RegistClass(TMousePositer);
  Factory.RegistClass(TSubmForm);
  Factory.RegistClass(TSubm);
finalization
end.
