unit GShape;

interface

Uses
  {$I Units.inc};

implementation

Type
TShapeCircle = class(TDecorator)
private
  FPos: TVec2D;
  FVel: TVec2D;
  FCir: TCircle;  
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure Update; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TShapeCircle.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FPos := Vec2D(0, 0);
  FCir := TCircle.Create(Self);
  FCir.Rad := 10;
end;

Destructor TShapeCircle.Destroy;
begin
  FCir.Free;
  Inherited Destroy;
end;

Procedure TShapeCircle.Render; 
begin
  Inherited Render;
  glColor3f(1, 0, 0);
  glRectf(FPos.X-FCir.Rad, FPos.Y-FCir.Rad, FPos.X+FCir.Rad, FPos.Y+FCir.Rad);
end;

Procedure TShapeCircle.Update; 
begin
  Inherited Update;
  FPos := Add2D(FPos, FVel);
  FCir.Cen := FPos;
end;

Procedure TShapeCircle.OnEvent(Event: TEvent);
  Var
    Collision: TCollision;
    Tar, Nor, Proj, Dop: TVec2D;
    D: Double;
    Line: TLine2D;
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvReqMathBody') then 
      TEvReqMathBody(Event).Body := FCir;
    If Event.IsType('TEvSetPos') then begin
      FPos := TEvSetPos(Event).Pos;
    end;
end;


initialization
  Factory.RegistClass(TShapeCircle);
end.
