unit MCollider;

interface

Uses
  SysUtils,

  DUtils,
  DDebug,
  DCore,
  DMath,
  DBinary,

  MCore,
  MCurve;

Type
TCollision = class(TEntity)
private
  FTest: Boolean;
  FPoints: Array Of TVec2D;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Function PointsCount: Integer;
  Function GetPoint(ID: Integer): TVec2D;
  Procedure AddPoint(Point: TVec2D);
  Property Test: Boolean read FTest write FTest;
  Property Points[Index: Integer]: TVec2D read GetPoint;
end;


TCollide = class(TBinary)
  Function Collide(A, B: TEntity): TCollision;
end;

TCollider = class(TEntity)
private
  FBinaries: Array Of TBinary;
  Procedure AddBinary(AType, BType: TClassEntity; Func: TBinaryFunc);
public
  Constructor Create(Owner: TEntity); override;
  Function Collide(A, B: TEntity): TCollision;
  Property PCollide[Index1, Index2: TEntity]: TCollision read Collide; default;
end;

Var
  Collider: TCollider;

Function CollideCircleVsCircle(Circle1, Circle2: TCircle): TCollision;
Function CollideLineVsCircle(Line: TLine2D; Circle: TCircle): TCollision;
Function CollideCurveVsCircle(Curve: TCurve2D; Circle: TCircle): TCollision;
Function CollideCurveVsRect(Curve: TCurve2D; Rect: TRect): TCollision;
Function CollideRest1DVsRest1D(A, B, C, D: Single; Var L, R: Single): Boolean;
Function CollideRectVsRect(A, B: TRect): TCollision;

implementation

Procedure _CollideLineVsCircle(Line: TLine2D;
	                        Circle: TCircle;
			        Var Test: Boolean;	
				Var P1, P2: TVec2D);
  Var
    D: Double;
    P: TVec2D;
    DotT: Double;
    DotN: Double;
    Off: Double;
begin
  D := Line.GetDist(Circle.Cen);
    If Abs(D) < Circle.Rad then 
      With Circle do begin
        Test := True;
        DotT := Dot2D(Line.T, Cen);
        DotN := Dot2D(Line.N, Cen);
        P := Add2D(Mul2D(Line.T, DotT),
                   Mul2D(Line.N, DotN + Sign(DotN)*D));
        Off := Sqrt(Sqr(Rad) - Sqr(D));
	P1 := Add2D(P, Mul2D(Line.T,  Off));
	P2 := Add2D(P, Mul2D(Line.T, -Off));
      end 
    else
      Test := False;  
end;

Function _CollidePointVsARect(P: TVec2D; A, B: TVec2D): Boolean;
begin
    //Writeln('_CollidePointVsARect');
    Result :=  (((A.X <= P.X+10e-6)And(P.X-10e-6 <= B.X))Or
                ((B.X <= P.X+10e-6)And(P.X-10e-6 <= A.X)))And
               (((A.Y <= P.Y+10e-6)And(P.Y-10e-6 <= B.Y))Or
                ((B.Y <= P.Y+10e-6)And(P.Y-10e-6 <= A.Y)));
end;

Procedure _CollideRestVsCircle(A, B: TVec2D;
	                       Circle: TCircle;
			       Var Count: Integer;
			       Var P1, P2: TVec2D);
  Var
    Line: TLine2D;
    Test: Boolean;
begin 
  Count := 0;
  Line := TLine2D.Create(Nil);
  Line.ComputePoints(A, B);
  _CollideLineVsCircle(Line, Circle, Test, P1, P2);
    If Test then begin
      If _CollidePointVsARect(P1, A, B) then
        Inc(Count);
	    If _CollidePointVsARect(P2, A, B) then begin
	        If Count = 0 then
	          P1 := P2;
	      Inc(Count);
	    end;
    end; 
end;

Function _CollideLineVsLine(Line1, Line2: TLine2D;
                            Var P: TVec2D): Boolean;
  Var
    D: Single;
begin
  D := Det2D(Vec2D(Line1.N.X, Line2.N.X), 
             Vec2D(Line1.N.Y, Line2.N.Y));
    If Abs(D) < 10e-6 then
      Result := False 
    else begin
      Result := True;
      D := 1/D;
      P.X := D * Det2D(Vec2D( -Line1.D,  -Line2.D), 
                       Vec2D(Line1.N.Y, Line2.N.Y));
      P.Y := D * Det2D(Vec2D(Line1.N.X, Line2.N.X), 
                       Vec2D( -Line1.D,  -Line2.D));
    end;
end;

Function _CollideRestVsRest(A, B: TVec2D;
                            C, D: TVec2D;
                            Var P: TVec2D): Boolean;
  Var
    Line1, Line2: TLine2D;
begin
  //Log.PrintLn('_Coll...');
  Line1 := TLine2D.Create(Nil);
  Line1.ComputePoints(A, B);
  Line2 := TLine2D.Create(Nil);
  Line2.ComputePoints(C, D);
    If _CollideLineVsLine(Line1, Line2, P) then begin
        If _CollidePointVsARect(P, A, B) And
           _CollidePointVsARect(P, C, D) then
          Result := True
        else
          Result := False;
    end else
      Result := False;
  Line1.Free;
  Line2.Free;
end;

Function b_CollideCircleVsCircle(A, B: TEntity): TEntity;
  Var
    Circle1, Circle2: TCircle;
begin
  Result := TCollision.Create(Nil);
  Circle1 := TCircle(A);
  Circle2 := TCircle(B);
    If Dist2D(Circle1.Cen, Circle2.Cen) < Circle1.Rad + Circle2.Rad then
      TCollision(Result).Test := True
    else
      TCollision(Result).Test := False;
end;

Function b_CollideLineVsCircle(A, B: TEntity): TEntity;
  Var
    Line: TLine2D;
    Circle: TCircle;
begin
  Line := TLine2D(A);
  Circle := TCircle(B);
  Result := CollideLineVsCircle(Line, Circle);
end;

Function b_CollideCurveVsCircle(A, B: TEntity): TEntity;
begin
  Result := CollideCurveVsCircle(TCurve2D(A), TCircle(B));
end;

Function b_CollideCurveVsRect(A, B: TEntity): TEntity;
begin
  Result := CollideCurveVsRect(TCurve2D(A), TRect(B));
end;

Function b_CollideRectVsRect(A, B: TEntity): TEntity;
begin
  Result := CollideRectVsRect(TRect(A), TRect(B));
end;

Function CollideCircleVsCircle(Circle1, Circle2: TCircle): TCollision;
begin
  Result := TCollision(b_CollideCircleVsCircle(Circle1, Circle2));
end;

Function CollideLineVsCircle(Line: TLine2D; Circle: TCircle): TCollision;
  Var
    D: Double;
    P: TVec2D;
    DotT: Double;
    DotN: Double;
    Off: Double;
begin
  Result := TCollision.Create(Nil);
  D := Line.GetDist(Circle.Cen);
    If Abs(D) < Circle.Rad then 
      With Circle do begin
        Result.Test := True;
        DotT := Dot2D(Line.T, Cen);
        DotN := Dot2D(Line.N, Cen);
        P := Add2D(Mul2D(Line.T, DotT),
                   Mul2D(Line.N, DotN + Sign(DotN)*D));
        Off := Sqrt(Sqr(Rad) - Sqr(D));
        Result.AddPoint(Add2D(P, Mul2D(Line.T,  Off)));
        Result.AddPoint(Add2D(P, Mul2D(Line.T, -Off)));
      end 
    else
        Result.Test := False;  
end;

Function CollideCurveVsCircle(Curve: TCurve2D; Circle: TCircle): TCollision;
  Var
    I: Integer;
    Count: Integer;
    P1, P2: TVec2D;
begin
  //Writeln('Curve  circle');
  Result := TCollision.Create(Nil);
    For I := 0 to Curve.PointCount - 2 do begin
      _CollideRestVsCircle(Curve[I], Curve[I+1], Circle, Count, P1, P2);
        If Count > 0 then
	  Result.AddPoint(P1);
	If Count > 1 then
	  Result.AddPoint(P2);
	If Result.PointsCount > 1 then
	  break;
    end;
    If Result.PointsCount > 1 then
      Result.Test := True;
end;

Function CollideCurveVsRect(Curve: TCurve2D; Rect: TRect): TCollision;
  Var
    I: Integer;
    Count: Integer;
    P: TVec2D;
begin
  //Writeln('Curve  circle');
  Result := TCollision.Create(Nil);
    For I := 0 to Curve.PointCount - 2 do begin
      //_CollideRestVsCircle(Curve[I], Curve[I+1], Circle, Count, P1, P2);
      If _CollideRestVsRest(Curve[I], Curve[I+1], Rect.LT, Rect.RT, P) Or
         _CollideRestVsRest(Curve[I], Curve[I+1], Rect.RT, Rect.RB, P) Or
         _CollideRestVsRest(Curve[I], Curve[I+1], Rect.RB, Rect.LB, P) Or
         _CollideRestVsRest(Curve[I], Curve[I+1], Rect.LB, Rect.LT, P) then begin
	      Result.AddPoint(Curve[I]);
        Result.AddPoint(Curve[I+1]);
      end;
	    //If Result.PointsCount > 1 then
	    //  break;
    end;
    If Result.PointsCount > 0 then
      Result.Test := True;


  // Debug
    {If Result.PointsCount > 0 then begin
      Result.Test := True;
      Log.PrintLN('Cur Vs Rect (' + IntToStr(Result.PointsCount) + ')');
    end;}
    If Result.PointsCount = 1 then begin
      Log.Print('Rect: ');
      Log.Print(' (' + IntToStr(Round(Rect.A.X*100)) + ', ' + IntToStr(Round(Rect.A.Y*100)) + ')');
      Log.Print(' (' + IntToStr(Round(Rect.B.X*100)) + ', ' + IntToStr(Round(Rect.B.Y*100)) + ')');
      Log.PrintLn;
        For I := 0 to Curve.PointCount - 1 do
          Log.Print(' (' + IntToStr(Round(Curve.GetPoint(I).X*100)) + ', ' + IntToStr(Round(Curve.GetPoint(I).Y*100)) + ')');
      Log.PrintLn;
    end;
    If (Result.PointsCount > 1)And(Dist2D(Result.GetPoint(0), Result.GetPoint(1)) < 10e-6) then begin
      Log.PrintLn('Near!');
    end;
end;

Function CollideRest1DVsRest1D(A, B, C, D: Single; Var L, R: Single): Boolean;
begin
  Result := False;
    If ((C < A)And(A < D))Or
       ((C < B)And(B < D))Or
       ((A < C)And(C < B))Or
       ((A < D)And(D < B)) then begin
      L := Max(A, B);
      R := Min(C, D);
      Result := True;
    end;
end;

Function CollideRectVsRect(A, B: TRect): TCollision;
  Var
    I: Integer;
    Count: Integer;
    P: TVec2D;
    LX, RX, TY, BY: Single;
begin
  Result := TCollision.Create(Nil);
    If CollideRest1DVsRest1D(A.LT.X, A.RT.X, B.LT.X, B.RT.X, LX, RX) And
       CollideRest1DVsRest1D(A.LT.Y, A.LB.Y, B.LT.Y, B.LB.Y, TY, BY) then begin
      Result.AddPoint(Vec2D(LX, TY));
      Result.AddPoint(Vec2D(RX, TY));
      Result.AddPoint(Vec2D(RX, BY));
      Result.AddPoint(Vec2D(LX, BY));
      //Log.PrintLn('RectVsRect!');
    end;
    If Result.PointsCount > 0 then
      Result.Test := True;
end;

//
//  TCollision
//
Constructor TCollision.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FTest := False;
end;

Destructor TCollision.Destroy;
begin
  Inherited Destroy;
end;

Function TCollision.PointsCount: Integer;
begin
  Result := Length(FPoints);
end;

Function TCollision.GetPoint(ID: Integer): TVec2D;
begin 
  Result := FPoints[ID];
end;

Procedure TCollision.AddPoint(Point: TVec2D);
begin
  SetLength(FPoints, Length(FPoints)+1);
  FPoints[High(FPoints)] := Point; 
end;

//
//  TCollide
//
Function TCollide.Collide(A, B: TEntity): TCollision;
begin
  Result := TCollision(Call(A, B));
end;

//
//  TCollider
//
Procedure TCollider.AddBinary(AType, BType: TClassEntity; Func: TBinaryFunc);
begin
  SetLength(FBinaries, Length(FBinaries) + 1);
  FBinaries[High(FBinaries)] := TCollide.Create(Self);
  FBinaries[High(FBinaries)].SetFields(AType, BType, Func, True);
end;

Constructor TCollider.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  //AddBinary(TLine2D, TCircle, b_CollideLineVsCircle);
  //AddBinary(TCircle, TCircle, b_CollideCircleVsCircle);
  AddBinary(TCurve2D, TCircle, b_CollideCurveVsCircle);
  AddBinary(TCurve2D, TRect, b_CollideCurveVsRect);
  AddBinary(TRect, TRect, b_CollideRectVsRect);
end;

Function TCollider.Collide(A, B: TEntity): TCollision;
  Var
    I: Integer;
begin
    For I := 0 to High(FBinaries) do begin
      Result := TCollide(FBinaries[I]).Collide(A, B);
        If Assigned(Result) then begin
	  try
            Exit;
          except
            Writeln('Default');
          end;
	end;
    end;
    If Result = Nil then
      Result := TCollision.Create(Nil);
end;

initialization
  Collider := TCollider.Create(Nil);
finalization
  Collider.Free;
end.
