unit MCore;

interface

Uses
  DUtils,
  DDebug,
  DCore,
  DMath;

Type
TMathBody = class(TEntity)
end;

TMathBody2D = class(TMathBody)
  Procedure SetPos(Pos: TVec2D); virtual;
end;

TLine2D = class(TMathBody2D)
private
  FN: TVec2D;
  FD: Double;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Flip;
  Procedure SetLine(N: TVec2D; D: Double; Normalize: Boolean = True);
  Procedure ComputePoints(A, B: TVec2D; Normilize: Boolean = True);
  Function GetDist(P: TVec2D): Double;
  Function GetTangent: TVec2D;
  Property N: TVec2D read FN;
  Property T: TVec2D read GetTangent;
  Property D: Double read FD;
end;

TCircle = class(TMathBody2D)
private
  FCen: TVec2D;
  FRad: Double;
public
  Constructor Create(Owner: TEntity); override;
  Procedure SetPos(Pos: TVec2D); override;
  Property Cen: TVec2D read FCen write FCen;
  Property Rad: Double read FRad write FRad;
end;

TRect = class(TMathBody2D)
private
  FA, FB: TVec2D;
public
  Constructor Create(Owner: TEntity); override;
  Procedure SetPos(Pos: TVec2D); override;
  Function GetLT: TVec2D;
  Function GetLB: TVec2D;
  Function GetRT: TVec2D;
  Function GetRB: TVec2D;
  Property A: TVec2D read FA write FA;
  Property B: TVec2D read FB write FB;
  Property LT: TVec2D read GetLT;
  Property LB: TVec2D read GetLB;
  Property RT: TVec2D read GetRT;
  Property RB: TVec2D read GetRB;
end;

implementation

//
//  TMathBody2D
//
Procedure TMathBody2D.SetPos(Pos: TVec2D);
begin
end;

//
//  TLine2D
//
Constructor TLine2D.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FN := Vec2D(0, 1);
  FD := 0; 
end;

Procedure TLine2D.Flip;
begin
  FN := Mul2D(FN, -1);
  FD := - FD;
end;

Procedure TLine2D.SetLine(N: TVec2D; D: Double; Normalize: Boolean = True);
begin
  FN := N;
    If Normalize then begin
      FD := D/Len2D(FN);
      FN := Norm2D(FN);
    end else
      FD := D;
end;

Procedure TLine2D.ComputePoints(A, B: TVec2D; Normilize: Boolean = True);
begin
  FN.X := A.Y - B.Y;
  FN.Y := B.X - A.X;
    If Normilize then 
      FN := Norm2D(FN);
  FD := - Dot2D(A, FN);
end;

Function TLine2D.GetDist(P: TVec2D): Double;
begin
  Result := Dot2D(P, FN) + FD;
end;

Function TLine2D.GetTangent: TVec2D;
begin
  Result := Vec2D(FN.Y, -FN.X);
end;

//
//  TCircle
//
Constructor TCircle.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
end;

Procedure TCircle.SetPos(Pos: TVec2D);
begin
  Inherited SetPos(Pos);
  FCen := Pos;
end;

//
//  TRect
//
Constructor TRect.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
end;

Procedure TRect.SetPos(Pos: TVec2D);
  Var
    W_2, H_2: Single;
begin
  W_2 := (RT.X - LT.X)/2;
  H_2 := (LB.Y - LT.Y)/2;
  FA := Sub2D(Pos, Vec2D(W_2, H_2));
  FB := Add2D(Pos, Vec2D(W_2, H_2));
end;

Function TRect.GetLT;
begin
  Result := Vec2D(Min(A.X, B.X),
                  Min(A.Y, B.Y));
end;

Function TRect.GetLB;
begin
  Result := Vec2D(Min(A.X, B.X),
                  Max(A.Y, B.Y)); 
end;

Function TRect.GetRT;
begin
  Result := Vec2D(Max(A.X, B.X),
                  Min(A.Y, B.Y));  
end;

Function TRect.GetRB;
begin
  Result := Vec2D(Max(A.X, B.X),
                  Max(A.Y, B.Y));
end;

end.
