unit MCurve;

interface

Uses 
  DDebug,
  DUtils,
  DCore,
  DMath,
  
  MCore,

  dglOpenGL;

Type
TCurve2D = class(TMathBody2D)
private
  FPoints: Array Of TVec2D;
  FLen: Double;
public
  Constructor Create(Owner: TEntity); override;
  Procedure ClearPoints;
  Procedure AddPoint(P: TVec2D);
  Procedure CopyLod(Curve: TCurve2D);
  Procedure Render; override;
  Function PointCount: Integer;
  Function GetPoint(ID: Integer): TVec2D;
  Function Len: Double; virtual;
  Procedure CutToLen(Len: Double);
  Property Points[Index: Integer]: TVec2D read GetPoint; default;
end;

implementation

Constructor TCurve2D.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  ClearPoints;
end;

Procedure TCurve2D.ClearPoints;
begin
  SetLength(FPoints, 0);
  FLen := 0;
end;

Procedure TCurve2D.AddPoint(P: TVec2D);
begin
  SetLength(FPoints, Length(FPoints) + 1);
  FPoints[High(FPoints)] := P;
    If Length(FPoints) > 1 then
      FLen := FLen + Dist2D(FPoints[High(FPoints)], FPoints[High(FPoints)-1]);
end;

Procedure TCurve2D.CopyLod(Curve: TCurve2D);
  Var
    Line: TLine2D;
    I, J: Integer;
begin
  ClearPoints;
    If Curve.PointCount < 1 then
      Exit;
  Line := TLine2D.Create(Self);
  AddPoint(Curve.GetPoint(0));
  I := 0;
    While I < Curve.PointCount-1 do begin
      J := I + 1;
      Line.ComputePoints(Curve.GetPoint(I), Curve.GetPoint(I+1));
        While (J < Curve.PointCount-1)And
              (Line.GetDist(Curve.GetPoint(J+2)) < 0.01) do
          Inc(J);
      AddPoint(Curve.GetPoint(J));
      I := J;
    end;
  AddPoint(Curve.GetPoint(Curve.PointCount-1));
  Line.Free;
end;

Procedure TCurve2D.Render;
  Var
    I: Integer;
begin
  glBegin(GL_LINE_STRIP);
    For I := 0 to High(FPoints) do
      glVertex2fv(@FPoints[I]);
  glEnd;
end;

Function TCurve2D.PointCount;
begin
  Result := Length(FPoints);
end;

Function TCurve2D.GetPoint;
begin
    If (ID < 0)Or(ID > High(FPoints)) then
      Result := Vec2D(0, 0)
    else
      Result := FPoints[ID];
end;

Function TCurve2D.Len: Double;
begin
  Result := FLen;
end;

Procedure TCurve2D.CutToLen(Len: Double);
  Var
    I: Integer;
    Next: Double;
begin
    If (Length(FPoints) < 2)Or(Len < 10e-6) then
      Exit;
  FLen := 0;
  I := 0;
    For I := 0 to High(FPoints)-1 do begin
      Next := Dist2D(FPoints[I], FPoints[I+1]);
        If FLen + Next > Len then begin
          SetLength(FPoints, I+1);
          Exit;
        end;
      FLen := FLen + Next;
    end;
end;

end.
