unit DWinApi;

interface

uses
  Windows,
  Messages,
  dglOpenGL,
  SysUtils,
  DCore;

Type
TProcWinMsg = Procedure(Msg: Cardinal; wParam: Integer; lParam: Integer);

TEventWindows = class(TEvent)
private
  FhWnd: HWND;
  FMsg: Cardinal;
  FwParam: Integer;
  FlParam: Integer;
public
  Constructor Create(hWnd: HWND; Msg: Cardinal; wParam: Integer; lParam: Integer);
  Property hWnd: HWND read FhWnd write FhWnd;
  Property Msg: Cardinal read FMsg write FMsg;
  Property wParam: Integer read FwParam write FwParam;
  Property lParam: Integer read FlParam write FlParam;
end;

TEventKey = class(TEvent)
private
  FKey: Integer;
  FDown: Boolean;
public
  Constructor Create(Key: Integer; Down: Boolean);
  Property Key: Integer read FKey write FKey;
  Property Down: Boolean read FDown write FDown;
end;

TEventMouseButton = class(TEventKey)
end;

TEventChar = class(TEventKey)
end;

TEvMouseMoved = class(TEvent)
private
  FX: Integer;
  FY: Integer;
public
  Constructor Create(X, Y: Integer);
  Property X: Integer read FX write FX;
  Property Y: Integer read FY write FY;
end;

Var
  Wnd: HWND;
  DC: HDC;
  RC: HGLRC;
  WWidth: Integer;
  WHeight: Integer;
  WFullScreen: Boolean = False;

  Msg: TMsg;
  PrevMouse, Mouse: TPoint;

  PrevLeftMouse: Boolean = False;
  PrevRightMouse: Boolean = False;
  LeftMouse: Boolean = False;
  RightMouse: Boolean = False;
  Keys: Array[0..255] Of Boolean;

Function CreateWindow(Width, Height, Color: Integer; Caption: String; FullScreen: Boolean): Boolean;
Procedure UpdateMousePos;
Procedure SetMousePos(X, Y: Integer);
Procedure MessageProcess;

Type
TWinApi = class(TEntity)
private
  FSubscriber: TEntity;
public
  Procedure Update; override;
  Procedure SendEvent(Event: TEvent); override;
  Property Subscriber: TEntity read FSubscriber write FSubscriber;
end;

Var
  WinApi: TWinApi;

implementation

Uses
  DDebug;

//
//  TEventWindows
//
Constructor TEventWindows.Create(hWnd: HWND; Msg: Cardinal; wParam: Integer; lParam: Integer);
begin
  Inherited Create(WinApi);
  FhWnd := Wnd;
  FMsg := Msg;
  FwParam := wParam;
  FlParam := lParam;
end;

//
//  TEventKey
//
Constructor TEventKey.Create(Key: Integer; Down: Boolean);
begin
  Inherited Create(WinApi);
  FKey := Key;
  FDown := Down;
end;

//
//  TEvMouseMoved
//
Constructor TEvMouseMoved.Create(X, Y: Integer);
begin
  Inherited Create(WinApi);
  FX := X;
  FY := Y;
end;

//  ,  
Function WndProc(hWnd: HWND; Msg: Cardinal; wParam: Integer; lParam: Integer): Integer; stdcall;
  Var
    Event: TEvent;
    Rect: TRect;
begin
    //ProcWinMsg(Msg, wParam, lParam);
    Case Msg of
    WM_DESTROY: //    
      begin
       PostQuitMessage(0);
       Result := 0;
      end;
    WM_KEYDOWN: //  
      begin
        Keys[wParam] := True;
        Event := TEventKey.Create(wParam, True);
        Result := 0;
      end;
    WM_KEYUP: //  
      begin
        Keys[wParam] := False;
        Event := TEventKey.Create(wParam, False);
        Result := 0;
      end;
    WM_LBUTTONDOWN: //    
      begin
        Event := TEventMouseButton.Create(0, True);
        LeftMouse := True;
        Result := 0;
        //ExeCmnd(OnLeftMouse);
      end;
    WM_LBUTTONUP: //    
      begin
        LeftMouse := False;
        Event := TEventMouseButton.Create(0, False);
        Result := 0;
      end;
    WM_RBUTTONDOWN: //    
      begin
        RightMouse := True;
        Event := TEventMouseButton.Create(2, True);
        Result := 0;
        //ExeCmnd(OnRightMouse);
      end;
    WM_RBUTTONUP: //    
      begin
        RightMouse := False;
        Event := TEventMouseButton.Create(2, False);
        Result := 0;
      end;
    WM_MOUSEMOVE:
      begin
        GetWindowRect(Wnd, Rect);
        Event := TEvMouseMoved.Create(LoWord(lParam) - Rect.Left,
                                      HiWord(lParam) - Rect.Top);
      end;
    WM_CHAR:
      begin
        Event := TEventChar.Create(wParam, True);
        Result := 0;
      end;
    else
      Event := TEventWindows.Create(hWnd, Msg, wParam, lParam);
      //    
      Result := DefWindowProc(hwnd, msg, wParam, lParam);
    end;
  WinApi.SendEventAndFree(Event);
end;

// ,      Name
Function CreateWindowClass(Name: String): Boolean;
  Var
    Wnd: TWNDCLASSEXA;
begin
  Result := False;
  FillChar(wnd, SizeOf(wnd), 0);
    With wnd do begin
      cbSize        := SizeOf(wnd);
      style         := CS_HREDRAW or
                       CS_VREDRAW or
                       CS_OWNDC;
      lpfnWndProc   := @WndProc;
      hCursor       := LoadCursor(0, IDC_ARROW);
      lpszClassName := PAnsiChar(Name);
    end;
    If RegisterClassEx(wnd) = 0 then
      Exit;
  Result := True;
  Log.PrintLn('CreateWindowClass("' + Name + '");');
end;

// ,      
Function CreatePixelFormat(ColorBits, DepthBits, StencilBits: Integer): Boolean;
  Var
    Pfd: PIXELFORMATDESCRIPTOR;
    PixelFormat: Integer;
begin
  Result := False;
  FillChar(pfd, SizeOf(pfd), 0);
    With pfd do begin
      nSize        := SizeOf(TPIXELFORMATDESCRIPTOR);
      nVersion     := 2;
      dwFlags      := PFD_DRAW_TO_WINDOW or
                      PFD_SUPPORT_OPENGL or
                      PFD_DOUBLEBUFFER;
      iPixelType   := PFD_TYPE_RGBA;
      cColorBits   := ColorBits;
      cDepthBits   := DepthBits;
      cStencilBits := StencilBits;
      iLayerType   := PFD_MAIN_PLANE;
    end;
  PixelFormat := ChoosePixelFormat(DC, @pfd);
    If (PixelFormat = 0) Or Not SetPixelFormat(DC, PixelFormat, @Pfd) then
      Exit;
  Result := True;
  Log.PrintLn('CreatePixelFormat(Color = ' + IntToStr(ColorBits) + ', ' +
                                'Depth = ' + IntToStr(DepthBits) + ', ' +
                                'Stencil = ' + IntToStr(StencilBits) + ');');
end;

// ,  
Function CreateWindow(Width, Height, Color: Integer; Caption: String; FullScreen: Boolean): Boolean;
  Var
    dmScreenSettings : DEVMODE;
    dwStyle : DWORD;
    //dwExStyle : DWORD;
    Rect  : TRect;
begin
  Result := False;
  InitOpenGL;
    If Not CreateWindowClass('OpenGL') then
      Exit;
  Wnd := CreateWindowEx(WS_EX_APPWINDOW or WS_EX_WINDOWEDGE, 'OpenGL', PChar(Caption),
                        WS_POPUP or WS_CAPTION, 
                        0, 0, Width, Height, 0, 0, 0, nil);
    If Wnd = 0 then
      Exit;
  DC := GetDC(Wnd);
    If (DC = 0) Or Not CreatePixelFormat(32, 24, 8) then
      Exit;
  RC := wglCreateContext(DC);
    If (RC = 0) Or Not wglMakeCurrent(DC, RC) then
      Exit;

  If FullScreen then begin
    ZeroMemory(@dmScreenSettings, SizeOf(dmScreenSettings));
      With dmScreenSettings do begin
        dmSize             := SizeOf(dmScreenSettings);
        dmPelsWidth        := Width;
        dmPelsHeight       := Height;
        dmBitsPerPel       := Color;
        dmDisplayFrequency := 0;
        dmFields           := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL or DM_DISPLAYFREQUENCY;
      end;
      If (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) = DISP_CHANGE_FAILED) then begin
        MessageBox(0, 'Unable to switch to fullscreen!', 'Error', MB_OK or MB_ICONERROR);
        Exit;
      end;
  end;

  if FullScreen then begin
    dwStyle := WS_POPUP or
               WS_CLIPCHILDREN or
               WS_CLIPSIBLINGS;
    //dwExStyle := WS_EX_APPWINDOW;
    ShowCursor(False);
  end;

  SetWindowLong(Wnd, dwStyle, dwStyle or WS_VISIBLE);
  Rect.Left   := 0;
  Rect.Top    := 0;
  Rect.Right  := width;
  Rect.Bottom := height;
  AdjustWindowRect(Rect, dwStyle, False);
    with Rect do
      MoveWindow(Wnd, 0, 0, Right - Left, Bottom - Top, True);

  ShowCursor(False);
  SetForegroundWindow(Wnd);
  ShowWindow(Wnd, SW_SHOW);
  SetFocus(Wnd);
  UpdateWindow(Wnd);
  //SetWindowLong(Wnd, GWL_STYLE, (WS_CAPTION*Ord(Not FullScreen)) or WS_VISIBLE);
  SetWindowLong(Wnd, GWL_STYLE,  WS_CAPTION or WS_VISIBLE);  
  SetWindowPos(Wnd, 0, 400, 200, Width, Height, SWP_FRAMECHANGED or SWP_NOOWNERZORDER);
  glViewport(0, 0, Width, Height);


  Result := True;
  WWidth := Width;
  WHeight := Height;
  WFullScreen := FullScreen;

  Log.PrintLn('CreateWindow(Size = ' + IntToStr(Width) + 'x' + IntToStr(Height) + ', ' +
                            'Caption = "' + Caption + '");');
end;

//     
//      Mouse
Procedure UpdateMousePos;
  Var
    Rect: TRect;
begin
  GetWindowRect(Wnd, Rect);
  GetCursorPos(Mouse);
  Mouse.X := Mouse.X - Rect.Left;
  Mouse.Y := Mouse.Y - Rect.Top;
end;

Procedure SetMousePos(X, Y: Integer);
  Var
    Rect: TRect;
begin
  GetWindowRect(Wnd, Rect);;
  SetCursorPos(X + Rect.Left, Y + Rect.Top);
  UpdateMousePos;
end;

Procedure MessageProcess;
begin
  PrevMouse := Mouse;
  PrevLeftMouse := LeftMouse;
  PrevRightMouse := RightMouse;
end;

//
//  TWinApi
//
Procedure TWinApi.Update;
begin
    While PeekMessage(msg, 0, 0, 0, PM_REMOVE) do begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end;
  UpdateMousePos;
end;

Procedure TWinApi.SendEvent(Event: TEvent);
begin
  Inherited SendEvent(Event);
    If FSubscriber <> Nil then
      FSubscriber.SendEvent(Event);
end;

initialization
  WinApi := TWinApi.Create(Nil);
finalization
  WinApi.Free;
end.
