unit RPrim;

interface

Uses
  DDebug,
  DUtils,
  DCore,
  DMath,
  DParser,

  MCore,
  MCurve,
  MCollider,

  dglOpenGL,
  DWinApi;

Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      Fill: Boolean = True); overload;
Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      R1, G1, B1, A1: Single;
                      R2, G2, B2, A2: Single); overload;
Procedure DrawEllipse(F1, F2: TVec2D;
                      Rad: Single;
                      Sides: Integer;
                      Fill: Boolean = True); overload;
Procedure DrawPixel(X, Y, R, G, B: Single);
Procedure DrawBubble(X, Y, Size: Single; R, G, B: Single);

Procedure DrawProgressBar(Pos, Size: TVec2D;
                          Color: TVec3D;
                          Value: Single;
                          Alpha: Boolean = True);

implementation

Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      Fill: Boolean = True); overload;
  Var
    I: Integer;
begin
    If Fill then      
      glBegin(GL_POLYGON)
    else
      glBegin(GL_LINE_STRIP);
    For I := 0 to Sides-1 do
      glVertex2f(X + RadX*Cos(2*pi*I/Sides),
                 Y + RadY*Sin(2*pi*I/Sides));
    If Not Fill then
      glVertex2f(X + RadX, Y);
  glEnd;
end;

Procedure DrawEllipse(X, Y, RadX, RadY: Single; Sides: Integer;
                      R1, G1, B1, A1: Single;
                      R2, G2, B2, A2: Single); overload;
  Var
    I: Integer;
begin
  glBegin(GL_TRIANGLE_FAN);
    glColor4f(R1, G1, B1, A1);
    glVertex2f(X, Y);
    glColor4f(R2, G2, B2, A2);
      For I := 0 to Sides do
        glVertex2f(X + RadX*Cos(2*pi*I/Sides),
                   Y + RadY*Sin(2*pi*I/Sides));
  glEnd;
end;

Procedure DrawEllipse(F1, F2: TVec2D;
                      Rad: Single;
                      Sides: Integer;
                      Fill: Boolean = True);
begin
end;

Procedure DrawPixel(X, Y, R, G, B: Single);
begin
  glPointSize(1);
  glBegin(GL_POINTS);
    glColor3f(R, G, B);
    glVertex2f(X, Y);
  glEnd;
end;

Procedure DrawBubble(X, Y, Size: Single; R, G, B: Single);
begin
  DrawEllipse(X, Y, Size, Size, 15,
              R, G, B, 1,
              R+0.1, G+0.1, B+0.1, 1);  
end;

Procedure DrawProgressBar;
begin
  glColor4f(Color.X, Color.Y, Color.Z, 0.5);
    If Alpha then begin
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glBegin(GL_QUADS);
          glVertex2f(Pos.X, Pos.Y);
          glVertex2f(Pos.X + Size.X, Pos.Y);
          glVertex2f(Pos.X + Size.X, Pos.Y + Size.Y);
          glVertex2f(Pos.X, Pos.Y + Size.Y);
        glEnd;
      glDisable(GL_BLEND);
    end;
  glBegin(GL_QUADS);
    glVertex2f(Pos.X, Pos.Y);
    glVertex2f(Pos.X + Size.X*Value, Pos.Y);
    glVertex2f(Pos.X + Size.X*Value, Pos.Y + Size.Y);
    glVertex2f(Pos.X, Pos.Y + Size.Y);
  glEnd;
  glBegin(GL_LINE_LOOP);
    glVertex2f(Pos.X, Pos.Y);
    glVertex2f(Pos.X + Size.X, Pos.Y);
    glVertex2f(Pos.X + Size.X, Pos.Y + Size.Y);
    glVertex2f(Pos.X, Pos.Y + Size.Y);
  glEnd;
  glColor4f(1, 1, 1, 1);
end;

end.
