unit RSprite;

interface

Uses
  dglOpenGL;

Type
TSprite = packed record
  Texture: Integer;
  R, G, B: Single;
  A: Single;
  X, Y: Single;
  Width, Height: Single;
  Angle: Single;
  X1, Y1, X2, Y2: Single;
end;

Var
  Sprite: TSprite;

Procedure SpriteSetTexture(Texture: Integer); cdecl;
Procedure SpriteSetColor(R, G, B: Single); cdecl;
Procedure SpriteSetAlpha(Alpha: Single); cdecl;
Procedure SpriteSetPos(X, Y: Single); cdecl;
Procedure SpriteSetSize(Width, Height: Single); cdecl;
Procedure SpriteSetAngle(Angle: Single); cdecl;
Procedure SpriteSetFullTexture; cdecl;
Procedure SpriteSetPartTexture(X1, Y1, X2, Y2: Single); cdecl;
Procedure SpriteSetFrame(FrameX, FrameY, Frame: Integer); cdecl;
Procedure SpriteDraw; cdecl;

implementation

Procedure SpriteSetTexture;
begin
  Sprite.Texture := Texture;
end;

Procedure SpriteSetColor(R, G, B: Single);
begin
  Sprite.R := R;
  Sprite.G := G;
  Sprite.B := B;
end;

Procedure SpriteSetAlpha(Alpha: Single);
begin
  Sprite.A := Alpha;
end;

Procedure SpriteSetPos;
begin
  Sprite.X := X;
  Sprite.Y := Y;
end;

Procedure SpriteSetSize;
begin
  Sprite.Width := Width;
  Sprite.Height := Height;
end;

Procedure SpriteSetAngle;
begin
  Sprite.Angle := Angle;
end;

Procedure SpriteSetFullTexture;
begin
  SpriteSetPartTexture(0, 0, 1, 1);
end;

Procedure SpriteSetPartTexture;
begin
  Sprite.X1 := X1;
  Sprite.Y1 := Y1;
  Sprite.X2 := X2;
  Sprite.Y2 := Y2;
end;

Procedure SpriteSetFrame;
  Var
    X, Y: Integer;
begin
  X := Frame mod FrameX;
  Y := Frame div FrameX;
  SpriteSetPartTexture(    X/FrameX,     Y/FrameY,
                       (X+1)/FrameX, (Y+1)/FrameY);
end;

Procedure SpriteDraw;
  Var
    S, C: Single;
begin
    With Sprite do begin      
      glColor4f(R, G, B, A);
      //BindTexture(Texture);
      S := Sin(Angle);
      C := Cos(Angle);
        glBegin(GL_QUADS);
          glTexCoord2f(X2, Y2);
          glVertex2f(X + (+C*Width/2-S*Height/2), Y + (+S*Width/2+C*Height/2));
          glTexCoord2f(X2, Y1);
          glVertex2f(X + (+C*Width/2+S*Height/2), Y + (+S*Width/2-C*Height/2));
          glTexCoord2f(X1, Y1);
          glVertex2f(X + (-C*Width/2+S*Height/2), Y + (-S*Width/2-C*Height/2));
          glTexCoord2f(X1, Y2);
          glVertex2f(X + (-C*Width/2-S*Height/2), Y + (-S*Width/2+C*Height/2));
        glEnd;
      //BindTexture(-1);
      glColor4f(1, 1, 1, 1); 
    end;
end;

initialization
  SpriteSetTexture(0);
  SpriteSetColor(1, 1, 1);
  SpriteSetAlpha(1);
  SpriteSetPos(0, 0);
  SpriteSetSize(10, 10);
  SpriteSetAngle(0);
  SpriteSetFullTexture; 
end.

