unit GRender;

interface

Uses
  {$I Units.inc};

implementation

Type
TRenderRect = class(TGameDecorator)
private
  FPos: TVec2D;
  FSize: TVec2D;
  FColor: TVec4D;
public
  Constructor Create(Owner: TEntity); override;
  Procedure Render; override;
  Procedure OnEvent(Event: TEvent); override;
end;

Constructor TRenderRect.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FPos := Vec2D(0, 0);
  FSize := Vec2D(50, 50);
  FColor := Vec4D(Random, Random, Random, 1);
end;

Procedure TRenderRect.Render;
begin
  Inherited Render;
  glColor4fv(@FColor);
  glRectf(FPos.X-FSize.X/2, FPos.Y-FSize.Y/2, 
          FPos.X+FSize.X/2, FPos.Y+FSize.Y/2);
end;

Procedure TRenderRect.OnEvent(Event: TEvent);
begin
  Inherited OnEvent(Event);
    If Event.IsType('TEvSetPos') then
      FPos := TEvSetPos(Event).Pos;
end;

//
//  
//
Type
TRenderAppear = class(TGameDecorator)
private
  FStart: Integer;
  FTime: Integer;
public
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Procedure TRenderAppear.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then begin
      FTime := StrToInt(Value);
      FStart := GetTimer;
    end;
end;

Procedure TRenderAppear.Render;
  Var
    I: Integer;
begin
  Inherited Render;
      If GetTimer > FStart + FTime then
        Exit;
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glColor4f(0, 0, 0, 1-(GetTimer - FStart)/FTime);
  glRectf(0, 0, 320, 240);
  glDisable(GL_BLEND);
end;

//
//  
//
Var
  Font: Integer = -1;

Type
TLabel = class(TGameDecorator)
private
  FText: String;
  FPos: TVec2D;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Constructor TLabel.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FText := '';
  FPos := Vec2D(0, 0);
    If Font = -1 then
      Font := CreateFont(10, 10, 'FixedSys');
end;

Procedure TLabel.AssignVar(Name, Value: String);
begin
    If Name = 'text' then
      FText := Value
    else If Name = 'x' then
      FPos.X := StrToFloat(Value)
    else If Name = 'y' then
      FPos.Y := StrToFloat(Value);
end;

Procedure TLabel.Render;
begin
  Inherited Render;
  glColor4f(1, 1, 1, 1);
  glDisable(GL_BLEND);
  glBlendFunc(GL_ONE, GL_ZERO);
  DrawText(Font, Round(FPos.X), Round(FPos.Y), PAnsiChar(FText));
end;

Type
  TLines = Array[0..1] Of Array[0..1] Of TVec2D;

Const
    Glava1: Array[0..428] Of Array[0..1] Of TVec2D = (
        ((x: 2.70000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 6.10000000000000E+0001)), 
        ((x: 2.70000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 6.30000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 6.30000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 6.30000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 6.30000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 7.90000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 7.90000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 7.90000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 7.90000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 8.70000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 8.70000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 8.70000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 8.70000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 9.20000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 9.20000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 9.20000000000000E+0001)), 
        ((x: 2.80000000000000E+0001; y: 9.20000000000000E+0001),
         (x: 2.80000000000000E+0001; y: 1.00000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.00000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.00000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.00000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.05000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.05000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.05000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.05000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.15000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.15000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.15000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.15000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.18000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.20000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.20000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.22000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.22000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.22000000000000E+0002)), 
        ((x: 2.80000000000000E+0001; y: 1.22000000000000E+0002),
         (x: 2.80000000000000E+0001; y: 1.26000000000000E+0002)), 
        ((x: 2.60000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 2.60000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 2.60000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 2.60000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 2.60000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 2.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 2.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 2.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 2.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 3.30000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 3.30000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 3.30000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 3.30000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 3.30000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 3.30000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.10000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.10000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.50000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.50000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.50000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.50000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.70000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.70000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.80000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.80000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 4.80000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 4.80000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.00000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.00000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.10000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.10000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.10000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.10000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.30000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.30000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.50000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.50000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.50000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.50000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 5.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 5.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 6.20000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 6.20000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 6.20000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 6.20000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 6.70000000000000E+0001; y: 6.00000000000000E+0001)), 
        ((x: 6.70000000000000E+0001; y: 6.00000000000000E+0001),
         (x: 6.70000000000000E+0001; y: 6.00000000000000E+0001)), 
        ((x: 6.70000000000000E+0001; y: 6.00000000000000E+0001),
         (x: 7.20000000000000E+0001; y: 6.00000000000000E+0001)), 
        ((x: 9.50000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 9.50000000000000E+0001; y: 5.70000000000000E+0001)), 
        ((x: 9.50000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 9.50000000000000E+0001; y: 5.70000000000000E+0001)), 
        ((x: 9.50000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 9.50000000000000E+0001; y: 6.00000000000000E+0001)), 
        ((x: 9.50000000000000E+0001; y: 6.00000000000000E+0001),
         (x: 9.50000000000000E+0001; y: 6.00000000000000E+0001)), 
        ((x: 9.50000000000000E+0001; y: 6.00000000000000E+0001),
         (x: 9.40000000000000E+0001; y: 6.50000000000000E+0001)), 
        ((x: 9.40000000000000E+0001; y: 6.50000000000000E+0001),
         (x: 9.40000000000000E+0001; y: 6.50000000000000E+0001)), 
        ((x: 9.40000000000000E+0001; y: 6.50000000000000E+0001),
         (x: 9.30000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 9.30000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 9.30000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 9.30000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 9.00000000000000E+0001; y: 7.20000000000000E+0001)), 
        ((x: 9.00000000000000E+0001; y: 7.20000000000000E+0001),
         (x: 9.00000000000000E+0001; y: 7.20000000000000E+0001)), 
        ((x: 9.00000000000000E+0001; y: 7.20000000000000E+0001),
         (x: 8.90000000000000E+0001; y: 7.60000000000000E+0001)), 
        ((x: 8.90000000000000E+0001; y: 7.60000000000000E+0001),
         (x: 8.90000000000000E+0001; y: 7.60000000000000E+0001)), 
        ((x: 8.90000000000000E+0001; y: 7.60000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 8.10000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 8.10000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 8.10000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 8.10000000000000E+0001),
         (x: 8.70000000000000E+0001; y: 8.50000000000000E+0001)), 
        ((x: 8.70000000000000E+0001; y: 8.50000000000000E+0001),
         (x: 8.70000000000000E+0001; y: 8.50000000000000E+0001)), 
        ((x: 8.70000000000000E+0001; y: 8.50000000000000E+0001),
         (x: 8.70000000000000E+0001; y: 8.80000000000000E+0001)), 
        ((x: 8.70000000000000E+0001; y: 8.80000000000000E+0001),
         (x: 8.70000000000000E+0001; y: 8.80000000000000E+0001)), 
        ((x: 8.70000000000000E+0001; y: 8.80000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.30000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.30000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.30000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.30000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.07000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.07000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.07000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.07000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.17000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.17000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.17000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.17000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.21000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.21000000000000E+0002),
         (x: 8.50000000000000E+0001; y: 1.21000000000000E+0002)), 
        ((x: 8.50000000000000E+0001; y: 1.21000000000000E+0002),
         (x: 8.40000000000000E+0001; y: 1.22000000000000E+0002)), 
        ((x: 8.40000000000000E+0001; y: 1.22000000000000E+0002),
         (x: 8.40000000000000E+0001; y: 1.22000000000000E+0002)), 
        ((x: 8.40000000000000E+0001; y: 1.22000000000000E+0002),
         (x: 8.30000000000000E+0001; y: 1.23000000000000E+0002)), 
        ((x: 9.90000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 9.90000000000000E+0001; y: 5.70000000000000E+0001)), 
        ((x: 9.90000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 9.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 9.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 9.90000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 9.90000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 1.04000000000000E+0002; y: 6.40000000000000E+0001)), 
        ((x: 1.04000000000000E+0002; y: 6.40000000000000E+0001),
         (x: 1.04000000000000E+0002; y: 6.40000000000000E+0001)), 
        ((x: 1.04000000000000E+0002; y: 6.40000000000000E+0001),
         (x: 1.09000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.09000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.09000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.09000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.13000000000000E+0002; y: 7.50000000000000E+0001)), 
        ((x: 1.13000000000000E+0002; y: 7.50000000000000E+0001),
         (x: 1.13000000000000E+0002; y: 7.50000000000000E+0001)), 
        ((x: 1.13000000000000E+0002; y: 7.50000000000000E+0001),
         (x: 1.17000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 1.17000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 1.17000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 1.17000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 1.17000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 1.17000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 1.17000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 1.17000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.20000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.20000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.20000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.20000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.18000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.18000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 1.19000000000000E+0002; y: 1.12000000000000E+0002)), 
        ((x: 1.19000000000000E+0002; y: 1.12000000000000E+0002),
         (x: 1.19000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.50000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 1.50000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 1.50000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 6.60000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 6.60000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 6.60000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 6.60000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.45000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 1.45000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 1.45000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 1.45000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 1.45000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 1.45000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 1.45000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 1.45000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 1.44000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 1.44000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 1.44000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 1.44000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 1.43000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.43000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.43000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.43000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.41000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.41000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.41000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.41000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.38000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 1.38000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 1.38000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 1.38000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.35000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 1.35000000000000E+0002; y: 1.16000000000000E+0002)), 
        ((x: 1.35000000000000E+0002; y: 1.16000000000000E+0002),
         (x: 1.34000000000000E+0002; y: 1.17000000000000E+0002)), 
        ((x: 1.52000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.52000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.52000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.53000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.53000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.53000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.53000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 1.57000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.57000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.57000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.57000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 6.90000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 6.90000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 6.90000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 6.90000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.69000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.69000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 1.71000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.73000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.73000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.73000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.73000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.73000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.73000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.73000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.73000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.73000000000000E+0002; y: 1.10000000000000E+0002)), 
        ((x: 1.41000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.41000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.41000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.42000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.42000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.42000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.42000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.44000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.44000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.44000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.44000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.48000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.48000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.48000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.48000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.52000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.52000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.52000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.52000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.57000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.57000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.57000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.57000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.59000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.59000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.60000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.60000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.60000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.60000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.63000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.63000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.64000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.64000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.65000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.65000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.66000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.66000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 1.94000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.94000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.94000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.93000000000000E+0002; y: 5.90000000000000E+0001)), 
        ((x: 1.93000000000000E+0002; y: 5.90000000000000E+0001),
         (x: 1.93000000000000E+0002; y: 5.90000000000000E+0001)), 
        ((x: 1.93000000000000E+0002; y: 5.90000000000000E+0001),
         (x: 1.96000000000000E+0002; y: 6.40000000000000E+0001)), 
        ((x: 1.96000000000000E+0002; y: 6.40000000000000E+0001),
         (x: 1.97000000000000E+0002; y: 6.60000000000000E+0001)), 
        ((x: 1.97000000000000E+0002; y: 6.60000000000000E+0001),
         (x: 1.97000000000000E+0002; y: 6.60000000000000E+0001)), 
        ((x: 1.97000000000000E+0002; y: 6.60000000000000E+0001),
         (x: 1.99000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 1.99000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 1.99000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 1.99000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 1.10000000000000E+0002)), 
        ((x: 1.95000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.95000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.95000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 1.95000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 1.95000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 2.00000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 2.00000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 2.11000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.11000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.11000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.11000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.31000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.31000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.31000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.31000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 6.40000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 6.40000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 6.40000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 6.40000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 2.30000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.30000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.30000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.30000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.28000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.28000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.28000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.28000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.25000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.25000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.25000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.25000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.22000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.22000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.22000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.22000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.22000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.22000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.17000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.17000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.17000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.17000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.15000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.15000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.15000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.15000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.12000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.12000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.11000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.11000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.08000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.08000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.06000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.06000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.05000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.05000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.05000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.05000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 7.50000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 7.50000000000000E+0001),
         (x: 2.03000000000000E+0002; y: 7.50000000000000E+0001)), 
        ((x: 2.03000000000000E+0002; y: 7.50000000000000E+0001),
         (x: 2.03000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.03000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.03000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.03000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.12000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.12000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.12000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.12000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 2.27000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 2.27000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 2.27000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 2.27000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 2.30000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.30000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.30000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.30000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.33000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 2.33000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 2.33000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 2.33000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 2.35000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.35000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.35000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.35000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.35000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.35000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.35000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.35000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.32000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 2.32000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 2.32000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 2.32000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 2.31000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.31000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.31000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.31000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.29000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 2.29000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 2.27000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 2.27000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 2.27000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 2.27000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 2.23000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 2.23000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 2.23000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 2.23000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 2.20000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.20000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.18000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.18000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.16000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.16000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.16000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.16000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.13000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.13000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.13000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.13000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.11000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.11000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.09000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.09000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.08000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.08000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.08000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.08000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.06000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.06000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.05000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.05000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.04000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.68000000000000E+0002; y: 5.40000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.40000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.40000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 6.00000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 6.00000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 6.00000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 6.00000000000000E+0001),
         (x: 2.63000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 2.63000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 2.63000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 2.63000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 2.60000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.60000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.60000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.60000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.60000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.60000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.55000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.55000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.55000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.55000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.53000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.53000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.53000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.53000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.50000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 2.50000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 2.48000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 2.48000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 2.48000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 2.48000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 2.46000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 2.46000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 2.46000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 2.46000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 2.44000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 2.44000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 2.44000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 2.44000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 2.44000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 2.44000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 2.41000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 2.67000000000000E+0002; y: 5.50000000000000E+0001),
         (x: 2.67000000000000E+0002; y: 5.50000000000000E+0001)), 
        ((x: 2.67000000000000E+0002; y: 5.50000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.72000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.72000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.72000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.72000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 2.57000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.61000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.61000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.61000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.61000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 1.51000000000000E+0002; y: 1.52000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.52000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.52000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.52000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.52000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.58000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.58000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.58000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.58000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.62000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.62000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.62000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.62000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.71000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.71000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.71000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.71000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.71000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.71000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.80000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.80000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.80000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.80000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.91000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.91000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.91000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.91000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 1.91000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 1.91000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.13000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.13000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.13000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.13000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.17000000000000E+0002)), 
        ((x: 1.23000000000000E+0002; y: 1.47000000000000E+0002),
         (x: 1.23000000000000E+0002; y: 1.47000000000000E+0002)), 
        ((x: 1.23000000000000E+0002; y: 1.47000000000000E+0002),
         (x: 1.23000000000000E+0002; y: 1.47000000000000E+0002)), 
        ((x: 1.23000000000000E+0002; y: 1.47000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.38000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.38000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.38000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.38000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 1.46000000000000E+0002),
         (x: 2.03000000000000E+0002; y: 1.46000000000000E+0002)), 
        ((x: 1.26000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 2.27000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 2.27000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 2.27000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.31000000000000E+0002; y: 2.27000000000000E+0002)), 
        ((x: 1.31000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.31000000000000E+0002; y: 2.27000000000000E+0002)), 
        ((x: 1.31000000000000E+0002; y: 2.27000000000000E+0002),
         (x: 1.53000000000000E+0002; y: 2.26000000000000E+0002)), 
        ((x: 1.53000000000000E+0002; y: 2.26000000000000E+0002),
         (x: 1.53000000000000E+0002; y: 2.26000000000000E+0002)), 
        ((x: 1.53000000000000E+0002; y: 2.26000000000000E+0002),
         (x: 1.76000000000000E+0002; y: 2.26000000000000E+0002)), 
        ((x: 1.76000000000000E+0002; y: 2.26000000000000E+0002),
         (x: 1.76000000000000E+0002; y: 2.26000000000000E+0002)), 
        ((x: 1.76000000000000E+0002; y: 2.26000000000000E+0002),
         (x: 1.89000000000000E+0002; y: 2.24000000000000E+0002)), 
        ((x: 1.89000000000000E+0002; y: 2.24000000000000E+0002),
         (x: 1.89000000000000E+0002; y: 2.24000000000000E+0002)), 
        ((x: 1.89000000000000E+0002; y: 2.24000000000000E+0002),
         (x: 1.94000000000000E+0002; y: 2.24000000000000E+0002))
    );

    Glava2: Array[0..392] Of Array[0..1] Of TVec2D = (
        ((x: 1.47000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 1.46000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 1.46000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 1.46000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 1.46000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 1.42000000000000E+0002; y: 6.00000000000000E+0001)), 
        ((x: 1.42000000000000E+0002; y: 6.00000000000000E+0001),
         (x: 1.42000000000000E+0002; y: 6.00000000000000E+0001)), 
        ((x: 1.42000000000000E+0002; y: 6.00000000000000E+0001),
         (x: 1.39000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.39000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.39000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.39000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 6.80000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 6.80000000000000E+0001),
         (x: 1.34000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 1.34000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 1.34000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 1.34000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 1.31000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 1.31000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 1.31000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 1.31000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 1.28000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 1.28000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 1.28000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 1.28000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 1.26000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.26000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.26000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.26000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.24000000000000E+0002; y: 9.50000000000000E+0001)), 
        ((x: 1.24000000000000E+0002; y: 9.50000000000000E+0001),
         (x: 1.24000000000000E+0002; y: 9.50000000000000E+0001)), 
        ((x: 1.24000000000000E+0002; y: 9.50000000000000E+0001),
         (x: 1.22000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 1.22000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 1.22000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 1.22000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 1.20000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.20000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.20000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.20000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.19000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.19000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.19000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.19000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.15000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.15000000000000E+0002),
         (x: 1.17000000000000E+0002; y: 1.19000000000000E+0002)), 
        ((x: 1.17000000000000E+0002; y: 1.19000000000000E+0002),
         (x: 1.17000000000000E+0002; y: 1.19000000000000E+0002)), 
        ((x: 1.17000000000000E+0002; y: 1.19000000000000E+0002),
         (x: 1.15000000000000E+0002; y: 1.23000000000000E+0002)), 
        ((x: 1.49000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 1.54000000000000E+0002; y: 5.90000000000000E+0001)), 
        ((x: 1.54000000000000E+0002; y: 5.90000000000000E+0001),
         (x: 1.54000000000000E+0002; y: 5.90000000000000E+0001)), 
        ((x: 1.54000000000000E+0002; y: 5.90000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 1.61000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 1.61000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 1.61000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 1.61000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 1.65000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 1.65000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 1.65000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 1.65000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 1.72000000000000E+0002; y: 9.20000000000000E+0001)), 
        ((x: 1.72000000000000E+0002; y: 9.20000000000000E+0001),
         (x: 1.72000000000000E+0002; y: 9.20000000000000E+0001)), 
        ((x: 1.72000000000000E+0002; y: 9.20000000000000E+0001),
         (x: 1.72000000000000E+0002; y: 9.20000000000000E+0001)), 
        ((x: 1.72000000000000E+0002; y: 9.20000000000000E+0001),
         (x: 1.74000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.74000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.74000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.74000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.78000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.78000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.78000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.78000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.82000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 1.82000000000000E+0002; y: 1.13000000000000E+0002),
         (x: 1.82000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.27000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.27000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.28000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 1.28000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 1.28000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 1.28000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 1.32000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.32000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.32000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.32000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.40000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.40000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.40000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.40000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.51000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.51000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.51000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.51000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.53000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.53000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.53000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.53000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.59000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.59000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.59000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.59000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.63000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.63000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.64000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.64000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.65000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.65000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.65000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.65000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.01000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.01000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 5.40000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 5.40000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 5.40000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 5.40000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 6.90000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 6.90000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 6.90000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 6.90000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 7.60000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 7.60000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.13000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.13000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.17000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.17000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.17000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.17000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.19000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.19000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.19000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 4.80000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.08000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.08000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.08000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.08000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.15000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.15000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.18000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.18000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.18000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.18000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.19000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.19000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.19000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.19000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 2.20000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 2.20000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 2.20000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 2.20000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 2.18000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.18000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.18000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.18000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.14000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.14000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.14000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.14000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.11000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.11000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.11000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.11000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.09000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.09000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.09000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.09000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.06000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.06000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.06000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.06000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.13000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.13000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.13000000000000E+0002; y: 8.60000000000000E+0001)), 
        ((x: 2.13000000000000E+0002; y: 8.60000000000000E+0001),
         (x: 2.19000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 2.19000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 2.19000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 2.19000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 2.22000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.22000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.22000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.22000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.26000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 2.26000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 2.26000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 2.26000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 2.26000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 2.26000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 2.26000000000000E+0002; y: 9.90000000000000E+0001)), 
        ((x: 2.26000000000000E+0002; y: 9.90000000000000E+0001),
         (x: 2.26000000000000E+0002; y: 9.90000000000000E+0001)), 
        ((x: 2.26000000000000E+0002; y: 9.90000000000000E+0001),
         (x: 2.23000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 2.23000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 2.23000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 2.23000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 2.19000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 2.19000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 2.19000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 2.19000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 2.17000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 2.17000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 2.17000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 2.17000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 2.17000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 2.17000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 2.14000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.14000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.14000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.14000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.11000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.11000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.09000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.09000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.09000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.69000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.69000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 2.69000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.66000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.66000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.64000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.64000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.64000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.64000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.63000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.63000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.63000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.63000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.60000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.60000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.60000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.60000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.57000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.57000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.53000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.53000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.53000000000000E+0002; y: 1.00000000000000E+0002)), 
        ((x: 2.53000000000000E+0002; y: 1.00000000000000E+0002),
         (x: 2.52000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 2.52000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 2.52000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 2.52000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 2.50000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 2.50000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 2.50000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 2.50000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 2.47000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 2.47000000000000E+0002; y: 1.15000000000000E+0002),
         (x: 2.47000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 2.68000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.76000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.76000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.76000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 2.76000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 2.81000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.81000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.81000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.81000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.88000000000000E+0002; y: 7.10000000000000E+0001)), 
        ((x: 2.88000000000000E+0002; y: 7.10000000000000E+0001),
         (x: 2.88000000000000E+0002; y: 7.10000000000000E+0001)), 
        ((x: 2.88000000000000E+0002; y: 7.10000000000000E+0001),
         (x: 2.93000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 2.93000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 2.93000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 2.93000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 2.98000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.98000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.98000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.98000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 3.01000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 3.01000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 3.01000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 3.01000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 3.01000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 3.01000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 3.01000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 2.62000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.62000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.62000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.62000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.62000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.68000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.68000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.78000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.78000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.78000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.78000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.87000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 2.87000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 2.87000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 2.87000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 8.00000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 8.00000000000000E+0001),
         (x: 2.98000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 5.60000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 5.60000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 5.60000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 5.60000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 5.60000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 6.20000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 6.20000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 6.20000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 6.20000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 6.80000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 6.80000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 6.80000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 6.80000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 7.70000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 7.70000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 7.70000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 7.70000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 7.70000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 7.70000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 8.30000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 8.30000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 8.30000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 8.30000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 1.50000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 1.50000000000000E+0001; y: 1.03000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.03000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.03000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.03000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.03000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.03000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.09000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.09000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.09000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.09000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.13000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.13000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.13000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.13000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.16000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.16000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.16000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.16000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 1.50000000000000E+0001; y: 1.18000000000000E+0002),
         (x: 1.50000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 2.10000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 2.10000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 2.10000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 2.10000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 2.10000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 1.90000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 1.90000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 1.80000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 1.80000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 1.80000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 1.80000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 2.30000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 2.30000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 2.30000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 2.30000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 3.60000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 3.60000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 3.60000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 3.60000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 4.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 4.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 4.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 4.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 4.60000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 4.60000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 4.90000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 4.90000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 5.00000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 5.00000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 5.00000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 5.00000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 5.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 8.10000000000000E+0001; y: 5.20000000000000E+0001),
         (x: 8.10000000000000E+0001; y: 5.20000000000000E+0001)), 
        ((x: 8.10000000000000E+0001; y: 5.20000000000000E+0001),
         (x: 8.10000000000000E+0001; y: 5.20000000000000E+0001)), 
        ((x: 8.10000000000000E+0001; y: 5.20000000000000E+0001),
         (x: 8.00000000000000E+0001; y: 6.10000000000000E+0001)), 
        ((x: 8.00000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 8.00000000000000E+0001; y: 6.10000000000000E+0001)), 
        ((x: 8.00000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 7.60000000000000E+0001; y: 7.20000000000000E+0001)), 
        ((x: 7.60000000000000E+0001; y: 7.20000000000000E+0001),
         (x: 7.60000000000000E+0001; y: 7.20000000000000E+0001)), 
        ((x: 7.60000000000000E+0001; y: 7.20000000000000E+0001),
         (x: 7.30000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 7.30000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 7.30000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 7.30000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 7.10000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 7.10000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 7.10000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 7.10000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 6.90000000000000E+0001; y: 9.70000000000000E+0001)), 
        ((x: 6.90000000000000E+0001; y: 9.70000000000000E+0001),
         (x: 6.90000000000000E+0001; y: 9.70000000000000E+0001)), 
        ((x: 6.90000000000000E+0001; y: 9.70000000000000E+0001),
         (x: 6.90000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 6.90000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 6.90000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 6.90000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 6.70000000000000E+0001; y: 1.08000000000000E+0002)), 
        ((x: 6.70000000000000E+0001; y: 1.08000000000000E+0002),
         (x: 6.70000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 6.70000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 6.70000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 6.70000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 6.70000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 6.70000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 6.70000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 8.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 6.10000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 6.10000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 6.10000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 7.90000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 7.90000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 7.90000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 7.90000000000000E+0001),
         (x: 8.40000000000000E+0001; y: 8.90000000000000E+0001)), 
        ((x: 8.40000000000000E+0001; y: 8.90000000000000E+0001),
         (x: 8.40000000000000E+0001; y: 8.90000000000000E+0001)), 
        ((x: 8.40000000000000E+0001; y: 8.90000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.50000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.50000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.50000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.50000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 9.90000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 9.90000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 8.60000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 8.60000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 8.90000000000000E+0001; y: 1.06000000000000E+0002)), 
        ((x: 8.90000000000000E+0001; y: 1.06000000000000E+0002),
         (x: 8.90000000000000E+0001; y: 1.06000000000000E+0002)), 
        ((x: 8.90000000000000E+0001; y: 1.06000000000000E+0002),
         (x: 9.30000000000000E+0001; y: 1.08000000000000E+0002)), 
        ((x: 9.30000000000000E+0001; y: 1.08000000000000E+0002),
         (x: 9.30000000000000E+0001; y: 1.08000000000000E+0002)), 
        ((x: 9.30000000000000E+0001; y: 1.08000000000000E+0002),
         (x: 9.70000000000000E+0001; y: 1.13000000000000E+0002)), 
        ((x: 9.70000000000000E+0001; y: 1.13000000000000E+0002),
         (x: 9.70000000000000E+0001; y: 1.13000000000000E+0002)), 
        ((x: 9.70000000000000E+0001; y: 1.13000000000000E+0002),
         (x: 9.90000000000000E+0001; y: 1.16000000000000E+0002)), 
        ((x: 9.90000000000000E+0001; y: 1.16000000000000E+0002),
         (x: 9.90000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 9.90000000000000E+0001; y: 1.18000000000000E+0002),
         (x: 9.90000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 9.90000000000000E+0001; y: 1.18000000000000E+0002),
         (x: 9.90000000000000E+0001; y: 1.21000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.53000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.53000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.53000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.53000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.53000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.56000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.56000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.56000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.56000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.56000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.56000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.65000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.65000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.65000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.65000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.72000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.72000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.72000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.72000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.78000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.78000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.78000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.78000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.84000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.84000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.84000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.84000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.87000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.87000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.87000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.87000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.93000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.93000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.93000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.93000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.97000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.97000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 1.97000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 1.97000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 2.08000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 2.08000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 2.08000000000000E+0002)), 
        ((x: 1.44000000000000E+0002; y: 2.08000000000000E+0002),
         (x: 1.44000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.55000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.55000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.55000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.55000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.55000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.55000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.55000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.58000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.58000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.58000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.58000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.63000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.63000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.63000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.63000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.69000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.69000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.69000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.69000000000000E+0002),
         (x: 1.72000000000000E+0002; y: 1.73000000000000E+0002)), 
        ((x: 1.72000000000000E+0002; y: 1.73000000000000E+0002),
         (x: 1.72000000000000E+0002; y: 1.73000000000000E+0002)), 
        ((x: 1.72000000000000E+0002; y: 1.73000000000000E+0002),
         (x: 1.72000000000000E+0002; y: 1.73000000000000E+0002)), 
        ((x: 1.72000000000000E+0002; y: 1.73000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 1.78000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 1.78000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 1.78000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 1.78000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.83000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.83000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.83000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.83000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.88000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.88000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.88000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.88000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.93000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.93000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 1.93000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 1.93000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 2.00000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 2.00000000000000E+0002),
         (x: 1.70000000000000E+0002; y: 2.00000000000000E+0002)), 
        ((x: 1.70000000000000E+0002; y: 2.00000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.05000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.05000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.05000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.05000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.08000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.08000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.08000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.08000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.10000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.10000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.10000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.10000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 2.14000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.32000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.32000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.32000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.32000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.32000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.38000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.38000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.38000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.38000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.59000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.59000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.59000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.59000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.77000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.77000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.77000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.77000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.86000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.86000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.86000000000000E+0002; y: 1.44000000000000E+0002)), 
        ((x: 1.86000000000000E+0002; y: 1.44000000000000E+0002),
         (x: 1.89000000000000E+0002; y: 1.43000000000000E+0002)), 
        ((x: 1.89000000000000E+0002; y: 1.43000000000000E+0002),
         (x: 1.89000000000000E+0002; y: 1.43000000000000E+0002)), 
        ((x: 1.28000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.28000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.28000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.28000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.28000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.30000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.30000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.30000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.30000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.37000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.37000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.37000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.37000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.54000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.54000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.54000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.54000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.54000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.54000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.83000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.83000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.83000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.83000000000000E+0002; y: 2.16000000000000E+0002),
         (x: 1.90000000000000E+0002; y: 2.15000000000000E+0002)), 
        ((x: 1.90000000000000E+0002; y: 2.15000000000000E+0002),
         (x: 1.90000000000000E+0002; y: 2.15000000000000E+0002)), 
        ((x: 1.90000000000000E+0002; y: 2.15000000000000E+0002),
         (x: 1.95000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.95000000000000E+0002; y: 2.14000000000000E+0002),
         (x: 1.95000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.95000000000000E+0002; y: 2.14000000000000E+0002),
         (x: 1.98000000000000E+0002; y: 2.14000000000000E+0002)), 
        ((x: 1.98000000000000E+0002; y: 2.14000000000000E+0002),
         (x: 1.98000000000000E+0002; y: 2.14000000000000E+0002))
    );

    Glava3: Array[0..370] Of Array[0..1] Of TVec2D = (
        ((x: 1.44000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 1.44000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 1.44000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 1.43000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 1.43000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 1.42000000000000E+0002; y: 5.30000000000000E+0001)), 
        ((x: 1.42000000000000E+0002; y: 5.30000000000000E+0001),
         (x: 1.39000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 1.39000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 1.37000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 1.37000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 1.35000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.35000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.35000000000000E+0002; y: 6.50000000000000E+0001)), 
        ((x: 1.35000000000000E+0002; y: 6.50000000000000E+0001),
         (x: 1.33000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.33000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.33000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 1.33000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 1.31000000000000E+0002; y: 7.30000000000000E+0001)), 
        ((x: 1.31000000000000E+0002; y: 7.30000000000000E+0001),
         (x: 1.30000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 1.30000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 1.29000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 1.29000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 1.28000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.28000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.26000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.26000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.26000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.26000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.26000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.26000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.22000000000000E+0002; y: 1.01000000000000E+0002)), 
        ((x: 1.22000000000000E+0002; y: 1.01000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.20000000000000E+0002; y: 1.06000000000000E+0002)), 
        ((x: 1.20000000000000E+0002; y: 1.06000000000000E+0002),
         (x: 1.20000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 1.20000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 1.19000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.19000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.18000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.18000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 1.17000000000000E+0002; y: 1.16000000000000E+0002)), 
        ((x: 1.17000000000000E+0002; y: 1.16000000000000E+0002),
         (x: 1.17000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 1.17000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 1.17000000000000E+0002; y: 1.21000000000000E+0002)), 
        ((x: 1.47000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 1.47000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 1.47000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 1.48000000000000E+0002; y: 5.30000000000000E+0001)), 
        ((x: 1.48000000000000E+0002; y: 5.30000000000000E+0001),
         (x: 1.51000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 1.51000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 1.51000000000000E+0002; y: 5.60000000000000E+0001)), 
        ((x: 1.51000000000000E+0002; y: 5.60000000000000E+0001),
         (x: 1.55000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 1.55000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 1.61000000000000E+0002; y: 7.20000000000000E+0001)), 
        ((x: 1.61000000000000E+0002; y: 7.20000000000000E+0001),
         (x: 1.63000000000000E+0002; y: 7.70000000000000E+0001)), 
        ((x: 1.63000000000000E+0002; y: 7.70000000000000E+0001),
         (x: 1.66000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 1.66000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 1.67000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 1.67000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.30000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.30000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 9.90000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 9.90000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 1.68000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 1.68000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 1.68000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 1.68000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 1.68000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 1.68000000000000E+0002; y: 1.10000000000000E+0002)), 
        ((x: 1.68000000000000E+0002; y: 1.10000000000000E+0002),
         (x: 1.68000000000000E+0002; y: 1.12000000000000E+0002)), 
        ((x: 1.68000000000000E+0002; y: 1.12000000000000E+0002),
         (x: 1.68000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.30000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.30000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.30000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.30000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.30000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.36000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.36000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.43000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.43000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.49000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.49000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.54000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.54000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.56000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.56000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.58000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.58000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.59000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 1.59000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 1.62000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 1.62000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 1.64000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 1.64000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 1.64000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 1.64000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 1.66000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.66000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.68000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.68000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 1.70000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 1.90000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 1.90000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 1.90000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 2.00000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 2.00000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 2.10000000000000E+0001; y: 5.50000000000000E+0001)), 
        ((x: 2.10000000000000E+0001; y: 5.50000000000000E+0001),
         (x: 2.10000000000000E+0001; y: 5.70000000000000E+0001)), 
        ((x: 2.10000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 6.20000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 6.20000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 6.20000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 6.20000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 6.70000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 6.70000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 6.70000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 6.70000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 7.10000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 7.10000000000000E+0001),
         (x: 2.30000000000000E+0001; y: 7.50000000000000E+0001)), 
        ((x: 2.30000000000000E+0001; y: 7.50000000000000E+0001),
         (x: 2.30000000000000E+0001; y: 7.80000000000000E+0001)), 
        ((x: 2.30000000000000E+0001; y: 7.80000000000000E+0001),
         (x: 2.50000000000000E+0001; y: 8.30000000000000E+0001)), 
        ((x: 2.50000000000000E+0001; y: 8.30000000000000E+0001),
         (x: 2.60000000000000E+0001; y: 8.70000000000000E+0001)), 
        ((x: 2.60000000000000E+0001; y: 8.70000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 9.10000000000000E+0001)), 
        ((x: 2.70000000000000E+0001; y: 9.10000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 9.50000000000000E+0001)), 
        ((x: 2.70000000000000E+0001; y: 9.50000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 2.70000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 2.70000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 2.70000000000000E+0001; y: 1.02000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.02000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.05000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.05000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.08000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.08000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.11000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.11000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.14000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.14000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.17000000000000E+0002)), 
        ((x: 2.70000000000000E+0001; y: 1.17000000000000E+0002),
         (x: 2.70000000000000E+0001; y: 1.18000000000000E+0002)), 
        ((x: 2.00000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 2.00000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 2.00000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 2.20000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 2.20000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 2.90000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 2.90000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 3.50000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 3.50000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 3.50000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 3.50000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 4.30000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 4.30000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 4.90000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 4.90000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.20000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.20000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.40000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.40000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.50000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.50000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.70000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.70000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.80000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.80000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 5.90000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 5.90000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 6.00000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 6.00000000000000E+0001; y: 5.30000000000000E+0001),
         (x: 6.10000000000000E+0001; y: 5.30000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.40000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.40000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.40000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.40000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.40000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.40000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.40000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 6.50000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 6.50000000000000E+0001),
         (x: 8.60000000000000E+0001; y: 6.90000000000000E+0001)), 
        ((x: 8.60000000000000E+0001; y: 6.90000000000000E+0001),
         (x: 8.50000000000000E+0001; y: 7.20000000000000E+0001)), 
        ((x: 8.50000000000000E+0001; y: 7.20000000000000E+0001),
         (x: 8.40000000000000E+0001; y: 7.80000000000000E+0001)), 
        ((x: 8.40000000000000E+0001; y: 7.80000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 8.20000000000000E+0001; y: 8.40000000000000E+0001)), 
        ((x: 8.20000000000000E+0001; y: 8.40000000000000E+0001),
         (x: 8.00000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 8.00000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 8.00000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 8.00000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 7.90000000000000E+0001; y: 9.40000000000000E+0001)), 
        ((x: 7.90000000000000E+0001; y: 9.40000000000000E+0001),
         (x: 7.70000000000000E+0001; y: 9.80000000000000E+0001)), 
        ((x: 7.70000000000000E+0001; y: 9.80000000000000E+0001),
         (x: 7.60000000000000E+0001; y: 1.01000000000000E+0002)), 
        ((x: 7.60000000000000E+0001; y: 1.01000000000000E+0002),
         (x: 7.50000000000000E+0001; y: 1.05000000000000E+0002)), 
        ((x: 7.50000000000000E+0001; y: 1.05000000000000E+0002),
         (x: 7.40000000000000E+0001; y: 1.07000000000000E+0002)), 
        ((x: 7.40000000000000E+0001; y: 1.07000000000000E+0002),
         (x: 7.30000000000000E+0001; y: 1.10000000000000E+0002)), 
        ((x: 7.30000000000000E+0001; y: 1.10000000000000E+0002),
         (x: 7.10000000000000E+0001; y: 1.13000000000000E+0002)), 
        ((x: 7.10000000000000E+0001; y: 1.13000000000000E+0002),
         (x: 7.00000000000000E+0001; y: 1.15000000000000E+0002)), 
        ((x: 7.00000000000000E+0001; y: 1.15000000000000E+0002),
         (x: 7.00000000000000E+0001; y: 1.15000000000000E+0002)), 
        ((x: 8.80000000000000E+0001; y: 5.20000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.20000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.20000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.40000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.40000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.70000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.70000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 5.90000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 5.90000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 6.20000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 6.20000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 6.60000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 6.60000000000000E+0001),
         (x: 8.80000000000000E+0001; y: 7.10000000000000E+0001)), 
        ((x: 8.80000000000000E+0001; y: 7.10000000000000E+0001),
         (x: 8.90000000000000E+0001; y: 7.60000000000000E+0001)), 
        ((x: 8.90000000000000E+0001; y: 7.60000000000000E+0001),
         (x: 9.00000000000000E+0001; y: 8.00000000000000E+0001)), 
        ((x: 9.00000000000000E+0001; y: 8.00000000000000E+0001),
         (x: 9.10000000000000E+0001; y: 8.50000000000000E+0001)), 
        ((x: 9.10000000000000E+0001; y: 8.50000000000000E+0001),
         (x: 9.30000000000000E+0001; y: 9.00000000000000E+0001)), 
        ((x: 9.30000000000000E+0001; y: 9.00000000000000E+0001),
         (x: 9.60000000000000E+0001; y: 9.50000000000000E+0001)), 
        ((x: 9.60000000000000E+0001; y: 9.50000000000000E+0001),
         (x: 9.60000000000000E+0001; y: 9.50000000000000E+0001)), 
        ((x: 9.60000000000000E+0001; y: 9.50000000000000E+0001),
         (x: 9.90000000000000E+0001; y: 1.01000000000000E+0002)), 
        ((x: 9.90000000000000E+0001; y: 1.01000000000000E+0002),
         (x: 1.00000000000000E+0002; y: 1.03000000000000E+0002)), 
        ((x: 1.00000000000000E+0002; y: 1.03000000000000E+0002),
         (x: 1.00000000000000E+0002; y: 1.05000000000000E+0002)), 
        ((x: 1.00000000000000E+0002; y: 1.05000000000000E+0002),
         (x: 1.02000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 1.02000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 1.02000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 1.02000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 1.02000000000000E+0002; y: 1.10000000000000E+0002)), 
        ((x: 1.02000000000000E+0002; y: 1.10000000000000E+0002),
         (x: 1.02000000000000E+0002; y: 1.11000000000000E+0002)), 
        ((x: 1.02000000000000E+0002; y: 1.11000000000000E+0002),
         (x: 1.03000000000000E+0002; y: 1.14000000000000E+0002)), 
        ((x: 1.03000000000000E+0002; y: 1.14000000000000E+0002),
         (x: 1.04000000000000E+0002; y: 1.16000000000000E+0002)), 
        ((x: 1.04000000000000E+0002; y: 1.16000000000000E+0002),
         (x: 1.04000000000000E+0002; y: 1.17000000000000E+0002)), 
        ((x: 1.95000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 1.95000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 1.95000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 1.98000000000000E+0002; y: 5.10000000000000E+0001)), 
        ((x: 1.98000000000000E+0002; y: 5.10000000000000E+0001),
         (x: 2.02000000000000E+0002; y: 5.50000000000000E+0001)), 
        ((x: 2.02000000000000E+0002; y: 5.50000000000000E+0001),
         (x: 2.04000000000000E+0002; y: 5.90000000000000E+0001)), 
        ((x: 2.04000000000000E+0002; y: 5.90000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 6.30000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 6.30000000000000E+0001),
         (x: 2.09000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 2.09000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 2.09000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 2.09000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 8.20000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 8.20000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 9.60000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 9.60000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.08000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.08000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.13000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.24000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.24000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.24000000000000E+0002)), 
        ((x: 2.10000000000000E+0002; y: 1.24000000000000E+0002),
         (x: 2.10000000000000E+0002; y: 1.28000000000000E+0002)), 
        ((x: 2.07000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.07000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 2.07000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.10000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.10000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.15000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.15000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 5.00000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 5.00000000000000E+0001),
         (x: 2.32000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.32000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.32000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.32000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.38000000000000E+0002; y: 5.50000000000000E+0001)), 
        ((x: 2.38000000000000E+0002; y: 5.50000000000000E+0001),
         (x: 2.39000000000000E+0002; y: 5.70000000000000E+0001)), 
        ((x: 2.39000000000000E+0002; y: 5.70000000000000E+0001),
         (x: 2.41000000000000E+0002; y: 6.10000000000000E+0001)), 
        ((x: 2.41000000000000E+0002; y: 6.10000000000000E+0001),
         (x: 2.42000000000000E+0002; y: 6.60000000000000E+0001)), 
        ((x: 2.42000000000000E+0002; y: 6.60000000000000E+0001),
         (x: 2.42000000000000E+0002; y: 6.90000000000000E+0001)), 
        ((x: 2.42000000000000E+0002; y: 6.90000000000000E+0001),
         (x: 2.42000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.42000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.39000000000000E+0002; y: 7.90000000000000E+0001)), 
        ((x: 2.39000000000000E+0002; y: 7.90000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.37000000000000E+0002; y: 8.30000000000000E+0001)), 
        ((x: 2.37000000000000E+0002; y: 8.30000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.34000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.34000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.32000000000000E+0002; y: 8.50000000000000E+0001)), 
        ((x: 2.32000000000000E+0002; y: 8.50000000000000E+0001),
         (x: 2.30000000000000E+0002; y: 8.70000000000000E+0001)), 
        ((x: 2.30000000000000E+0002; y: 8.70000000000000E+0001),
         (x: 2.28000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.28000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.24000000000000E+0002; y: 8.90000000000000E+0001)), 
        ((x: 2.24000000000000E+0002; y: 8.90000000000000E+0001),
         (x: 2.21000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.21000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.20000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.20000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.23000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.23000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.28000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.28000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.33000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.33000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.36000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.36000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.36000000000000E+0002; y: 9.00000000000000E+0001)), 
        ((x: 2.36000000000000E+0002; y: 9.00000000000000E+0001),
         (x: 2.38000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.38000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.41000000000000E+0002; y: 9.50000000000000E+0001)), 
        ((x: 2.41000000000000E+0002; y: 9.50000000000000E+0001),
         (x: 2.44000000000000E+0002; y: 9.90000000000000E+0001)), 
        ((x: 2.44000000000000E+0002; y: 9.90000000000000E+0001),
         (x: 2.47000000000000E+0002; y: 1.04000000000000E+0002)), 
        ((x: 2.47000000000000E+0002; y: 1.04000000000000E+0002),
         (x: 2.48000000000000E+0002; y: 1.09000000000000E+0002)), 
        ((x: 2.48000000000000E+0002; y: 1.09000000000000E+0002),
         (x: 2.48000000000000E+0002; y: 1.13000000000000E+0002)), 
        ((x: 2.48000000000000E+0002; y: 1.13000000000000E+0002),
         (x: 2.46000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.46000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.44000000000000E+0002; y: 1.24000000000000E+0002)), 
        ((x: 2.44000000000000E+0002; y: 1.24000000000000E+0002),
         (x: 2.44000000000000E+0002; y: 1.24000000000000E+0002)), 
        ((x: 2.44000000000000E+0002; y: 1.24000000000000E+0002),
         (x: 2.40000000000000E+0002; y: 1.28000000000000E+0002)), 
        ((x: 2.40000000000000E+0002; y: 1.28000000000000E+0002),
         (x: 2.37000000000000E+0002; y: 1.31000000000000E+0002)), 
        ((x: 2.37000000000000E+0002; y: 1.31000000000000E+0002),
         (x: 2.37000000000000E+0002; y: 1.31000000000000E+0002)), 
        ((x: 2.37000000000000E+0002; y: 1.31000000000000E+0002),
         (x: 2.37000000000000E+0002; y: 1.31000000000000E+0002)), 
        ((x: 2.37000000000000E+0002; y: 1.31000000000000E+0002),
         (x: 2.32000000000000E+0002; y: 1.33000000000000E+0002)), 
        ((x: 2.32000000000000E+0002; y: 1.33000000000000E+0002),
         (x: 2.30000000000000E+0002; y: 1.34000000000000E+0002)), 
        ((x: 2.30000000000000E+0002; y: 1.34000000000000E+0002),
         (x: 2.26000000000000E+0002; y: 1.35000000000000E+0002)), 
        ((x: 2.26000000000000E+0002; y: 1.35000000000000E+0002),
         (x: 2.22000000000000E+0002; y: 1.35000000000000E+0002)), 
        ((x: 2.22000000000000E+0002; y: 1.35000000000000E+0002),
         (x: 2.20000000000000E+0002; y: 1.36000000000000E+0002)), 
        ((x: 2.20000000000000E+0002; y: 1.36000000000000E+0002),
         (x: 2.20000000000000E+0002; y: 1.36000000000000E+0002)), 
        ((x: 2.80000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.80000000000000E+0002; y: 4.70000000000000E+0001)), 
        ((x: 2.80000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.79000000000000E+0002; y: 4.90000000000000E+0001)), 
        ((x: 2.79000000000000E+0002; y: 4.90000000000000E+0001),
         (x: 2.77000000000000E+0002; y: 5.30000000000000E+0001)), 
        ((x: 2.77000000000000E+0002; y: 5.30000000000000E+0001),
         (x: 2.75000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.75000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.74000000000000E+0002; y: 6.30000000000000E+0001)), 
        ((x: 2.74000000000000E+0002; y: 6.30000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 7.00000000000000E+0001)), 
        ((x: 2.72000000000000E+0002; y: 7.00000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 7.80000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 7.80000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 2.71000000000000E+0002; y: 8.40000000000000E+0001)), 
        ((x: 2.71000000000000E+0002; y: 8.40000000000000E+0001),
         (x: 2.70000000000000E+0002; y: 9.10000000000000E+0001)), 
        ((x: 2.70000000000000E+0002; y: 9.10000000000000E+0001),
         (x: 2.70000000000000E+0002; y: 9.70000000000000E+0001)), 
        ((x: 2.70000000000000E+0002; y: 9.70000000000000E+0001),
         (x: 2.70000000000000E+0002; y: 1.04000000000000E+0002)), 
        ((x: 2.70000000000000E+0002; y: 1.04000000000000E+0002),
         (x: 2.70000000000000E+0002; y: 1.10000000000000E+0002)), 
        ((x: 2.70000000000000E+0002; y: 1.10000000000000E+0002),
         (x: 2.70000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 2.70000000000000E+0002; y: 1.15000000000000E+0002),
         (x: 2.70000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.70000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.70000000000000E+0002; y: 1.20000000000000E+0002)), 
        ((x: 2.70000000000000E+0002; y: 1.20000000000000E+0002),
         (x: 2.69000000000000E+0002; y: 1.22000000000000E+0002)), 
        ((x: 2.82000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.84000000000000E+0002; y: 4.70000000000000E+0001)), 
        ((x: 2.84000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.85000000000000E+0002; y: 4.70000000000000E+0001)), 
        ((x: 2.85000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.85000000000000E+0002; y: 4.70000000000000E+0001)), 
        ((x: 2.85000000000000E+0002; y: 4.70000000000000E+0001),
         (x: 2.87000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.87000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.87000000000000E+0002; y: 5.20000000000000E+0001)), 
        ((x: 2.87000000000000E+0002; y: 5.20000000000000E+0001),
         (x: 2.90000000000000E+0002; y: 5.80000000000000E+0001)), 
        ((x: 2.90000000000000E+0002; y: 5.80000000000000E+0001),
         (x: 2.92000000000000E+0002; y: 6.20000000000000E+0001)), 
        ((x: 2.92000000000000E+0002; y: 6.20000000000000E+0001),
         (x: 2.93000000000000E+0002; y: 6.70000000000000E+0001)), 
        ((x: 2.93000000000000E+0002; y: 6.70000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 7.40000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 7.40000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 8.10000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 8.10000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 8.80000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 8.80000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 9.40000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 9.40000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 2.94000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 2.94000000000000E+0002; y: 1.02000000000000E+0002)), 
        ((x: 2.94000000000000E+0002; y: 1.02000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.07000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.07000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.12000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.12000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.15000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.15000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.18000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.18000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.21000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.21000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.22000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.22000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.23000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.23000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.23000000000000E+0002)), 
        ((x: 2.95000000000000E+0002; y: 1.23000000000000E+0002),
         (x: 2.95000000000000E+0002; y: 1.26000000000000E+0002)), 
        ((x: 2.72000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.72000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.72000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.73000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.73000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.77000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.77000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.82000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.82000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.82000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.82000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.87000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.87000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.89000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.89000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.90000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.90000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.91000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.91000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.92000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.92000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.93000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.93000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.94000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 2.94000000000000E+0002; y: 9.80000000000000E+0001),
         (x: 2.95000000000000E+0002; y: 9.80000000000000E+0001)), 
        ((x: 1.21000000000000E+0002; y: 1.52000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.52000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.52000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.53000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.53000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.55000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.55000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.59000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.59000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.64000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.64000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.69000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.69000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.74000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.74000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.77000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.77000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.77000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.77000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.83000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.83000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.87000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.87000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.91000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.91000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.96000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.96000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.00000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.00000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.04000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.04000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.06000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.06000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.09000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.09000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.10000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.10000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.12000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.12000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.13000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.13000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.15000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.15000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.16000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.54000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.54000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.54000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.60000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.60000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.64000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.64000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.70000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.70000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.70000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.70000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.77000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.77000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.77000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.77000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.82000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.82000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.85000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.85000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.89000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.89000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.94000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.94000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 1.98000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 1.98000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 2.03000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 2.03000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 2.05000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 2.05000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 2.07000000000000E+0002)), 
        ((x: 1.57000000000000E+0002; y: 2.07000000000000E+0002),
         (x: 1.57000000000000E+0002; y: 2.08000000000000E+0002)), 
        ((x: 1.86000000000000E+0002; y: 1.57000000000000E+0002),
         (x: 1.86000000000000E+0002; y: 1.57000000000000E+0002)), 
        ((x: 1.86000000000000E+0002; y: 1.57000000000000E+0002),
         (x: 1.86000000000000E+0002; y: 1.59000000000000E+0002)), 
        ((x: 1.86000000000000E+0002; y: 1.59000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.66000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.66000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.71000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.71000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.76000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.76000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.80000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.80000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.82000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.82000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.85000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.85000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.87000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.87000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.90000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.90000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.92000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.92000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.95000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.95000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 1.98000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 1.98000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.00000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.00000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.01000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.01000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.05000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.05000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.09000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.09000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.12000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.12000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.15000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.15000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.17000000000000E+0002)), 
        ((x: 1.87000000000000E+0002; y: 2.17000000000000E+0002),
         (x: 1.87000000000000E+0002; y: 2.18000000000000E+0002)), 
        ((x: 1.10000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.10000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.10000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.11000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.11000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.14000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.14000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.27000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.27000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.32000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.32000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.38000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.38000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.43000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.43000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.49000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.49000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.49000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.49000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.58000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.58000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.65000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.65000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.71000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.71000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.74000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.74000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.78000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.78000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.82000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.82000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.84000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.84000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.84000000000000E+0002; y: 1.48000000000000E+0002)), 
        ((x: 1.84000000000000E+0002; y: 1.48000000000000E+0002),
         (x: 1.90000000000000E+0002; y: 1.49000000000000E+0002)), 
        ((x: 1.90000000000000E+0002; y: 1.49000000000000E+0002),
         (x: 1.97000000000000E+0002; y: 1.51000000000000E+0002)), 
        ((x: 1.97000000000000E+0002; y: 1.51000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.51000000000000E+0002)), 
        ((x: 2.02000000000000E+0002; y: 1.51000000000000E+0002),
         (x: 2.02000000000000E+0002; y: 1.51000000000000E+0002)), 
        ((x: 1.16000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.16000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.16000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.21000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.21000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.36000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.36000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.36000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.36000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.51000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.51000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.58000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.58000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.61000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.61000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.67000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.67000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.72000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.72000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.77000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.77000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.79000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.79000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.83000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.83000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.83000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.83000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.89000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.89000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.92000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.92000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 1.96000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 1.96000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 2.01000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 2.01000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 2.08000000000000E+0002; y: 2.21000000000000E+0002)), 
        ((x: 2.08000000000000E+0002; y: 2.21000000000000E+0002),
         (x: 2.12000000000000E+0002; y: 2.21000000000000E+0002))
    );



//
//  TRenderLines
//
Type
TRenderLines = class(TGameDecorator)
protected
  FLines: Pointer;
  FCount: Integer;
  FStart: Integer;
  FTime: Integer;
  FAlpha1: Single;
  FAlpha2: Single;
public
  Constructor Create(Owner: TEntity); override;
  Procedure AssignVar(Name, Value: String); override;
  Procedure Render; override;
end;

Constructor TRenderLines.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FAlpha1 := 1;
  FAlpha2 := 0;
  FTime := 3000;
  FStart := GetTimer;
end;

Procedure TRenderLines.AssignVar(Name, Value: String);
begin
  Inherited AssignVar(Name, Value);
    If Name = 'time' then begin
      FTime := StrToInt(Value);
      FStart := GetTimer;
    end else If Name = 'a1' then
      FAlpha1 := StrToFloat(Value)
    else If Name = 'a2' then
      FAlpha2 := StrToFloat(Value);
end;

Procedure TRenderLines.Render;
  Var
    I: Integer;
begin
  Inherited Render;
      If GetTimer > FStart + FTime then
        Exit;
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glColor4f(1, 1, 1, (FAlpha2-FAlpha1)*(GetTimer - FStart)/FTime + FAlpha1);
  //glColor4f(1, 1, 1, 1);
  glBegin(GL_LINES);
  //glVertex2f(0, 0);
  //glVertex2f(320, 240);
    For I := 0 to FCount - 1 do begin
      glVertex2f((TLines(FLines^)[I][0]).X, (TLines(FLines^)[I][0]).Y);
      glVertex2f((TLines(FLines^)[I][1]).X, (TLines(FLines^)[I][1]).Y);
    end;
  glEnd;
  glDisable(GL_BLEND);
end;

Type
TRenderGlava1 = class(TRenderLines)
  Constructor Create(Owner: TEntity); override;
end;

Constructor TRenderGlava1.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FLines := @Glava1[0][0];
  FCount := 429;
end;

Type
TRenderGlava2 = class(TRenderLines)
  Constructor Create(Owner: TEntity); override;
end;

Constructor TRenderGlava2.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FLines := @Glava2[0][0];
  FCount := 393;
end;

Type
TRenderGlava3 = class(TRenderLines)
  Constructor Create(Owner: TEntity); override;
end;

Constructor TRenderGlava3.Create(Owner: TEntity);
begin
  Inherited Create(Owner);
  FLines := @Glava3[0][0];
  FCount := 371;
end;

initialization
  Factory.RegistClass(TRenderRect);
  Factory.RegistClass(TLabel);
  Factory.RegistClass(TRenderAppear);
  Factory.RegistClass(TRenderGlava1);
  Factory.RegistClass(TRenderGlava2);
  Factory.RegistClass(TRenderGlava3);
end.
